@@ -1,8 +1,8 @@
-#!/usr/local/bin/python2.3
+#!/usr/local/bin/python2.4
 
 """Print a text summary of the contents of a FileStorage."""
 
-from ZODB.fsdump import fsdump
+from ZODB.FileStorage.fsdump import fsdump
 
 if __name__ == "__main__":
     import sys
