@@ -14,12 +14,13 @@
 
 This class is mixed into the database manager in App.ApplicationManager.
 
-$Id: CacheManager.py 40222 2005-11-18 15:46:28Z andreasjung $'''
+$Id: CacheManager.py 40300 2005-11-21 16:54:03Z efge $'''
 __version__='$Revision: 1.31 $'[11:-2]
 
 import time
 
 import Globals
+from Globals import InitializeClass
 from DateTime import DateTime
 
 class CacheManager:
@@ -294,5 +295,4 @@ class CacheManager:
                }
         return res
 
-
-Globals.default__class_init__(CacheManager)
+InitializeClass(CacheManager)
