@@ -13,9 +13,29 @@
 
 
 # Implement the manage_addProduct method of object managers
+import types
 import Acquisition, sys, Products
+from Globals import InitializeClass
+from AccessControl import ClassSecurityInfo
 from AccessControl.PermissionMapping import aqwrap
 from AccessControl.Owned import UnownableOwner
+import Zope2
+
+def _product_packages():
+    """Returns all product packages including the regularly defined
+    zope2 packages and those without the Products namespace package.
+    """
+    
+    packages = {}
+    for x in dir(Products):
+        m = getattr(Products, x)
+        if isinstance(m, types.ModuleType):
+            packages[x] = m
+
+    for m in getattr(Products, '_registered_packages', []):
+        packages[m.__name__] = m
+    
+    return packages
 
 class ProductDispatcher(Acquisition.Implicit):
     " "
@@ -30,7 +50,7 @@ class ProductDispatcher(Acquisition.Implicit):
 
         # Try to get a custom dispatcher from a Python product
         dispatcher_class=getattr(
-            getattr(Products, name, None),
+            _product_packages().get(name, None),
             '__FactoryDispatcher__',
             FactoryDispatcher)
 
@@ -41,6 +61,8 @@ class FactoryDispatcher(Acquisition.Implicit):
     """Provide a namespace for product "methods"
     """
 
+    security = ClassSecurityInfo()
+
     _owner=UnownableOwner
 
     def __init__(self, product, dest, REQUEST=None):
@@ -55,13 +77,15 @@ class FactoryDispatcher(Acquisition.Implicit):
                 v=v[:v.rfind('/')]
                 self._u=v[:v.rfind('/')]
 
+    security.declarePublic('Destination')
     def Destination(self):
         "Return the destination for factory output"
         return self.__dict__['_d'] # we don't want to wrap the result!
+
+    security.declarePublic('this')
     this=Destination
-    this__roles__=Destination__roles__=None
-
 
+    security.declarePublic('DestinationURL')
     def DestinationURL(self):
         "Return the URL for the destination for factory output"
         url=getattr(self, '_u', None)
@@ -69,8 +93,6 @@ class FactoryDispatcher(Acquisition.Implicit):
             url=self.Destination().absolute_url()
         return url
 
-    DestinationURL__roles__=None
-
     def __getattr__(self, name):
         p=self.__dict__['_product']
         d=p.__dict__
@@ -102,3 +124,4 @@ class FactoryDispatcher(Acquisition.Implicit):
         d = update_menu and '/manage_main?update_menu=1' or '/manage_main'
         REQUEST['RESPONSE'].redirect(self.DestinationURL()+d)
 
+InitializeClass(FactoryDispatcher)
