@@ -4,44 +4,48 @@
 # All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
 # FOR A PARTICULAR PURPOSE
 #
 ##############################################################################
-import sys, os, time, random
-import os, sys
+import random
+from unittest import TestCase, TestSuite, TextTestRunner, makeSuite
 
 from BTrees.OOBTree import OOBTree, OOBucket, OOSet, OOTreeSet
 from BTrees.IOBTree import IOBTree, IOBucket, IOSet, IOTreeSet
 from BTrees.IIBTree import IIBTree, IIBucket, IISet, IITreeSet
+from BTrees.IFBTree import IFBTree, IFBucket, IFSet, IFTreeSet
 from BTrees.OIBTree import OIBTree, OIBucket, OISet, OITreeSet
 
+from BTrees.IIBTree import using64bits
 from BTrees.check import check
 
-from unittest import TestCase, TestSuite, TextTestRunner, makeSuite
-
-from glob import glob
-
-from ZODB.tests.StorageTestBase import removefs
+import transaction
 from ZODB import DB
 from ZODB.MappingStorage import MappingStorage
 
-class Base:
+class Base(TestCase):
     """ Tests common to all types: sets, buckets, and BTrees """
 
     db = None
 
     def tearDown(self):
-        self.t = None
-        del self.t
         if self.db is not None:
             self.db.close()
+        self.t = None
+        del self.t
 
     def _getRoot(self):
         if self.db is None:
+            # Unclear:  On the next line, the ZODB4 flavor of this routine
+            # [asses a cache_size argument:
+            #     self.db = DB(MappingStorage(), cache_size=1)
+            # If that's done here, though, testLoadAndStore() and
+            # testGhostUnghost() both nail the CPU and seemingly
+            # never finish.
             self.db = DB(MappingStorage())
         return self.db.open().root()
 
@@ -55,7 +59,7 @@ class Base:
             root = None
             root = self._getRoot()
             root[i] = t
-            get_transaction().commit()
+            transaction.commit()
 
             root2 = self._getRoot()
             if hasattr(t, 'items'):
@@ -72,11 +76,11 @@ class Base:
             self._populate(t, i)
             root = self._getRoot()
             root[i] = t
-            get_transaction().commit()
+            transaction.commit()
 
             root2 = self._getRoot()
             root2[i]._p_deactivate()
-            get_transaction().commit()
+            transaction.commit()
             if hasattr(t, 'items'):
                 self.assertEqual(list(root2[i].items()) , list(t.items()))
             else:
@@ -85,6 +89,42 @@ class Base:
             self._closeRoot(root)
             self._closeRoot(root2)
 
+    def testSimpleExclusiveKeyRange(self):
+        t = self.t.__class__()
+        self.assertEqual(list(t.keys()), [])
+        self.assertEqual(list(t.keys(excludemin=True)), [])
+        self.assertEqual(list(t.keys(excludemax=True)), [])
+        self.assertEqual(list(t.keys(excludemin=True, excludemax=True)), [])
+
+        self._populate(t, 1)
+        self.assertEqual(list(t.keys()), [0])
+        self.assertEqual(list(t.keys(excludemin=True)), [])
+        self.assertEqual(list(t.keys(excludemax=True)), [])
+        self.assertEqual(list(t.keys(excludemin=True, excludemax=True)), [])
+
+        t.clear()
+        self._populate(t, 2)
+        self.assertEqual(list(t.keys()), [0, 1])
+        self.assertEqual(list(t.keys(excludemin=True)), [1])
+        self.assertEqual(list(t.keys(excludemax=True)), [0])
+        self.assertEqual(list(t.keys(excludemin=True, excludemax=True)), [])
+
+        t.clear()
+        self._populate(t, 3)
+        self.assertEqual(list(t.keys()), [0, 1, 2])
+        self.assertEqual(list(t.keys(excludemin=True)), [1, 2])
+        self.assertEqual(list(t.keys(excludemax=True)), [0, 1])
+        self.assertEqual(list(t.keys(excludemin=True, excludemax=True)), [1])
+
+        self.assertEqual(list(t.keys(-1, 3, excludemin=True, excludemax=True)),
+                         [0, 1, 2])
+        self.assertEqual(list(t.keys(0, 3, excludemin=True, excludemax=True)),
+                         [1, 2])
+        self.assertEqual(list(t.keys(-1, 2, excludemin=True, excludemax=True)),
+                         [0, 1])
+        self.assertEqual(list(t.keys(0, 2, excludemin=True, excludemax=True)),
+                         [1])
+
 class MappingBase(Base):
     """ Tests common to mappings (buckets, btrees) """
 
@@ -92,6 +132,19 @@ class MappingBase(Base):
         # Make some data
         for i in range(l): t[i]=i
 
+    def testRepr(self):
+        # test the repr because buckets have a complex repr implementation
+        # internally the cutoff from a stack allocated buffer to a heap
+        # allocated buffer is 10000.
+        for i in range(1000):
+            self.t[i] = i
+        r = repr(self.t)
+        # Make sure the repr is 10000 bytes long for a bucket.
+        # But since the test is also run for btrees, skip the length
+        # check if the repr starts with '<'
+        if not r.startswith('<'):
+            self.assert_(len(r) > 10000)
+
     def testGetItemFails(self):
         self.assertRaises(KeyError, self._getitemfail)
 
@@ -126,16 +179,23 @@ class MappingBase(Base):
     def testHasKeyWorks(self):
         self.t[1] = 1
         self.assert_(self.t.has_key(1))
+        self.assert_(1 in self.t)
+        self.assert_(0 not in self.t)
+        self.assert_(2 not in self.t)
 
     def testValuesWorks(self):
         for x in range(100):
             self.t[x] = x*x
         v = self.t.values()
         for i in range(100):
-            self.assertEqual(v[i],i*i , (i*i,i))
+            self.assertEqual(v[i], i*i)
+        self.assertRaises(IndexError, lambda: v[i+1])
+        i = 0
+        for value in self.t.itervalues():
+            self.assertEqual(value, i*i)
+            i += 1
 
     def testValuesWorks1(self):
-
         for x in range(100):
             self.t[99-x] = x
 
@@ -144,6 +204,19 @@ class MappingBase(Base):
             lst.sort()
             self.assertEqual(lst,range(0+x,99-x+1))
 
+            lst = list(self.t.values(max=99-x, min=0+x))
+            lst.sort()
+            self.assertEqual(lst,range(0+x,99-x+1))
+
+    def testValuesNegativeIndex(self):
+        L = [-3, 6, -11, 4]
+        for i in L:
+            self.t[i] = i
+        L.sort()
+        vals = self.t.values()
+        for i in range(-1, -5, -1):
+            self.assertEqual(vals[i], L[i])
+        self.assertRaises(IndexError, lambda: vals[-5])
 
     def testKeysWorks(self):
         for x in range(100):
@@ -153,24 +226,58 @@ class MappingBase(Base):
         for x in v:
             self.assertEqual(x,i)
             i = i + 1
+        self.assertRaises(IndexError, lambda: v[i])
 
         for x in range(40):
             lst = self.t.keys(0+x,99-x)
-            self.assertEqual(list(lst),range(0+x,99-x+1))
+            self.assertEqual(list(lst), range(0+x, 99-x+1))
 
-        # BTree items must lie about their lengths, so we convert to list
-        self.assertEqual(len(v) , 100, len(v))
-        #self.assertEqual(len(v) , 100, len(v))
+            lst = self.t.keys(max=99-x, min=0+x)
+            self.assertEqual(list(lst), range(0+x, 99-x+1))
+
+        self.assertEqual(len(v), 100)
+
+    def testKeysNegativeIndex(self):
+        L = [-3, 6, -11, 4]
+        for i in L:
+            self.t[i] = i
+        L.sort()
+        keys = self.t.keys()
+        for i in range(-1, -5, -1):
+            self.assertEqual(keys[i], L[i])
+        self.assertRaises(IndexError, lambda: keys[-5])
 
     def testItemsWorks(self):
         for x in range(100):
-            self.t[x] = x
+            self.t[x] = 2*x
         v = self.t.items()
         i = 0
         for x in v:
-            self.assertEqual(x[0] , i, (x[0], i))
-            self.assertEqual(x[1] , i, (x[0], i))
-            i = i + 1
+            self.assertEqual(x[0], i)
+            self.assertEqual(x[1], 2*i)
+            i += 1
+        self.assertRaises(IndexError, lambda: v[i+1])
+
+        i = 0
+        for x in self.t.iteritems():
+            self.assertEqual(x, (i, 2*i))
+            i += 1
+
+        items = list(self.t.items(min=12, max=20))
+        self.assertEqual(items, zip(range(12, 21), range(24, 43, 2)))
+
+        items = list(self.t.iteritems(min=12, max=20))
+        self.assertEqual(items, zip(range(12, 21), range(24, 43, 2)))
+
+    def testItemsNegativeIndex(self):
+        L = [-3, 6, -11, 4]
+        for i in L:
+            self.t[i] = i
+        L.sort()
+        items = self.t.items()
+        for i in range(-1, -5, -1):
+            self.assertEqual(items[i], (L[i], L[i]))
+        self.assertRaises(IndexError, lambda: items[-5])
 
     def testDeleteInvalidKeyRaisesKeyError(self):
         self.assertRaises(KeyError, self._deletefail)
@@ -189,12 +296,26 @@ class MappingBase(Base):
         self.t[4] = 150
         del self.t[7]
         t = self.t
-        self.assertEqual(t.maxKey() , 10)
-        self.assertEqual(t.maxKey(6) , 6)
-        self.assertEqual(t.maxKey(9) , 8)
-        self.assertEqual(t.minKey() , 1)
-        self.assertEqual(t.minKey(3) , 3)
-        self.assertEqual(t.minKey(9) , 10)
+        self.assertEqual(t.maxKey(), 10)
+        self.assertEqual(t.maxKey(6), 6)
+        self.assertEqual(t.maxKey(9), 8)
+        self.assertEqual(t.minKey(), 1)
+        self.assertEqual(t.minKey(3), 3)
+        self.assertEqual(t.minKey(9), 10)
+
+        try:
+            t.maxKey(t.minKey() - 1)
+        except ValueError, err:
+            self.assertEqual(str(err), "no key satisfies the conditions")
+        else:
+            self.fail("expected ValueError")
+
+        try:
+            t.minKey(t.maxKey() + 1)
+        except ValueError, err:
+            self.assertEqual(str(err), "no key satisfies the conditions")
+        else:
+            self.fail("expected ValueError")
 
     def testClear(self):
         r = range(100)
@@ -203,7 +324,7 @@ class MappingBase(Base):
             self.t[rnd] = 0
         self.t.clear()
         diff = lsubtract(list(self.t.keys()), [])
-        self.assertEqual(diff , [], diff)
+        self.assertEqual(diff, [])
 
     def testUpdate(self):
         d={}
@@ -217,13 +338,25 @@ class MappingBase(Base):
         items.sort()
 
         self.t.update(d)
-        self.assertEqual(list(self.t.items()) , items)
+        self.assertEqual(list(self.t.items()), items)
 
         self.t.clear()
-        self.assertEqual(list(self.t.items()) , [])
+        self.assertEqual(list(self.t.items()), [])
 
         self.t.update(l)
-        self.assertEqual(list(self.t.items()) , items)
+        self.assertEqual(list(self.t.items()), items)
+
+    # Before ZODB 3.4.2, update/construction from PersistentMapping failed.
+    def testUpdateFromPersistentMapping(self):
+        from persistent.mapping import PersistentMapping
+
+        pm = PersistentMapping({1: 2})
+        self.t.update(pm)
+        self.assertEqual(list(self.t.items()), [(1, 2)])
+
+        # Construction goes thru the same internals as .update().
+        t = self.t.__class__(pm)
+        self.assertEqual(list(t.items()), [(1, 2)])
 
     def testEmptyRangeSearches(self):
         t = self.t
@@ -241,6 +374,12 @@ class MappingBase(Base):
         keys = t.keys(200, 50)
         self.assertEqual(len(keys), 0)
         self.assertEqual(list(keys), [])
+        self.assertEqual(list(t.iterkeys(200, 50)), [])
+
+        keys = t.keys(max=50, min=200)
+        self.assertEqual(len(keys), 0)
+        self.assertEqual(list(keys), [])
+        self.assertEqual(list(t.iterkeys(max=50, min=200)), [])
 
     def testSlicing(self):
         # Test that slicing of .keys()/.values()/.items() works exactly the
@@ -318,6 +457,84 @@ class MappingBase(Base):
         self.assertEqual(len(tslice), 60)
         self.assertEqual(list(tslice), zip(range(20, 80), [1]*60))
 
+    def testIterators(self):
+        t = self.t
+
+        for keys in [], [-2], [1, 4], range(-170, 2000, 6):
+            t.clear()
+            for k in keys:
+                t[k] = -3 * k
+
+            self.assertEqual(list(t), keys)
+
+            x = []
+            for k in t:
+                x.append(k)
+            self.assertEqual(x, keys)
+
+            it = iter(t)
+            self.assert_(it is iter(it))
+            x = []
+            try:
+                while 1:
+                    x.append(it.next())
+            except StopIteration:
+                pass
+            self.assertEqual(x, keys)
+
+            self.assertEqual(list(t.iterkeys()), keys)
+            self.assertEqual(list(t.itervalues()), list(t.values()))
+            self.assertEqual(list(t.iteritems()), list(t.items()))
+
+    def testRangedIterators(self):
+        t = self.t
+
+        for keys in [], [-2], [1, 4], range(-170, 2000, 13):
+            t.clear()
+            values = []
+            for k in keys:
+                value = -3 * k
+                t[k] = value
+                values.append(value)
+            items = zip(keys, values)
+
+            self.assertEqual(list(t.iterkeys()), keys)
+            self.assertEqual(list(t.itervalues()), values)
+            self.assertEqual(list(t.iteritems()), items)
+
+            if not keys:
+                continue
+
+            min_mid_max = (keys[0], keys[len(keys) >> 1], keys[-1])
+            for key1 in min_mid_max:
+                for lo in range(key1 - 1, key1 + 2):
+                    # Test one-sided range iterators.
+                    goodkeys = [k for k in keys if lo <= k]
+                    got = t.iterkeys(lo)
+                    self.assertEqual(goodkeys, list(got))
+
+                    goodvalues = [t[k] for k in goodkeys]
+                    got = t.itervalues(lo)
+                    self.assertEqual(goodvalues, list(got))
+
+                    gooditems = zip(goodkeys, goodvalues)
+                    got = t.iteritems(lo)
+                    self.assertEqual(gooditems, list(got))
+
+                    for key2 in min_mid_max:
+                        for hi in range(key2 - 1, key2 + 2):
+                            goodkeys = [k for k in keys if lo <= k <= hi]
+                            got = t.iterkeys(min=lo, max=hi)
+                            self.assertEqual(goodkeys, list(got))
+
+                            goodvalues = [t[k] for k in goodkeys]
+                            got = t.itervalues(lo, max=hi)
+                            self.assertEqual(goodvalues, list(got))
+
+                            gooditems = zip(goodkeys, goodvalues)
+                            got = t.iteritems(max=hi, min=lo)
+                            self.assertEqual(gooditems, list(got))
+
     def testBadUpdateTupleSize(self):
         # This one silently ignored the excess in Zope3.
         try:
@@ -339,16 +556,124 @@ class MappingBase(Base):
         self.t.update([(1, 2)])
         self.assertEqual(list(self.t.items()), [(1, 2)])
 
+    def testSimpleExclusivRanges(self):
+        def identity(x):
+            return x
+        def dup(x):
+            return [(y, y) for y in x]
+
+        for methodname, f in (("keys", identity),
+                              ("values", identity),
+                              ("items", dup),
+                              ("iterkeys", identity),
+                              ("itervalues", identity),
+                              ("iteritems", dup)):
+
+            t = self.t.__class__()
+            meth = getattr(t, methodname, None)
+            if meth is None:
+                continue
+
+            self.assertEqual(list(meth()), [])
+            self.assertEqual(list(meth(excludemin=True)), [])
+            self.assertEqual(list(meth(excludemax=True)), [])
+            self.assertEqual(list(meth(excludemin=True, excludemax=True)), [])
+
+            self._populate(t, 1)
+            self.assertEqual(list(meth()), f([0]))
+            self.assertEqual(list(meth(excludemin=True)), [])
+            self.assertEqual(list(meth(excludemax=True)), [])
+            self.assertEqual(list(meth(excludemin=True, excludemax=True)), [])
+
+            t.clear()
+            self._populate(t, 2)
+            self.assertEqual(list(meth()), f([0, 1]))
+            self.assertEqual(list(meth(excludemin=True)), f([1]))
+            self.assertEqual(list(meth(excludemax=True)), f([0]))
+            self.assertEqual(list(meth(excludemin=True, excludemax=True)), [])
+
+            t.clear()
+            self._populate(t, 3)
+            self.assertEqual(list(meth()), f([0, 1, 2]))
+            self.assertEqual(list(meth(excludemin=True)), f([1, 2]))
+            self.assertEqual(list(meth(excludemax=True)), f([0, 1]))
+            self.assertEqual(list(meth(excludemin=True, excludemax=True)),
+                            f([1]))
+            self.assertEqual(list(meth(-1, 3, excludemin=True,
+                                       excludemax=True)),
+                             f([0, 1, 2]))
+            self.assertEqual(list(meth(0, 3, excludemin=True,
+                                       excludemax=True)),
+                             f([1, 2]))
+            self.assertEqual(list(meth(-1, 2, excludemin=True,
+                                       excludemax=True)),
+                             f([0, 1]))
+            self.assertEqual(list(meth(0, 2, excludemin=True,
+                                       excludemax=True)),
+                             f([1]))
+
+    def testSetdefault(self):
+        t = self.t
+
+        self.assertEqual(t.setdefault(1, 2), 2)
+        # That should also have associated 1 with 2 in the tree.
+        self.assert_(1 in t)
+        self.assertEqual(t[1], 2)
+        # And trying to change it again should have no effect.
+        self.assertEqual(t.setdefault(1, 666), 2)
+        self.assertEqual(t[1], 2)
+
+        # Not enough arguments.
+        self.assertRaises(TypeError, t.setdefault)
+        self.assertRaises(TypeError, t.setdefault, 1)
+        # Too many arguments.
+        self.assertRaises(TypeError, t.setdefault, 1, 2, 3)
+
+
+    def testPop(self):
+        t = self.t
+
+        # Empty container.
+        # If no default given, raises KeyError.
+        self.assertRaises(KeyError, t.pop, 1)
+        # But if default given, returns that instead.
+        self.assertEqual(t.pop(1, 42), 42)
+
+        t[1] = 3
+        # KeyError when key is not in container and default is not passed.
+        self.assertRaises(KeyError, t.pop, 5)
+        self.assertEqual(list(t.items()), [(1, 3)])
+        # If key is in container, returns the value and deletes the key.
+        self.assertEqual(t.pop(1), 3)
+        self.assertEqual(len(t), 0)
+
+        # If key is present, return value bypassing default.
+        t[1] = 3
+        self.assertEqual(t.pop(1, 7), 3)
+        self.assertEqual(len(t), 0)
+
+        # Pop only one item.
+        t[1] = 3
+        t[2] = 4
+        self.assertEqual(len(t), 2)
+        self.assertEqual(t.pop(1), 3)
+        self.assertEqual(len(t), 1)
+        self.assertEqual(t[2], 4)
+        self.assertEqual(t.pop(1, 3), 3)
+
+        # Too few arguments.
+        self.assertRaises(TypeError, t.pop)
+        # Too many arguments.
+        self.assertRaises(TypeError, t.pop, 1, 2, 3)
+
 
 class NormalSetTests(Base):
     """ Test common to all set types """
 
-
     def _populate(self, t, l):
         # Make some data
         t.update(range(l))
 
-
     def testInsertReturnsValue(self):
         t = self.t
         self.assertEqual(t.insert(5) , 1)
@@ -362,6 +687,8 @@ class NormalSetTests(Base):
         t = self.t
         t.insert(1)
         self.assert_(t.has_key(1))
+        self.assert_(1 in t)
+        self.assert_(2 not in t)
 
     def testBigInsert(self):
         t = self.t
@@ -370,6 +697,7 @@ class NormalSetTests(Base):
             t.insert(x)
         for x in r:
             self.assert_(t.has_key(x))
+            self.assert_(x in t)
 
     def testRemoveSucceeds(self):
         t = self.t
@@ -386,13 +714,16 @@ class NormalSetTests(Base):
     def testHasKeyFails(self):
         t = self.t
         self.assert_(not t.has_key(1))
+        self.assert_(1 not in t)
 
     def testKeys(self):
         t = self.t
         r = xrange(1000)
-        for x in r: t.insert(x)
+        for x in r:
+            t.insert(x)
         diff = lsubtract(t.keys(), r)
-        self.assertEqual(diff , [], diff)
+        self.assertEqual(diff, [])
+
 
     def testClear(self):
         t = self.t
@@ -418,6 +749,24 @@ class NormalSetTests(Base):
         self.assertEqual(t.minKey() , 1)
         self.assertEqual(t.minKey(3) , 3)
         self.assertEqual(t.minKey(9) , 10)
+        self.assert_(t.minKey() in t)
+        self.assert_(t.minKey()-1 not in t)
+        self.assert_(t.maxKey() in t)
+        self.assert_(t.maxKey()+1 not in t)
+
+        try:
+            t.maxKey(t.minKey() - 1)
+        except ValueError, err:
+            self.assertEqual(str(err), "no key satisfies the conditions")
+        else:
+            self.fail("expected ValueError")
+
+        try:
+            t.minKey(t.maxKey() + 1)
+        except ValueError, err:
+            self.assertEqual(str(err), "no key satisfies the conditions")
+        else:
+            self.fail("expected ValueError")
 
     def testUpdate(self):
         d={}
@@ -427,11 +776,11 @@ class NormalSetTests(Base):
             d[k]=i
             l.append(k)
 
-        items=d.keys()
+        items = d.keys()
         items.sort()
 
         self.t.update(l)
-        self.assertEqual(list(self.t.keys()) , items)
+        self.assertEqual(list(self.t.keys()), items)
 
     def testEmptyRangeSearches(self):
         t = self.t
@@ -450,6 +799,10 @@ class NormalSetTests(Base):
         self.assertEqual(len(keys), 0)
         self.assertEqual(list(keys), [])
 
+        keys = t.keys(max=50, min=200)
+        self.assertEqual(len(keys), 0)
+        self.assertEqual(list(keys), [])
+
     def testSlicing(self):
         # Test that slicing of .keys() works exactly the same way as slicing
         # a Python list with the same contents.
@@ -482,6 +835,29 @@ class NormalSetTests(Base):
                     x = kslice[lo:hi]
                     self.assertEqual(list(x), keys[lo:hi])
 
+    def testIterator(self):
+        t = self.t
+
+        for keys in [], [-2], [1, 4], range(-170, 2000, 6):
+            t.clear()
+            t.update(keys)
+
+            self.assertEqual(list(t), keys)
+
+            x = []
+            for k in t:
+                x.append(k)
+            self.assertEqual(x, keys)
+
+            it = iter(t)
+            self.assert_(it is iter(it))
+            x = []
+            try:
+                while 1:
+                    x.append(it.next())
+            except StopIteration:
+                pass
+            self.assertEqual(x, keys)
 
 class ExtendedSetTests(NormalSetTests):
     def testLen(self):
@@ -497,10 +873,6 @@ class ExtendedSetTests(NormalSetTests):
         for x in r:
             self.assertEqual(t[x] , x)
 
-class BucketTests(MappingBase):
-    """ Tests common to all buckets """
-    pass
-
 class BTreeTests(MappingBase):
     """ Tests common to all BTrees """
 
@@ -609,6 +981,7 @@ class BTreeTests(MappingBase):
         for x in r:
             k = random.choice(r)
             if self.t.has_key(k):
+                self.assert_(k in self.t)
                 del self.t[k]
                 deleted.append(k)
                 if self.t.has_key(k):
@@ -738,7 +1111,7 @@ class BTreeTests(MappingBase):
         # to "go backwards" in the BTree then; if it doesn't, it will
         # erroneously claim that the range is empty.
         del t[firstkey]
-        therange = t.keys(-1, firstkey)
+        therange = t.keys(min=-1, max=firstkey)
         self.assertEqual(len(therange), firstkey)
         self.assertEqual(list(therange), range(firstkey))
 
@@ -773,17 +1146,126 @@ class BTreeTests(MappingBase):
                                               "changed size")
                 break
 
-## BTree tests
 
-class TestIOBTrees(BTreeTests, TestCase):
-    def setUp(self):
-        self.t = IOBTree()
+LARGEST_32_BITS = 2147483647
+SMALLEST_32_BITS = -LARGEST_32_BITS - 1
 
-    def nonIntegerKeyRaises(self):
+SMALLEST_POSITIVE_33_BITS = LARGEST_32_BITS + 1
+LARGEST_NEGATIVE_33_BITS = SMALLEST_32_BITS - 1
+
+LARGEST_64_BITS = 0x7fffffffffffffff
+SMALLEST_64_BITS = -LARGEST_64_BITS - 1
+
+SMALLEST_POSITIVE_65_BITS = LARGEST_64_BITS + 1
+LARGEST_NEGATIVE_65_BITS = SMALLEST_64_BITS - 1
+
+class TestLongIntSupport:
+
+    def getTwoValues(self):
+        """Return two distinct values; these must compare as un-equal.
+
+        These values must be usable as values.
+
+        """
+        return object(), object()
+
+    def getTwoKeys(self):
+        """Return two distinct values, these must compare as un-equal.
+
+        These values must be usable as keys.
+
+        """
+        return 0, 1
+
+    def _set_value(self, key, value):
+        self.t[key] = value
+
+class TestLongIntKeys(TestLongIntSupport):
+
+    def testLongIntKeysWork(self):
+        o1, o2 = self.getTwoValues()
+        assert o1 != o2
+
+        # Test some small key values first:
+        self.t[0L] = o1
+        self.assertEqual(self.t[0], o1)
+        self.t[0] = o2
+        self.assertEqual(self.t[0L], o2)
+        self.assertEqual(list(self.t.keys()), [0])
+
+        # Test some large key values too:
+        k1 = SMALLEST_POSITIVE_33_BITS
+        k2 = LARGEST_64_BITS
+        k3 = SMALLEST_64_BITS
+        self.t[k1] = o1
+        self.t[k2] = o2
+        self.t[k3] = o1
+        self.assertEqual(self.t[k1], o1)
+        self.assertEqual(self.t[k2], o2)
+        self.assertEqual(self.t[k3], o1)
+        self.assertEqual(list(self.t.keys()), [k3, 0, k1, k2])
+
+    def testLongIntKeysOutOfRange(self):
+        o1, o2 = self.getTwoValues()
+        self.assertRaises(
+            ValueError,
+            self._set_value, SMALLEST_POSITIVE_65_BITS, o1)
+        self.assertRaises(
+            ValueError,
+            self._set_value, LARGEST_NEGATIVE_65_BITS, o1)
+
+class TestLongIntValues(TestLongIntSupport):
+
+    def testLongIntValuesWork(self):
+        keys = list(self.getTwoKeys())
+        keys.sort()
+        k1, k2 = keys
+        assert k1 != k2
+
+        # This is the smallest positive integer that requires 33 bits:
+        v1 = SMALLEST_POSITIVE_33_BITS
+        v2 = v1 + 1
+
+        self.t[k1] = v1
+        self.t[k2] = v2
+        self.assertEqual(self.t[k1], v1)
+        self.assertEqual(self.t[k2], v2)
+        self.assertEqual(list(self.t.values()), [v1, v2])
+
+    def testLongIntValuesOutOfRange(self):
+        k1, k2 = self.getTwoKeys()
+        self.assertRaises(
+            ValueError,
+            self._set_value, k1, SMALLEST_POSITIVE_65_BITS)
+        self.assertRaises(
+            ValueError,
+            self._set_value, k1, LARGEST_NEGATIVE_65_BITS)
+
+
+if not using64bits:
+    # We're not using 64-bit ints in this build, so we don't expect
+    # the long-integer tests to pass.
+
+    class TestLongIntKeys:
+        pass
+
+    class TestLongIntValues:
+        pass
+
+
+# tests of various type errors
+
+class TypeTest(TestCase):
+
+    def testBadTypeRaises(self):
         self.assertRaises(TypeError, self._stringraises)
         self.assertRaises(TypeError, self._floatraises)
         self.assertRaises(TypeError, self._noneraises)
 
+class TestIOBTrees(TypeTest):
+    def setUp(self):
+        self.t = IOBTree()
+
     def _stringraises(self):
         self.t['c'] = 1
 
@@ -793,6 +1275,19 @@ class TestIOBTrees(BTreeTests, TestCase):
     def _noneraises(self):
         self.t[None] = 1
 
+class TestOIBTrees(TypeTest):
+    def setUp(self):
+        self.t = OIBTree()
+
+    def _stringraises(self):
+        self.t[1] = 'c'
+
+    def _floatraises(self):
+        self.t[1] = 1.4
+
+    def _noneraises(self):
+        self.t[1] = None
+
     def testEmptyFirstBucketReportedByGuido(self):
         b = self.t
         for i in xrange(29972): # reduce to 29971 and it works
@@ -803,29 +1298,7 @@ class TestIOBTrees(BTreeTests, TestCase):
 
         self.assertEqual(b.keys()[0], 30)
 
-class TestOOBTrees(BTreeTests, TestCase):
-    def setUp(self):
-        self.t = OOBTree()
-
-class TestOIBTrees(BTreeTests, TestCase):
-    def setUp(self):
-        self.t = OIBTree()
-
-    def testNonIntegerValueRaises(self):
-        self.assertRaises(TypeError, self._stringraises)
-        self.assertRaises(TypeError, self._floatraises)
-        self.assertRaises(TypeError, self._noneraises)
-
-    def _stringraises(self):
-        self.t[1] = 'c'
-
-    def _floatraises(self):
-        self.t[1] = 1.4
-
-    def _noneraises(self):
-        self.t[1] = None
-
-class TestIIBTrees(BTreeTests, TestCase):
+class TestIIBTrees(TestCase):
     def setUp(self):
         self.t = IIBTree()
 
@@ -857,9 +1330,40 @@ class TestIIBTrees(BTreeTests, TestCase):
     def _noneraisesvalue(self):
         self.t[1] = None
 
-## Set tests
+class TestIFBTrees(TestCase):
+    def setUp(self):
+        self.t = IFBTree()
 
-class TestIOSets(ExtendedSetTests, TestCase):
+    def testNonIntegerKeyRaises(self):
+        self.assertRaises(TypeError, self._stringraiseskey)
+        self.assertRaises(TypeError, self._floatraiseskey)
+        self.assertRaises(TypeError, self._noneraiseskey)
+
+    def testNonNumericValueRaises(self):
+        self.assertRaises(TypeError, self._stringraisesvalue)
+        self.assertRaises(TypeError, self._noneraisesvalue)
+        self.t[1] = 1
+        self.t[1] = 1.0
+
+    def _stringraiseskey(self):
+        self.t['c'] = 1
+
+    def _floatraiseskey(self):
+        self.t[2.5] = 1
+
+    def _noneraiseskey(self):
+        self.t[None] = 1
+
+    def _stringraisesvalue(self):
+        self.t[1] = 'c'
+
+    def _floatraisesvalue(self):
+        self.t[1] = 1.4
+
+    def _noneraisesvalue(self):
+        self.t[1] = None
+
+class TestIOSets(TestCase):
     def setUp(self):
         self.t = IOSet()
 
@@ -877,30 +1381,7 @@ class TestIOSets(ExtendedSetTests, TestCase):
     def _insertnoneraises(self):
         self.t.insert(None)
 
-class TestOOSets(ExtendedSetTests, TestCase):
-    def setUp(self):
-        self.t = OOSet()
-
-class TestIISets(ExtendedSetTests, TestCase):
-    def setUp(self):
-        self.t = IISet()
-
-class TestOISets(ExtendedSetTests, TestCase):
-    def setUp(self):
-        self.t = OISet()
-
-class TestIOTreeSets(NormalSetTests, TestCase):
-    def setUp(self):
-        self.t = IOTreeSet()
-
-class TestOOTreeSets(NormalSetTests, TestCase):
-    def setUp(self):
-        self.t = OOTreeSet()
-
-class TestIITreeSets(NormalSetTests, TestCase):
-    def setUp(self):
-        self.t = IITreeSet()
-
+class DegenerateBTree(TestCase):
     # Build a degenerate tree (set).  Boxes are BTree nodes.  There are
     # 5 leaf buckets, each containing a single int.  Keys in the BTree
     # nodes don't appear in the buckets.  Seven BTree nodes are purely
@@ -991,7 +1472,7 @@ class TestIITreeSets(NormalSetTests, TestCase):
         check(t)
         return t, [1, 3, 5, 7, 11]
 
-    def testDegenerateBasicOps(self):
+    def testBasicOps(self):
         t, keys = self._build_degenerate_tree()
         self.assertEqual(len(t), len(keys))
         self.assertEqual(list(t.keys()), keys)
@@ -1002,7 +1483,7 @@ class TestIITreeSets(NormalSetTests, TestCase):
         self.assertEqual(t.has_key(7), 5)
         self.assertEqual(t.has_key(11), 5)
         for i in 0, 2, 4, 6, 8, 9, 10, 12:
-            self.assertEqual(t.has_key(i), 0)
+            self.assert_(i not in t)
 
     def _checkRanges(self, tree, keys):
         self.assertEqual(len(tree), len(keys))
@@ -1010,7 +1491,7 @@ class TestIITreeSets(NormalSetTests, TestCase):
         sorted_keys.sort()
         self.assertEqual(list(tree.keys()), sorted_keys)
         for k in keys:
-            self.assert_(tree.has_key(k))
+            self.assert_(k in tree)
         if keys:
             lokey = sorted_keys[0]
             hikey = sorted_keys[-1]
@@ -1022,9 +1503,16 @@ class TestIITreeSets(NormalSetTests, TestCase):
         # Try all range searches.
         for lo in range(lokey - 1, hikey + 2):
             for hi in range(lo - 1, hikey + 2):
-                want = [k for k in keys if lo <= k <= hi]
-                got = list(tree.keys(lo, hi))
-                self.assertEqual(want, got)
+                for skipmin in False, True:
+                    for skipmax in False, True:
+                        wantlo, wanthi = lo, hi
+                        if skipmin:
+                            wantlo += 1
+                        if skipmax:
+                            wanthi -= 1
+                        want = [k for k in keys if wantlo <= k <= wanthi]
+                        got = list(tree.keys(lo, hi, skipmin, skipmax))
+                        self.assertEqual(want, got)
 
     def testRanges(self):
         t, keys = self._build_degenerate_tree()
@@ -1061,27 +1549,85 @@ class TestIITreeSets(NormalSetTests, TestCase):
             # at some unrelated line.
             del t   # trigger destructor
 
-class TestOITreeSets(NormalSetTests, TestCase):
+class IIBucketTest(MappingBase):
     def setUp(self):
-        self.t = OITreeSet()
-
-## Bucket tests
-
-class TestIOBuckets(BucketTests, TestCase):
+        self.t = IIBucket()
+class IFBucketTest(MappingBase):
+    def setUp(self):
+        self.t = IFBucket()
+class IOBucketTest(MappingBase):
     def setUp(self):
         self.t = IOBucket()
-
-class TestOOBuckets(BucketTests, TestCase):
+class OIBucketTest(MappingBase):
+    def setUp(self):
+        self.t = OIBucket()
+class OOBucketTest(MappingBase):
     def setUp(self):
         self.t = OOBucket()
 
-class TestIIBuckets(BucketTests, TestCase):
+class IITreeSetTest(NormalSetTests):
     def setUp(self):
-        self.t = IIBucket()
+        self.t = IITreeSet()
+class IFTreeSetTest(NormalSetTests):
+    def setUp(self):
+        self.t = IFTreeSet()
+class IOTreeSetTest(NormalSetTests):
+    def setUp(self):
+        self.t = IOTreeSet()
+class OITreeSetTest(NormalSetTests):
+    def setUp(self):
+        self.t = OITreeSet()
+class OOTreeSetTest(NormalSetTests):
+    def setUp(self):
+        self.t = OOTreeSet()
 
-class TestOIBuckets(BucketTests, TestCase):
+class IISetTest(ExtendedSetTests):
     def setUp(self):
-        self.t = OIBucket()
+        self.t = IISet()
+
+    # Collector 1843.  Error returns were effectively ignored in
+    # Bucket_rangeSearch(), leading to "delayed" errors, or worse.
+    def testNonIntKeyRaises(self):
+        self.t.insert(1)
+        # This one used to fail to raise the TypeError when it occurred.
+        self.assertRaises(TypeError, self.t.keys, "")
+        # This one used to segfault.
+        self.assertRaises(TypeError, self.t.keys, 0, "")
+
+class IFSetTest(ExtendedSetTests):
+    def setUp(self):
+        self.t = IFSet()
+class IOSetTest(ExtendedSetTests):
+    def setUp(self):
+        self.t = IOSet()
+class OISetTest(ExtendedSetTests):
+    def setUp(self):
+        self.t = OISet()
+class OOSetTest(ExtendedSetTests):
+    def setUp(self):
+        self.t = OOSet()
+
+class IIBTreeTest(BTreeTests, TestLongIntKeys, TestLongIntValues):
+    def setUp(self):
+        self.t = IIBTree()
+    def getTwoValues(self):
+        return 1, 2
+class IFBTreeTest(BTreeTests, TestLongIntKeys):
+    def setUp(self):
+        self.t = IFBTree()
+    def getTwoValues(self):
+        return 0.5, 1.5
+class IOBTreeTest(BTreeTests, TestLongIntKeys):
+    def setUp(self):
+        self.t = IOBTree()
+class OIBTreeTest(BTreeTests, TestLongIntValues):
+    def setUp(self):
+        self.t = OIBTree()
+    def getTwoKeys(self):
+        return object(), object()
+class OOBTreeTest(BTreeTests):
+    def setUp(self):
+        self.t = OOBTree()
 
 # cmp error propagation tests
 
@@ -1096,53 +1642,43 @@ class TestCmpError(TestCase):
         try:
             t[DoesntLikeBeingCompared()] = None
         except ValueError,e:
-            assert str(e)=='incomparable'
+            self.assertEqual(str(e), 'incomparable')
         else:
-            raise ValueError('incomarable objects should not be allowed into the tree')
+            self.fail('incomarable objects should not be allowed into '
+                      'the tree')
 
 def test_suite():
-    TIOBTree = makeSuite(TestIOBTrees, 'test')
-    TOOBTree = makeSuite(TestOOBTrees, 'test')
-    TOIBTree = makeSuite(TestOIBTrees, 'test')
-    TIIBTree = makeSuite(TestIIBTrees, 'test')
+    s = TestSuite()
 
-    TIOSet = makeSuite(TestIOSets, 'test')
-    TOOSet = makeSuite(TestOOSets, 'test')
-    TOISet = makeSuite(TestOISets, 'test')
-    TIISet = makeSuite(TestIISets, 'test')
-
-    TIOTreeSet = makeSuite(TestIOTreeSets, 'test')
-    TOOTreeSet = makeSuite(TestOOTreeSets, 'test')
-    TOITreeSet = makeSuite(TestOITreeSets, 'test')
-    TIITreeSet = makeSuite(TestIITreeSets, 'test')
-
-    TIOBucket = makeSuite(TestIOBuckets, 'test')
-    TOOBucket = makeSuite(TestOOBuckets, 'test')
-    TOIBucket = makeSuite(TestOIBuckets, 'test')
-    TIIBucket = makeSuite(TestIIBuckets, 'test')
-
-    alltests = TestSuite((TIOSet, TOOSet, TOISet, TIISet,
-                          TIOTreeSet, TOOTreeSet, TOITreeSet, TIITreeSet,
-                          TIOBucket, TOOBucket, TOIBucket, TIIBucket,
-                          TIOBTree, TOOBTree, TOIBTree, TIIBTree,
-                          makeSuite(TestCmpError),
-                         ))
-
-    return alltests
+    for klass in (
+        IIBucketTest, IIBTreeTest, IISetTest, IITreeSetTest,
+        IFBucketTest, IFBTreeTest, IFSetTest, IFTreeSetTest,
+        IOBucketTest, IOBTreeTest, IOSetTest, IOTreeSetTest,
+        OOBucketTest, OOBTreeTest, OOSetTest, OOTreeSetTest,
+        OIBucketTest, OIBTreeTest, OISetTest, OITreeSetTest,
 
+        # Note:  there is no TestOOBTrees.  The next three are
+        # checking for assorted TypeErrors, and when both keys
+        # and values oare objects (OO), there's nothing to test.
+        TestIIBTrees, TestIFBTrees,  TestIOBTrees,  TestOIBTrees,
+        TestIOSets,
+        DegenerateBTree,
+        TestCmpError):
+        s.addTest(makeSuite(klass))
 
+    return s
 
 ## utility functions
 
 def lsubtract(l1, l2):
-    l1=list(l1)
-    l2=list(l2)
+    l1 = list(l1)
+    l2 = list(l2)
     l = filter(lambda x, l1=l1: x not in l1, l2)
     l = l + filter(lambda x, l2=l2: x not in l2, l1)
     return l
 
 def realseq(itemsob):
-    return map(lambda x: x, itemsob)
+    return [x for x in itemsob]
 
 def permutations(x):
     # Return a list of all permutations of list x.
