@@ -12,7 +12,7 @@
 ##############################################################################
 """Application support
 
-$Id: Application.py 69793 2006-08-27 13:44:44Z shh $
+$Id: Application.py 80016 2007-09-25 15:13:02Z tseaver $
 """
 
 import os, sys, traceback
@@ -569,8 +569,8 @@ class AppInitializer:
                 transaction.commit()
         except:
             bad_things=1
-            LOG.error('The attempt to rebuild the registry failed.',
-                error=sys.exc_info())
+            LOG.error('The attempt to rebuild the registry failed.',exc_info=True)
+               
             transaction.abort()
 
         # Now we need to see if any (disk-based) products were installed
@@ -633,6 +633,12 @@ def install_products(app):
         install_product(app, product_dir, product_name, meta_types,
                         folder_permissions, raise_exc=debug_mode)
 
+    # Delayed install of packages-as-products
+    for module, init_func in getattr(Products, '_packages_to_initialize', []):
+        install_package(app, module, init_func, raise_exc=debug_mode)
+    if hasattr(Products, '_packages_to_initialize'):
+        del Products._packages_to_initialize
+
     Products.meta_types=Products.meta_types+tuple(meta_types)
     InitializeClass(Folder.Folder)
 
@@ -861,6 +867,34 @@ def install_product(app, product_dir, product_name, meta_types,
             if raise_exc:
                 raise
 
+
+def install_package(app, module, init_func, raise_exc=False, log_exc=True):
+    """Installs a Python package like a product."""
+    try:
+        product = App.Product.initializeProduct(module,
+                                                module.__name__,
+                                                module.__path__[0],
+                                                app)
+        product.package_name = module.__name__
+        if init_func is not None:
+            newContext = ProductContext(product, app, module)
+            init_func(newContext)
+
+        if not doInstall():
+            transaction.abort()
+        else:
+            transaction.get().note('Installed package %s' % module.__name__)
+            transaction.commit()
+
+    except:
+        if log_exc:
+            LOG.error("Couldn't install %s" % module.__name__,
+                      exc_info=True)
+        transaction.abort()
+        if raise_exc:
+            raise
+
+
 def install_standards(app):
     # Check to see if we've already done this before
     # Don't do it twice (Casey)
