@@ -12,23 +12,26 @@
 ##############################################################################
 """Cacheable object and cache management base classes.
 
-$Id: Cache.py 70698 2006-10-16 15:24:16Z tseaver $
+$Id: Cache.py 65833 2006-03-06 15:31:58Z shh $
 """
 import time, sys
-import logging
-logger = logging.getLogger('Cache')
-
+from logging import getLogger
 import Globals
+from Globals import InitializeClass
 from Globals import DTMLFile
 from Acquisition import aq_get, aq_acquire, aq_inner, aq_parent, aq_base
+
+from AccessControl import ClassSecurityInfo
 from AccessControl import getSecurityManager
 from AccessControl.Role import _isBeingUsedAsAMethod
 from AccessControl import Unauthorized
+from AccessControl.Permissions import view_management_screens
 
 ZCM_MANAGERS = '__ZCacheManager_ids__'
 
-ViewManagementScreensPermission = 'View management screens'
+ViewManagementScreensPermission = view_management_screens
 ChangeCacheSettingsPermission = 'Change cache settings'
+LOG = getLogger('Cache')
 
 
 def isCacheable(ob):
@@ -88,23 +91,11 @@ class Cacheable:
         'help':('OFSP','Cacheable-properties.stx'),
         },)
 
-    __ac_permissions__ = (
-        (ViewManagementScreensPermission,
-         ('ZCacheable_manage',
-          'ZCacheable_invalidate',
-          'ZCacheable_enabled',
-          'ZCacheable_getManagerId',
-          'ZCacheable_getManagerIds',
-          'ZCacheable_configHTML',
-          'ZCacheable_isAMethod',
-          'ZCacheable_getManagerURL',
-          )),
-        (ChangeCacheSettingsPermission,
-         ('ZCacheable_setManagerId',
-          'ZCacheable_setEnabled',
-          ), ('Manager',)),
-        )
+    security = ClassSecurityInfo()
+    security.setPermissionDefault(ChangeCacheSettingsPermission, ('Manager',))
 
+    security.declareProtected(ViewManagementScreensPermission,
+                              'ZCacheable_manage')
     ZCacheable_manage = DTMLFile('dtml/cacheable', globals())
 
     _v_ZCacheable_cache = None
@@ -113,7 +104,7 @@ class Cacheable:
     __enabled = 1
     _isCacheable = 1
 
-    ZCacheable_getManager__roles__ = ()
+    security.declarePrivate('ZCacheable_getManager')
     def ZCacheable_getManager(self):
         '''Returns the currently associated cache manager.'''
         manager_id = self.__manager_id
@@ -126,7 +117,7 @@ class Cacheable:
         except AttributeError:
             return None
 
-    ZCacheable_getCache__roles__ = ()
+    security.declarePrivate('ZCacheable_getCache')
     def ZCacheable_getCache(self):
         '''Gets the cache associated with this object.
         '''
@@ -147,7 +138,7 @@ class Cacheable:
         self._v_ZCacheable_manager_timestamp = manager_timestamp
         return c
 
-    ZCacheable_isCachingEnabled__roles__ = ()
+    security.declarePrivate('ZCacheable_isCachingEnabled')
     def ZCacheable_isCachingEnabled(self):
         '''
         Returns true only if associated with a cache manager and
@@ -155,6 +146,8 @@ class Cacheable:
         '''
         return self.__enabled and self.ZCacheable_getCache()
 
+    security.declareProtected(ViewManagementScreensPermission,
+                              'ZCacheable_isAMethod')
     def ZCacheable_isAMethod(self):
         '''
         Returns 1 when this object is a ZClass method.
@@ -162,7 +155,7 @@ class Cacheable:
         m = _isBeingUsedAsAMethod(self)
         return m
 
-    ZCacheable_getObAndView__roles__ = ()
+    security.declarePrivate('ZCacheable_getObAndView')
     def ZCacheable_getObAndView(self, view_name):
         """
         If this object is a method of a ZClass and we're working
@@ -182,7 +175,7 @@ class Cacheable:
                 ob = self
         return ob, view_name
 
-    ZCacheable_get__roles__ = ()
+    security.declarePrivate('ZCacheable_get')
     def ZCacheable_get(self, view_name='', keywords=None,
                        mtime_func=None, default=None):
         '''Retrieves the cached view for the object under the
@@ -197,11 +190,12 @@ class Cacheable:
                                    mtime_func, default)
                 return val
             except:
-                logger.warn('ZCache_get() exception', exc_info=True)
+                LOG.warn('ZCache_get() exception',
+                    exc_info=sys.exc_info())
                 return default
         return default
 
-    ZCacheable_set__roles__ = ()
+    security.declarePrivate('ZCacheable_set')
     def ZCacheable_set(self, data, view_name='', keywords=None,
                        mtime_func=None):
         '''Cacheable views should call this method after generating
@@ -214,8 +208,11 @@ class Cacheable:
                 c.ZCache_set(ob, data, view_name, keywords,
                              mtime_func)
             except:
-                logger.warn('ZCache_set() exception', exc_info=True)
+                LOG.warn('ZCache_set() exception',
+                    exc_info=sys.exc_info())
 
+    security.declareProtected(ViewManagementScreensPermission,
+                              'ZCacheable_invalidate')
     def ZCacheable_invalidate(self, view_name='', REQUEST=None):
         '''Called after a cacheable object is edited. Causes all
         cache entries that apply to the view_name to be removed.
@@ -231,7 +228,8 @@ class Cacheable:
             except:
                 exc = sys.exc_info()
                 try:
-                    logger.warn('ZCache_invalidate() exception', exc_info=True)
+                    LOG.warn('ZCache_invalidate() exception',
+                        exc_info=exc)
                     message = 'An exception occurred: %s: %s' % exc[:2]
                 finally:
                     exc = None
@@ -244,7 +242,7 @@ class Cacheable:
         else:
             return message
 
-    ZCacheable_getModTime__roles__=()
+    security.declarePrivate('ZCacheable_getModTime')
     def ZCacheable_getModTime(self, mtime_func=None):
         '''Returns the highest of the last mod times.'''
         # Based on:
@@ -272,10 +270,14 @@ class Cacheable:
                 mtime = max(getattr(klass, '_p_mtime', mtime), mtime)
         return mtime
 
+    security.declareProtected(ViewManagementScreensPermission,
+                              'ZCacheable_getManagerId')
     def ZCacheable_getManagerId(self):
         '''Returns the id of the current ZCacheManager.'''
         return self.__manager_id
 
+    security.declareProtected(ViewManagementScreensPermission,
+                              'ZCacheable_getManagerURL')
     def ZCacheable_getManagerURL(self):
         '''Returns the URL of the current ZCacheManager.'''
         manager = self.ZCacheable_getManager()
@@ -283,6 +285,8 @@ class Cacheable:
             return manager.absolute_url()
         return None
 
+    security.declareProtected(ViewManagementScreensPermission,
+                              'ZCacheable_getManagerIds')
     def ZCacheable_getManagerIds(self):
         '''Returns a list of mappings containing the id and title
         of the available ZCacheManagers.'''
@@ -304,6 +308,8 @@ class Cacheable:
             ob = aq_parent(aq_inner(ob))
         return tuple(rval)
 
+    security.declareProtected(ChangeCacheSettingsPermission,
+                              'ZCacheable_setManagerId')
     def ZCacheable_setManagerId(self, manager_id, REQUEST=None):
         '''Changes the manager_id for this object.'''
         self.ZCacheable_invalidate()
@@ -320,11 +326,15 @@ class Cacheable:
                 self, REQUEST, management_view='Cache',
                 manage_tabs_message='Cache settings changed.')
 
+    security.declareProtected(ViewManagementScreensPermission,
+                              'ZCacheable_enabled')
     def ZCacheable_enabled(self):
         '''Returns true if caching is enabled for this object
         or method.'''
         return self.__enabled
 
+    security.declareProtected(ChangeCacheSettingsPermission,
+                              'ZCacheable_setEnabled')
     def ZCacheable_setEnabled(self, enabled=0, REQUEST=None):
         '''Changes the enabled flag. Normally used only when
         setting up cacheable ZClass methods.'''
@@ -334,6 +344,8 @@ class Cacheable:
                 self, REQUEST, management_view='Cache',
                 manage_tabs_message='Cache settings changed.')
 
+    security.declareProtected(ViewManagementScreensPermission,
+                              'ZCacheable_configHTML')
     def ZCacheable_configHTML(self):
         '''Override to provide configuration of caching
         behavior that can only be specific to the cacheable object.
@@ -341,7 +353,7 @@ class Cacheable:
         return ''
 
 
-Globals.default__class_init__(Cacheable)
+InitializeClass(Cacheable)
 
 
 def findCacheables(ob, manager_id, require_assoc, subfolders,
@@ -433,19 +445,15 @@ class CacheManager:
     A base class for cache managers.  Implement ZCacheManager_getCache().
     '''
 
-    ZCacheManager_getCache__roles__ = ()
+    security = ClassSecurityInfo()
+    security.setPermissionDefault(ChangeCacheSettingsPermission, ('Manager',))
+
+    security.declarePrivate('ZCacheManager_getCache')
     def ZCacheManager_getCache(self):
         raise NotImplementedError
 
     _isCacheManager = 1
 
-    __ac_permissions__ = (
-        ('Change cache settings', ('ZCacheManager_locate',
-                                   'ZCacheManager_setAssociations',
-                                   'ZCacheManager_associate'),
-         ('Manager',)),
-        )
-
     manage_options = (
         {'label':'Associate',
          'action':'ZCacheManager_associate',
@@ -474,8 +482,12 @@ class CacheManager:
                 global manager_timestamp
                 manager_timestamp = time.time()
 
+    security.declareProtected(ChangeCacheSettingsPermission,
+                              'ZCacheManager_associate')
     ZCacheManager_associate = DTMLFile('dtml/cmassoc', globals())
 
+    security.declareProtected(ChangeCacheSettingsPermission,
+                              'ZCacheManager_locate')
     def ZCacheManager_locate(self, require_assoc, subfolders,
                              meta_types=[], REQUEST=None):
         '''Locates cacheable objects.
@@ -495,6 +507,8 @@ class CacheManager:
         else:
             return rval
 
+    security.declareProtected(ChangeCacheSettingsPermission,
+                              'ZCacheManager_setAssociations')
     def ZCacheManager_setAssociations(self, props=None, REQUEST=None):
         '''Associates and un-associates cacheable objects with this
         cache manager.
@@ -531,4 +545,4 @@ class CacheManager:
                 (addcount, remcount)
                 )
 
-Globals.default__class_init__(CacheManager)
+InitializeClass(CacheManager)
