@@ -14,11 +14,11 @@
 
 Folders are the basic container objects and are analogous to directories.
 
-$Id: Folder.py 40222 2005-11-18 15:46:28Z andreasjung $
+$Id: Folder.py 40300 2005-11-21 16:54:03Z efge $
 """
 
 import AccessControl.Role, webdav.Collection
-import Globals
+from Globals import InitializeClass
 from AccessControl import getSecurityManager
 from AccessControl import Unauthorized
 from AccessControl.Permissions import add_page_templates
@@ -108,4 +108,4 @@ class Folder(
         if id is not None:
             self.id = str(id)
 
-Globals.default__class_init__(Folder)
+InitializeClass(Folder)
