@@ -1,23 +1,21 @@
 ##############################################################################
 #
-# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
+# Copyright (c) 2002 Zope Corporation and Contributors. All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
 # FOR A PARTICULAR PURPOSE
 #
 ##############################################################################
-import Shared.DC.xml.ppml
-ppml=Shared.DC.xml.ppml
 from base64 import encodestring
 from cStringIO import StringIO
 from ZODB.serialize import referencesf
 from ZODB.ExportImport import TemporaryFile, export_end_marker
+from Shared.DC.xml import ppml
 
-StringType=type('')
 
 magic='<?xm' # importXML(jar, file, clue)}
 
@@ -37,7 +35,7 @@ def XMLrecord(oid, len, p):
 def exportXML(jar, oid, file=None):
 
     if file is None: file=TemporaryFile()
-    elif type(file) is StringType: file=open(file,'w+b')
+    elif type(file) is str: file=open(file,'w+b')
     write=file.write
     write('<?xml version="1.0"?>\012<ZopeData>\012')
     version=jar._version
@@ -98,10 +96,9 @@ def save_record(parser, tag, data):
     return v
 
 def importXML(jar, file, clue=''):
-    import Shared.DC.xml.pyexpat.pyexpat
-    pyexpat=Shared.DC.xml.pyexpat.pyexpat
-    if type(file) is StringType:
-        file=open(file)
+    import xml.parsers.expat
+    if type(file) is str:
+        file=open(file, 'rb')
     outfile=TemporaryFile()
     data=file.read()
     F=ppml.xmlPickler()
@@ -110,7 +107,7 @@ def importXML(jar, file, clue=''):
     F.start_handlers['ZopeData'] = start_zopedata
     F.binary=1
     F.file=outfile
-    p=pyexpat.ParserCreate()
+    p=xml.parsers.expat.ParserCreate()
     p.CharacterDataHandler=F.handle_data
     p.StartElementHandler=F.unknown_starttag
     p.EndElementHandler=F.unknown_endtag
