@@ -13,17 +13,25 @@
 ##############################################################################
 """Initialize the Five product
 
-$Id: __init__.py 62056 2005-11-26 09:26:04Z philikon $
+$Id: __init__.py 71269 2006-11-22 13:37:06Z yuppie $
 """
-import Acquisition
-from Globals import INSTANCE_HOME
-
-import zcml
+from Products.Five import zcml
 
 # public API provided by Five
 # usage: from Products.Five import <something>
-from browser import BrowserView
-from skin.standardmacros import StandardMacros
+from Products.Five.browser import BrowserView
+from Products.Five.skin.standardmacros import StandardMacros
 
+# load the site's ZCML tree (usually site.zcml) upon product
+# initialization
 def initialize(context):
     zcml.load_site()
+
+# some convenience methods/decorators
+
+def fivemethod(func):
+    func.__five_method__ = True
+    return func
+
+def isFiveMethod(m):
+    return hasattr(m, '__five_method__')
