@@ -13,12 +13,12 @@
 ##############################################################################
 """Provide basic browser functionality
 
-$Id: __init__.py 61072 2005-10-31 17:43:51Z philikon $
+$Id: __init__.py 67820 2006-05-02 09:21:45Z philikon $
 """
 import Acquisition
-import zope.app.publisher.browser
+import zope.publisher.browser
 
-class BrowserView(Acquisition.Explicit, zope.app.publisher.browser.BrowserView):
+class BrowserView(Acquisition.Explicit, zope.publisher.browser.BrowserView):
     """Five browser view
 
     Mixes in explicit acquisition so that security can be acquired for
