@@ -13,7 +13,7 @@
 ##############################################################################
 """Provide basic resource functionality
 
-$Id: resource.py 71269 2006-11-22 13:37:06Z yuppie $
+$Id: resource.py 73646 2007-03-26 18:39:09Z jinty $
 """
 import os
 import urllib
@@ -61,6 +61,9 @@ class PageTemplateResource(BrowserView, Resource):
 
     def render(self):
         """Rendered content"""
+        # ZPublisher might have called setBody with an incorrect URL
+        # we definitely don't want that if we are plain html
+        self.request.RESPONSE.setBase(None)
         pt = self.context
         return pt(self.request)
 
