@@ -13,16 +13,19 @@
 ##############################################################################
 """Component tests
 
-$Id: tests.py 67584 2006-04-25 08:49:16Z philikon $
+$Id: tests.py 71269 2006-11-22 13:37:06Z yuppie $
 """
 import unittest
-from zope.testing.doctestunit import DocFileSuite, DocTestSuite
+from zope.testing.doctestunit import DocFileSuite
+from Testing.ZopeTestCase import FunctionalDocFileSuite
 
 __docformat__ = "reStructuredText"
 
 def test_suite():
     return unittest.TestSuite([
         DocFileSuite('component.txt', package="Products.Five.component"),
+        FunctionalDocFileSuite('makesite.txt',
+                               package="Products.Five.component"),
         ])
 
 if __name__ == '__main__':
