@@ -13,15 +13,14 @@
 ##############################################################################
 """Five ZCML directive schemas
 
-$Id: fivedirectives.py 61010 2005-10-31 01:44:26Z efge $
+$Id: fivedirectives.py 67874 2006-05-02 23:22:01Z rocky $
 """
 from zope.interface import Interface
 from zope.app.publisher.browser.metadirectives import IBasicResourceInformation
-from zope.app.security.fields import Permission
+from zope.security.zcml import Permission
 from zope.configuration.fields import GlobalObject, Tokens, PythonIdentifier
 from zope.configuration.fields import Bool
-from zope.schema import ASCII
-from zope.schema import TextLine
+from zope.schema import ASCII, TextLine, BytesLine
 
 class IImplementsDirective(Interface):
     """State that a class implements something.
@@ -37,6 +36,7 @@ class IImplementsDirective(Interface):
         value_type=GlobalObject()
         )
 
+# BBB 2006/05/01 -- to be removed after 12 months
 class ITraversableDirective(Interface):
     """Make instances of class traversable publically.
 
@@ -49,6 +49,7 @@ class ITraversableDirective(Interface):
         required=True
         )
 
+# BBB 2006/05/01 -- to be removed after 12 months
 class IDefaultViewableDirective(Interface):
     """Make instances of class viewable publically.
 
@@ -168,3 +169,31 @@ class IRegisterClassDirective(Interface):
         default=True,
         required=False
         )
+
+
+
+class IInclude(Interface):
+
+    file = BytesLine(
+        title=u'Configuration file name',
+        description=u'Name of a configuration file to be included for each '
+                    u'installed Product. If the file does not exist, for a '
+                    u'particular product, no error is raised.',
+        required=False)
+
+class IRegisterPackageDirective(Interface):
+    """Registers the given python package which at a minimum fools zope2 into
+    thinking of it as a zope2 product.
+    """
+
+    package = GlobalObject(
+        title=u'Target package',
+        required=True
+        )
+
+    initialize = GlobalObject(
+        title=u'Initialization function to invoke',
+        description=u'The dotted name of a function that will get invoked '
+                    u'with a ProductContext instance',
+        required=False
+        )
