@@ -13,14 +13,11 @@
 ##############################################################################
 """Five baseclasses for zope.formlib.form
 
-$Id: view.py 65461 2006-02-25 19:50:17Z philikon $
+$Id: view.py 71269 2006-11-22 13:37:06Z yuppie $
 """
-from zope.formlib import form, page
-
-from Products.Five.browser import BrowserView
-from Products.Five.formlib.formbase import AddForm, EditForm, Form
-from Products.Five.browser.pagetemplatefile import ZopeTwoPageTemplateFile
+from zope.formlib import form
 
+from Products.Five.formlib.formbase import AddForm, EditForm
 from Products.Five.formlib.tests.content import IContent, Content
 
 class AddContentForm(AddForm):
