@@ -13,13 +13,12 @@
 ##############################################################################
 """Local sites
 
-$Id: localsite.py 69271 2006-07-27 14:15:46Z regebro $
+$Id: localsite.py 71269 2006-11-22 13:37:06Z yuppie $
 """
+from zope.app.component.interfaces import ISite, IPossibleSite
+from zope.component import getGlobalSiteManager
 from zope.interface import implements
 from zope.interface.interface import InterfaceClass
-from zope.component import getGlobalSiteManager
-from zope.component.interfaces import ComponentLookupError
-from zope.app.component.interfaces import ISite, IPossibleSite
 from Acquisition import aq_parent, aq_inner
 from Products.Five.site.interfaces import IFiveSiteManager, IFiveUtilityRegistry
 
@@ -51,7 +50,7 @@ class FiveSiteManager(object):
     def __bases__(self):
         next = self.next
         if next is None:
-            return ()
+            return (getGlobalSiteManager(),)
         return (next,)
 
     @property
