@@ -18,13 +18,8 @@ These directives are specific to Five and have no equivalents in Zope 3.
 $Id: fiveconfigure.py 18581 2005-10-14 16:54:25Z regebro $
 """
 from zope.interface import classImplements, classImplementsOnly, implementedBy
-from zope.interface.interface import InterfaceClass
-from zope.configuration.exceptions import ConfigurationError
-from zope.component.zcml import adapter
 from zope.app.component.interfaces import IPossibleSite
 
-from Products.Five.site.localsite import FiveSite
-
 import logging, warnings
 
 LOG = logging.getLogger('Five')
@@ -45,7 +40,12 @@ def installSiteHook(_context, class_, site_class=None):
                            "See Five/doc/localsite.txt .",
                            DeprecationWarning, 
                            _context.info.file, _context.info.line)
-    if site_class is not None:
+
+    # only install the hook once
+    already = getattr(class_, '_localsite_marker', False)
+
+    if site_class is not None and not already:
+        class_._localsite_marker = True
         _context.action(
             discriminator = (class_,),
             callable = classSiteHook,
@@ -66,6 +66,8 @@ def uninstallSiteHooks():
         delattr(class_, 'getSiteManager')
         delattr(class_, 'setSiteManager')
         classImplementsOnly(class_, implementedBy(class_)-IPossibleSite)
+        if getattr(class_, '_localsite_marker', False):
+            delattr(class_, '_localsite_marker')
 
 from zope.testing.cleanup import addCleanUp
 addCleanUp(uninstallSiteHooks)
