@@ -13,9 +13,9 @@
 ##############################################################################
 """Dummy test fixtures
 
-$Id: dummy.py 60814 2005-10-26 17:24:47Z philikon $
+$Id: dummy.py 67820 2006-05-02 09:21:45Z philikon $
 """
-from zope.interface import implements, Interface
+from zope.interface import implements, implementsOnly, Interface
 from OFS.SimpleItem import SimpleItem
 from Products.Five.tests.testing import FiveTraversableFolder
 
@@ -25,7 +25,10 @@ class IDummySite(Interface):
 class DummySite(FiveTraversableFolder):
     """A very dummy Site
     """
-    implements(IDummySite)
+    # we specifically do not let this site inherit any interfaces from
+    # the superclasses so that this class does not implement
+    # IPossibleSite under any circumstances
+    implementsOnly(IDummySite)
 
 def manage_addDummySite(self, id, REQUEST=None):
     """Add the dummy site."""
