@@ -13,7 +13,7 @@
 ##############################################################################
 """Test local sites
 
-$Id: test_localsite.py 69269 2006-07-27 12:33:37Z regebro $
+$Id: test_localsite.py 71269 2006-11-22 13:37:06Z yuppie $
 """
 import os, sys
 
@@ -21,7 +21,6 @@ if __name__ == '__main__':
     execfile(os.path.join(sys.path[0], 'framework.py'))
 
 import unittest
-from Testing import ZopeTestCase
 
 import zope.interface
 from zope.component import getGlobalSiteManager, getSiteManager
@@ -29,7 +28,7 @@ from zope.component.interfaces import ComponentLookupError
 from zope.component.interfaces import IComponentLookup
 from zope.traversing.interfaces import IContainmentRoot
 from zope.app.component.hooks import setSite, getSite, setHooks
-from zope.app.component.interfaces import IPossibleSite, ISite
+from zope.app.component.interfaces import ISite
 from zope.app.testing.placelesssetup import PlacelessSetup
 
 from Acquisition import Implicit
