@@ -13,12 +13,11 @@
 ##############################################################################
 """Test folders
 
-$Id: folder.py 56257 2005-07-10 11:39:32Z philikon $
+$Id: folder.py 67728 2006-04-28 17:57:47Z regebro $
 """
 from OFS.Folder import Folder
 from OFS.interfaces import IFolder
 from zope.interface import implements
-from Products.Five.traversable import Traversable
 
 class NoVerifyPasteFolder(Folder):
     """Folder that does not perform paste verification.
@@ -33,7 +32,7 @@ def manage_addNoVerifyPasteFolder(container, id, title=''):
     folder.id = id
     folder.title = title
 
-class FiveTraversableFolder(Traversable, Folder):
+class FiveTraversableFolder(Folder):
     """Folder that is five-traversable
     """
     implements(IFolder)
