@@ -1,9 +1,9 @@
 ##############################################################################
 #
-# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
+# Copyright (c) 2002 Zope Corporation and Contributors. All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
@@ -11,7 +11,7 @@
 #
 ##############################################################################
 __doc__='''Object system core
-$Id: __init__.py,v 1.38 2003/06/12 10:21:01 yuppie Exp $'''
+$Id: __init__.py 40218 2005-11-18 14:39:19Z andreasjung $'''
 __version__='$Revision: 1.38 $'[11:-2]
 
 import Version, OFS.Image, OFS.Folder, AccessControl.User
@@ -22,6 +22,7 @@ import ZClasses.ObjectManager
 from AccessControl.Permissions import add_documents_images_and_files
 from AccessControl.Permissions import add_folders
 from ZClasses import createZClassForBase
+from App.ImageFile import ImageFile
 
 createZClassForBase( OFS.DTMLMethod.DTMLMethod, globals()
                    , 'ZDTMLMethod', 'DTML Method' )
@@ -39,6 +40,10 @@ createZClassForBase( AccessControl.User.UserFolder, globals()
 createZClassForBase( AccessControl.User.User, globals()
                    , 'ZUser', 'User' )
 
+misc_={
+    'version.gif':ImageFile('images/version.gif', globals())
+    }
+
 # This is the new way to initialize products.  It is hoped
 # that this more direct mechanism will be more understandable.
 def initialize(context):
@@ -105,13 +110,17 @@ def initialize(context):
         legacy=(AccessControl.User.manage_addUserFolder,),
         )
 
+    ## Those both classes are relicts. We only withdraw them from the Add menu.
+    ## This way people will stop using them. They are undocumented anyway. 
+    ## People also have the chance to softly migrate their data and stop using the
+    ## versions they still use.
 
-    context.registerClass(
-        Version.Version,
-        constructors=(Version.manage_addVersionForm,
-                      Version.manage_addVersion),
-        icon='images/version.gif'
-        )
+    #context.registerClass(
+    #    Version.Version,
+    #    constructors=(Version.manage_addVersionForm,
+    #                  Version.manage_addVersion),
+    #    icon='images/version.gif'
+    #    )
 
     #context.registerClass(
     #    Draft.Draft,
