@@ -14,7 +14,7 @@
 
 Handler for Python expressions that uses the RestrictedPython package.
 
-$Id: ZRPythonExpr.py 68333 2006-05-29 10:34:19Z philikon $
+$Id: ZRPythonExpr.py 73153 2007-03-12 17:24:50Z andreasjung $
 """
 from AccessControl import safe_builtins
 from AccessControl.ZopeGuards import guarded_getattr, get_safe_globals
@@ -28,8 +28,14 @@ class PythonExpr(PythonExpr):
     _globals['__debug__' ] = __debug__
 
     def __init__(self, name, expr, engine):
-        self.text = text = expr.strip().replace('\n', ' ')
-        code, err, warn, use = compile_restricted_eval(text, str(self))
+        self.text = self.expr = text = expr.strip().replace('\n', ' ')
+
+        # Unicode expression are not handled properly by RestrictedPython
+        # We convert the expression to UTF-8 (ajung)
+        if isinstance(text, unicode):
+            text = text.encode('utf-8')
+        code, err, warn, use = compile_restricted_eval(text, 
+                                                       self.__class__.__name__)
         if err:
             raise engine.getCompilerError()('Python expression error:\n%s' %
                                             '\n'.join(err))            
