@@ -12,7 +12,7 @@
 ##############################################################################
 """Zope Page Template module (wrapper for the Zope 3 ZPT implementation)
 
-$Id: ZopePageTemplate.py 72205 2007-01-24 10:08:19Z andreasjung $
+$Id: ZopePageTemplate.py 78365 2007-07-26 22:04:15Z hannosch $
 """
 import re
 import os
@@ -126,7 +126,7 @@ class ZopePageTemplate(Script, PageTemplate, Historical, Cacheable,
         encoding = None
         output_encoding = None
 
-        if content_type in ('text/xml',):
+        if content_type.startswith('text/xml'):
 
             if is_unicode:
                 encoding = None
@@ -134,9 +134,8 @@ class ZopePageTemplate(Script, PageTemplate, Historical, Cacheable,
             else:
                 encoding = encodingFromXMLPreamble(text)
                 output_encoding = 'utf-8'
-            
 
-        elif content_type in ('text/html',) :
+        elif content_type.startswith('text/html'):
 
             charset = charsetFromMetaEquiv(text)
 
