@@ -15,12 +15,18 @@
 This wrapper allows the Page Template modules to be segregated in a
 separate package.
 
-$Id: __init__.py 68333 2006-05-29 10:34:19Z philikon $
+$Id: __init__.py 72027 2007-01-14 14:57:11Z philikon $
 """
 
 # Placeholder for Zope Product data
 misc_ = {}
 
+# import ZTUtils in order to make i importable through
+# ZopeGuards.load_module() where an importable modules must be
+# available in sys.modules
+import ZTUtils
+
+
 def initialize(context):
     # Import lazily, and defer initialization to the module
     import ZopePageTemplate
