@@ -11,12 +11,17 @@
 #
 ##############################################################################
 
-import os, sys, unittest
+import unittest
 
+import zope.component.testing
+from zope.component import provideUtility
+from zope.traversing.adapters import DefaultTraversable
 from Products.PageTemplates.tests import util
 from Products.PageTemplates.PageTemplate import PageTemplate
 from Products.PageTemplates.GlobalTranslationService import \
      setGlobalTranslationService
+from Products.PageTemplates.interfaces import IUnicodeEncodingConflictResolver
+from Products.PageTemplates.unicodeconflictresolver import DefaultUnicodeEncodingConflictResolver
 from AccessControl import SecurityManager
 from AccessControl.SecurityManagement import noSecurityManager
 
@@ -59,9 +64,14 @@ class UnitTestSecurityPolicy:
     def checkPermission( self, permission, object, context) :
         return 1
 
-class HTMLTests(unittest.TestCase):
+class HTMLTests(zope.component.testing.PlacelessSetup, unittest.TestCase):
 
     def setUp(self):
+        super(HTMLTests, self).setUp()
+        zope.component.provideAdapter(DefaultTraversable, (None,))
+
+        provideUtility(DefaultUnicodeEncodingConflictResolver, IUnicodeEncodingConflictResolver)
+
         self.folder = f = Folder()
         f.laf = AqPageTemplate()
         f.t = AqPageTemplate()
@@ -70,6 +80,7 @@ class HTMLTests(unittest.TestCase):
         noSecurityManager()  # Use the new policy.
 
     def tearDown(self):
+        super(HTMLTests, self).tearDown()
         SecurityManager.setSecurityPolicy( self.oldPolicy )
         noSecurityManager()  # Reset to old policy.
 
@@ -156,6 +167,12 @@ class HTMLTests(unittest.TestCase):
         self.assert_expected(self.folder.t, 'CheckI18nTranslateHooked.html')
         setGlobalTranslationService(old_ts)
 
+    def checkImportOldStyleClass(self):
+        self.assert_expected(self.folder.t, 'CheckImportOldStyleClass.html')
+
+    def checkRepeatVariable(self):
+        self.assert_expected(self.folder.t, 'RepeatVariable.html')
+
     def checkBooleanAttributesAndDefault(self):
         # Zope 2.9 and below support the semantics that an HTML
         # "boolean" attribute (e.g. 'selected', 'disabled', etc.) can
