@@ -12,14 +12,13 @@
 ##############################################################################
 """Date index.
 
-$Id: DateIndex.py 68835 2006-06-24 22:45:20Z tseaver $
+$Id: DateIndex.py 41190 2006-01-08 10:22:28Z andreasjung $
 """
 
 import time
+from logging import getLogger
 from datetime import date, datetime
 from datetime import tzinfo, timedelta
-import logging
-logger = logging.getLogger('UnIndex')
 from types import StringType, FloatType, IntType
 
 import BTrees.Length
@@ -37,6 +36,7 @@ from Products.PluginIndexes.common.UnIndex import UnIndex
 from Products.PluginIndexes.common.util import parseIndexRequest
 from Products.PluginIndexes.interfaces import IDateIndex
 
+LOG = getLogger('DateIndex')
 _marker = []
 
 ###############################################################################
@@ -145,10 +145,9 @@ class DateIndex(UnIndex, PropertyManager):
                     except ConflictError:
                         raise
                     except:
-                        logger.error(
-                            ("Should not happen: ConvertedDate was there,"
-                             " now it's not, for document with id %s" %
-                             documentId))
+                        LOG.error("Should not happen: ConvertedDate was there,"
+                                  " now it's not, for document with id %s" %
+                                  documentId)
 
             if ConvertedDate is not _marker:
                 self.insertForwardIndexEntry( ConvertedDate, documentId )
