@@ -15,6 +15,7 @@ import re, time, sys
 from logging import getLogger
 import Globals
 from OFS.SimpleItem import Item
+from ZODB.POSException import ConflictError
 from Acquisition import Implicit, Explicit, aq_base
 from Persistence import Persistent
 from AccessControl.Owned import Owned
@@ -219,6 +220,8 @@ class SessionDataManager(Item, Implicit, Persistent, RoleManager, Owned, Tabs):
                 LOG.debug('External data container at %s in use' % args)
                 self._v_wrote_dc_type = 1
             return self.unrestrictedTraverse(self.obpath)
+        except ConflictError:
+            raise
         except:
             raise SessionDataManagerErr, (
                 "External session data container '%s' not found." %
