@@ -10,8 +10,8 @@ from AccessControl.Permissions import view as View
 from OFS.SimpleItem import Item
 from Acquisition import Implicit, aq_inner, aq_parent
 from ZPublisher import BeforeTraverse
+from ZPublisher.BaseRequest import quote
 from zExceptions import BadRequest
-from urllib import quote
 import os
 
 from AccessRule import _swallow
