@@ -4,7 +4,7 @@
 # All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
@@ -13,7 +13,7 @@
 ##############################################################################
 """Site error log module.
 
-$Id: SiteErrorLog.py 30812 2005-06-16 16:24:13Z yuppie $
+$Id: SiteErrorLog.py 68064 2006-05-09 11:28:02Z shh $
 """
 
 import os
@@ -71,7 +71,7 @@ class SiteErrorLog (SimpleItem):
     id = 'error_log'
 
     keep_entries = 20
-    copy_to_zlog = 0
+    copy_to_zlog = True
 
     security = ClassSecurityInfo()
 
@@ -213,18 +213,18 @@ class SiteErrorLog (SimpleItem):
                 LOG.error('Error while logging', exc_info=sys.exc_info())
             else:
                 if self.copy_to_zlog:
-                    self._do_copy_to_zlog(now,strtype,str(url),info)
+                    self._do_copy_to_zlog(now,strtype,str(url),tb_text)
                 return '%s/showEntry?id=%s' % (self.absolute_url(), entry_id)
         finally:
             info = None
 
-    def _do_copy_to_zlog(self,now,strtype,url,info):
+    def _do_copy_to_zlog(self,now,strtype,url,tb_text):
         when = _rate_restrict_pool.get(strtype,0)
         if now>when:
             next_when = max(when, now-_rate_restrict_burst*_rate_restrict_period)
             next_when += _rate_restrict_period
             _rate_restrict_pool[strtype] = next_when
-            LOG.error(str(url), exc_info=info)
+            LOG.error('%s\n%s' % (url, tb_text.rstrip()))
 
     security.declareProtected(use_error_logging, 'getProperties')
     def getProperties(self):
