@@ -1,9 +1,9 @@
 ##############################################################################
 #
-# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
+# Copyright (c) 2002 Zope Corporation and Contributors. All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
@@ -13,7 +13,7 @@
 """
 Transient Object Container Class ('timeslice'-based design, no index).
 
-$Id: Transience.py,v 1.32.12.10 2004/10/11 02:40:48 chrism Exp $
+$Id: Transience.py 41203 2006-01-08 11:08:49Z andreasjung $
 """
 
 import math
@@ -40,9 +40,10 @@ from AccessControl import ClassSecurityInfo, getSecurityManager
 from AccessControl.SecurityManagement import newSecurityManager, \
      setSecurityManager
 from AccessControl.User import nobody
-from zLOG import LOG, WARNING, INFO
+from logging import getLogger
 
 from TransientObject import TransientObject
+from Fake import FakeIOBTree
 
 ADD_CONTAINER_PERM = 'Add Transient Object Container'
 MGMT_SCREEN_PERM = 'View management screens'
@@ -58,6 +59,7 @@ STRICT = os.environ.get('Z_TOC_STRICT', '')
 DEBUG = int(os.environ.get('Z_TOC_DEBUG', 0))
 
 _marker = []
+LOG = getLogger('Transience')
 
 def setStrict(on=''):
     """ Turn on assertions (which may cause conflicts) """
@@ -71,13 +73,13 @@ def TLOG(*args):
     for arg in args:
         sargs.append(str(arg))
     msg = ' '.join(sargs)
-    LOG('Transience', INFO, msg)
+    LOG.info(msg)
 
 constructTransientObjectContainerForm = HTMLFile(
     'dtml/addTransientObjectContainer', globals())
 
 def constructTransientObjectContainer(self, id, title='', timeout_mins=20,
-    addNotification=None, delNotification=None, limit=0, period_secs=60,
+    addNotification=None, delNotification=None, limit=0, period_secs=20,
     REQUEST=None):
     """ """
     ob = TransientObjectContainer(id, title, timeout_mins,
@@ -110,7 +112,6 @@ class TransientObjectContainer(SimpleItem):
     )
 
     security = ClassSecurityInfo()
-    security.declareObjectProtected('View')
     security.setPermissionDefault(MANAGE_CONTAINER_PERM,
                                 ['Manager',])
     security.setPermissionDefault(MGMT_SCREEN_PERM,
@@ -141,7 +142,7 @@ class TransientObjectContainer(SimpleItem):
     gc_lock = thread.allocate_lock()
 
     def __init__(self, id, title='', timeout_mins=20, addNotification=None,
-                 delNotification=None, limit=0, period_secs=60):
+                 delNotification=None, limit=0, period_secs=20):
         self.id = id
         self.title=title
         self._setTimeout(timeout_mins, period_secs)
@@ -243,7 +244,10 @@ class TransientObjectContainer(SimpleItem):
         # we need to maintain the length of the index structure separately
         # because getting the length of a BTree is very expensive, and it
         # doesn't really tell us which ones are "active" anyway.
-        self._length = self.getLen = Length2()
+        try:
+            self._length.set(0)
+        except AttributeError:
+            self._length = self.getLen = Length2()
 
     def _getCurrentSlices(self, now):
         if self._timeout_slices:
@@ -431,10 +435,9 @@ class TransientObjectContainer(SimpleItem):
             length = self._length() # XXX ReadConflictError hotspot
 
             if self._limit and length >= self._limit:
-                LOG('Transience', WARNING,
-                    ('Transient object container %s max subobjects '
-                     'reached' % self.getId())
-                    )
+                LOG.warn('Transient object container %s max subobjects '
+                         'reached' % self.getId())
+                    
                 raise MaxTransientObjectsExceeded, (
                  "%s exceeds maximum number of subobjects %s" %
                  (length, self._limit))
@@ -793,11 +796,8 @@ class TransientObjectContainer(SimpleItem):
             except (KeyError, AttributeError):
                 path = self.getPhysicalPath()
                 err = 'No such onAdd/onDelete method %s referenced via %s'
-                LOG('Transience',
-                    WARNING,
-                    err % (callback, '/'.join(path)),
-                    error=sys.exc_info()
-                    )
+                LOG.warn(err % (callback, '/'.join(path)),
+                         exc_info=sys.exc_info())
                 return
         else:
             method = callback
@@ -814,22 +814,16 @@ class TransientObjectContainer(SimpleItem):
                 except:
                     # dont raise, just log
                     path = self.getPhysicalPath()
-                    LOG('Transience',
-                        WARNING,
-                        '%s failed when calling %s in %s' % (name,callback,
-                                                        '/'.join(path)),
-                        error=sys.exc_info()
-                        )
+                    LOG.warn('%s failed when calling %s in %s' % (name,callback,
+                                                                 '/'.join(path)),
+                             exc_info=sys.exc_info())
             finally:
                 setSecurityManager(sm)
         else:
             err = '%s in %s attempted to call non-callable %s'
             path = self.getPhysicalPath()
-            LOG('Transience',
-                WARNING,
-                err % (name, '/'.join(path), callback),
-                error=sys.exc_info()
-                )
+            LOG.warn(err % (name, '/'.join(path), callback),
+                     exc_info=sys.exc_info())
 
     def getId(self):
         return self.id
@@ -858,7 +852,7 @@ class TransientObjectContainer(SimpleItem):
     # TransientItemContainer methods
 
     security.declareProtected(MANAGE_CONTAINER_PERM, 'setTimeoutMinutes')
-    def setTimeoutMinutes(self, timeout_mins, period_secs=20, reset=False):
+    def setTimeoutMinutes(self, timeout_mins, period_secs=20):
         """ The period_secs parameter is defaulted to preserve backwards API
         compatibility.  In older versions of this code, period was
         hardcoded to 20. """
@@ -869,10 +863,6 @@ class TransientObjectContainer(SimpleItem):
             # do nothing unless something has changed
             self._setTimeout(timeout_mins, period_secs)
             self._reset()
-        elif reset:
-            # this is to facilitate upgrades (services the "reset" button
-            # on the management page).
-            self._reset()
 
     def getTimeoutMinutes(self):
         """ """
@@ -944,17 +934,11 @@ class TransientObjectContainer(SimpleItem):
         'manage_changeTransientObjectContainer')
     def manage_changeTransientObjectContainer(
         self, title='', timeout_mins=20, addNotification=None,
-        delNotification=None, limit=0, period_secs=60, do_toc_reset=False,
-        REQUEST=None
+        delNotification=None, limit=0, period_secs=20, REQUEST=None
         ):
         """ Change an existing transient object container. """
-        if do_toc_reset is not False:
-            # cope preventatively with browser checkbox issues
-            if do_toc_reset in ('off', 'no', 'false', ''):
-                do_toc_reset = False
-
         self.title = title
-        self.setTimeoutMinutes(timeout_mins, period_secs, do_toc_reset)
+        self.setTimeoutMinutes(timeout_mins, period_secs)
         self.setSubobjectLimit(limit)
         if not addNotification:
             addNotification = None
@@ -969,8 +953,6 @@ class TransientObjectContainer(SimpleItem):
                 )
         
     def __setstate__(self, state):
-        SimpleItem.__setstate__(self, state)
-
         # upgrade versions of Transience in Zope versions less
         # than 2.7.1, which used a different transience mechanism.  Note:
         # this will not work for upgrading versions older than 2.6.0,
@@ -980,7 +962,7 @@ class TransientObjectContainer(SimpleItem):
         # f/w compat: 2.8 cannot use __len__ as an instance variable
         if not state.has_key('_length'):
             length = state.get('__len__', Length2())
-            state['_length'] = self.getLen = length
+            self._length = self.getLen = length
 
         oldlength = state['_length']
         if isinstance(oldlength, BTreesLength):
@@ -988,11 +970,11 @@ class TransientObjectContainer(SimpleItem):
             # the TOC length object, replace it with our own Length2
             # that does our conflict resolution correctly:
             sz = oldlength()
-            state['_length'] = self.getLen = Length2(sz)
+            self._length = self.getLen = Length2(sz)
 
         # TOCs prior to 2.7.1 took their period from a global
         if not state.has_key('_period'):
-            state['_period'] = 20 # this was the default for all prior releases
+            self._period = 20 # this was the default for all prior releases
 
         # TOCs prior to 2.7.1 used a different set of data structures
         # for efficiently keeping tabs on the maximum slice
