@@ -51,6 +51,7 @@ from Persistence import Persistent
 from BTrees.IOBTree import IOBTree
 from BTrees.OIBTree import OIBTree
 from BTrees.IIBTree import IIBTree
+import transaction
 
 from Products.ZCTextIndex.NBest import NBest
 from Products.ZCTextIndex.OkapiIndex import OkapiIndex
@@ -441,8 +442,6 @@ class Indexer:
             self.updatefolder(f, f.listmessages())
             print "Total", len(self.docpaths)
         self.commit()
-        print "Indexed", self.index.lexicon._nbytes, "bytes and",
-        print self.index.lexicon._nwords, "words;",
         print len(self.index.lexicon._words), "unique words."
 
     def updatefolder(self, f, msgs):
@@ -468,7 +467,7 @@ class Indexer:
             self.index.index_text(docid, text)
             self.maycommit()
         # Remove messages from the folder that no longer exist
-        for path in self.path2docid.keys(f.name):
+        for path in list(self.path2docid.keys(f.name)):
             if not path.startswith(f.name + "/"):
                 break
             if self.getmtime(path) == 0:
@@ -544,7 +543,7 @@ class Indexer:
             st = os.stat(path)
         except os.error, msg:
             return 0
-        return st[ST_MTIME]
+        return int(st[ST_MTIME])
 
     def maycommit(self):
         self.trans_count += 1
@@ -554,7 +553,7 @@ class Indexer:
     def commit(self):
         if self.trans_count > 0:
             print "committing..."
-            get_transaction().commit()
+            transaction.commit()
             self.trans_count = 0
             self.pack_count += 1
             if self.pack_count >= self.pack_limit > 0:
