@@ -17,9 +17,9 @@ __version__='$Revision: 1.2 $'[11:-2]
 
 import sys, os
 import signal
-import logging
+from logging import getLogger
 
-LOG = logging.getLogger('Signals') 
+LOG = getLogger('SignalHandler') 
 
 class SignalHandler:
 
@@ -40,7 +40,6 @@ class SignalHandler:
             signal.signal(signum, self.signalHandler)
             signame = get_signal_name(signum)
             LOG.debug("Installed sighandler for %s" % signame)
-
         items.insert(0, handler)
 
     def getRegisteredSignals(self):
@@ -61,7 +60,8 @@ class SignalHandler:
                 # if we trap SystemExit, we can't restart
                 raise
             except:
-                LOG.warn('A handler for %s failed!' % signame, exc_info=sys.exc_info())
+                LOG.warn('A handler for %s failed!' % signame,
+                         exc_info=sys.exc_info())
 
 _signals = None
 
