@@ -1,9 +1,9 @@
 ##############################################################################
 #
-# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
+# Copyright (c) 2002 Zope Corporation and Contributors. All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
@@ -12,8 +12,8 @@
 ##############################################################################
 """Rendering object hierarchies as Trees
 """
-__rcs_id__='$Id: TreeTag.py,v 1.53.68.5 2004/04/29 18:58:56 caseman Exp $'
-__version__='$Revision: 1.53.68.5 $'[11:-2]
+__rcs_id__='$Id: TreeTag.py 40218 2005-11-18 14:39:19Z andreasjung $'
+__version__='$Revision: 1.58 $'[11:-2]
 
 from DocumentTemplate.DT_Util import *
 from DocumentTemplate.DT_String import String
@@ -584,10 +584,8 @@ def decode_seq(state):
         state=a2b_base64(state)
 
     state=decompress(state)
-    try:
-        return list(MiniUnpickler(StringIO(state)).load())
-    except:
-        return []
+    try: return list(MiniUnpickler(StringIO(state)).load())
+    except: return []
 
 def decompress(input,max_size=10240):
     # This sillyness can go away in python 2.2
@@ -673,7 +671,7 @@ class MiniUnpickler(pickle.Unpickler):
     dispatch = pickle.Unpickler.dispatch.copy()
 
     for k,v in dispatch.items():
-        if k=='' or k in '().012FGIJKLMNSTUVX]adeghjlpqrstu}':
+        if k=='' or k in '().012FGIJKLMNTUVXS]adeghjlpqrstu}':
             # This key is necessary and safe, so leave it in the map
             pass
         else:
@@ -689,12 +687,12 @@ class MiniUnpickler(pickle.Unpickler):
     del v
 
 def _should_succeed(x,binary=1):
-    if x != MiniUnpickler(StringIO(dumps(x,binary))).load():
+    if x != MiniUnpickler(StringIO(pickle.dumps(x,binary))).load():
         raise ValueError(x)
 
 def _should_fail(x,binary=1):
     try:
-        MiniUnpickler(StringIO(dumps(x,binary))).load()
+        MiniUnpickler(StringIO(pickle.dumps(x,binary))).load()
         raise ValueError(x)
     except pickle.UnpicklingError, e:
         if e[0]!='Refused': raise ValueError(x)
@@ -702,7 +700,6 @@ def _should_fail(x,binary=1):
 class _junk_class: pass
 
 def _test():
-    _should_succeed('hello', binary=0)
     _should_succeed('hello')
     _should_succeed(1)
     _should_succeed(1L)
@@ -715,5 +712,4 @@ def _test():
     _should_fail(_junk_class())
 
 # Test MiniPickle on every import
-# XXX This *really* should be moved to a unittest!
 _test()
