@@ -1,9 +1,9 @@
 ##############################################################################
 #
-# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
+# Copyright (c) 2002 Zope Corporation and Contributors. All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
@@ -13,9 +13,16 @@
 """Provide management of common instance property sheets
 """
 
+import transaction
+
 import OFS.PropertySheets, Globals, OFS.SimpleItem, OFS.PropertyManager
 import Acquisition
+from Globals import InitializeClass
+from AccessControl import ClassSecurityInfo
 from AccessControl.Permission import pname
+from AccessControl.Permissions import manage_zclasses
+from AccessControl.Permissions import manage_properties
+from AccessControl.Permissions import access_contents_information
 
 class ClassCaretaker:
     def __init__(self, klass): self.__dict__['_k']=klass
@@ -23,14 +30,14 @@ class ClassCaretaker:
     def __setattr__(self, name, v):
         klass=self._k
         setattr(klass, name, v)
-        if not getattr(klass,'_p_changed',None):
-            get_transaction().register(klass)
+        if not getattr(klass,'_p_changed',None) and klass._p_jar is not None:
+            transaction.get().register(klass)
             klass._p_changed=1
     def __delattr__(self, name):
         klass=self._k
         delattr(klass, name)
-        if not getattr(klass,'_p_changed',None):
-            get_transaction().register(klass)
+        if not getattr(klass,'_p_changed',None) and klass._p_jar is not None:
+            transaction.get().register(klass)
             klass._p_changed=1
 
 
@@ -46,9 +53,10 @@ class ZCommonSheet(OFS.PropertySheets.PropertySheet, OFS.SimpleItem.Item):
          'help':('OFSP','Security_Define-Permissions.stx')},
         )
 
-    __ac_permissions__=(
-        ('Manage Z Classes', ('', 'manage')),
-        )
+    security = ClassSecurityInfo()
+    security.declareObjectProtected(manage_zclasses)
+
+    security.declareProtected(manage_zclasses, 'manage')
 
     def __init__(self, id, title):
         self.id=id
@@ -236,7 +244,8 @@ class ZCommonSheet(OFS.PropertySheets.PropertySheet, OFS.SimpleItem.Item):
                 self, REQUEST,
                 manage_tabs_message='The permission mapping has been updated')
 
-Globals.default__class_init__(ZCommonSheet)
+InitializeClass(ZCommonSheet)
+
 
 property_sheet_permissions=(
     # 'Access contents information',
@@ -248,30 +257,32 @@ class ZInstanceSheet(OFS.PropertySheets.FixedSchema,
                     ):
     "Waaa this is too hard"
 
+    security = ClassSecurityInfo()
+    security.declareObjectProtected(access_contents_information)
+
+    security.declareProtected(access_contents_information, 'hasProperty')
+    security.declareProtected(access_contents_information, 'propertyIds')
+    security.declareProtected(access_contents_information, 'propertyValues')
+    security.declareProtected(access_contents_information, 'propertyItems')
+    security.declareProtected(access_contents_information, 'propertyMap')
+    security.declareProtected(manage_properties, 'manage')
+    security.declareProtected(manage_properties, 'manage_addProperty')
+    security.declareProtected(manage_properties, 'manage_editProperties')
+    security.declareProtected(manage_properties, 'manage_delProperties')
+    security.declareProtected(manage_properties, 'manage_changeProperties')
+
     _Manage_properties_Permission='_Manage_properties_Permission'
     _Access_contents_information_Permission='_View_Permission'
 
-    __ac_permissions__=(
-        ('Manage properties', ('manage_addProperty',
-                               'manage_editProperties',
-                               'manage_delProperties',
-                               'manage_changeProperties',
-                               'manage',
-                               )),
-        ('Access contents information', ('hasProperty', 'propertyIds',
-                                         'propertyValues','propertyItems',
-                                         'propertyMap', ''),
-         ),
-        )
-
     def v_self(self):
         return self.aq_inner.aq_parent.aq_parent
 
-Globals.default__class_init__(ZInstanceSheet)
+InitializeClass(ZInstanceSheet)
+
 
 def rclass(klass):
-    if not getattr(klass, '_p_changed', 0):
-        get_transaction().register(klass)
+    if not getattr(klass, '_p_changed', 0) and klass._p_jar is not None:
+        transaction.get().register(klass)
         klass._p_changed=1
 
 class ZInstanceSheetsSheet(OFS.Traversable.Traversable,
@@ -346,5 +357,4 @@ class ZInstanceSheets(OFS.PropertySheets.PropertySheets, Globals.Persistent):
             r.append(getattr(self, id))
         return propsets+tuple(r)
 
-
-Globals.default__class_init__(ZInstanceSheets)
+InitializeClass(ZInstanceSheets)
