@@ -14,9 +14,15 @@ needed by a product and to define product meta data.
 
 This sample product publishes a folder-ish and a simple object.
 
-$Id: __init__.py,v 1.7 2002/08/14 22:03:39 mj Exp $'''
+$Id: __init__.py 40977 2005-12-22 11:13:46Z andreasjung $'''
 __version__='$Revision: 1.7 $'[11:-2]
 
+
+import warnings
+warnings.warn('ZClasses are deprecated, unmaintained and should no longer be used',
+              DeprecationWarning,
+              stacklevel=2) 
+
 import ZClass
 import ZClassOwner
 
