@@ -4,7 +4,7 @@
 # All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
@@ -25,20 +25,25 @@ import tempfile
 import threading
 import time
 import types
+import logging
 
-from ZEO import ClientCache, ServerStub
+from ZEO import ServerStub
+from ZEO.cache import ClientCache
 from ZEO.TransactionBuffer import TransactionBuffer
-from ZEO.Exceptions import ClientStorageError, UnrecognizedResult, \
-     ClientDisconnected, AuthError
+from ZEO.Exceptions import ClientStorageError, ClientDisconnected, AuthError
 from ZEO.auth import get_module
 from ZEO.zrpc.client import ConnectionManager
 
 from ZODB import POSException
-from ZODB.TimeStamp import TimeStamp
-from zLOG import LOG, PROBLEM, INFO, BLATHER, ERROR
+from ZODB.loglevels import BLATHER
+from persistent.TimeStamp import TimeStamp
 
-def log2(type, msg, subsys="ZCS:%d" % os.getpid()):
-    LOG(subsys, type, msg)
+logger = logging.getLogger('ZEO.ClientStorage')
+_pid = str(os.getpid())
+
+def log2(msg, level=logging.INFO, subsys=_pid, exc_info=False):
+    message = "(%s) %s" % (subsys, msg)
+    logger.log(level, message, exc_info=exc_info)
 
 try:
     from ZODB.ConflictResolution import ResolvedSerial
@@ -56,8 +61,7 @@ def get_timestamp(prev_ts=None):
     the argument.
     """
     t = time.time()
-    t = time.gmtime(t)[:5] + (t % 60,)
-    t = TimeStamp(*t)
+    t = TimeStamp(*time.gmtime(t)[:5] + (t % 60,))
     if prev_ts is not None:
         t = t.laterThan(prev_ts)
     return t
@@ -92,7 +96,7 @@ class ClientStorage(object):
     # Classes we instantiate.  A subclass might override.
 
     TransactionBufferClass = TransactionBuffer
-    ClientCacheClass = ClientCache.ClientCache
+    ClientCacheClass = ClientCache
     ConnectionManagerClass = ConnectionManager
     StorageServerStubClass = ServerStub.StorageServer
 
@@ -178,7 +182,7 @@ class ClientStorage(object):
         testConnection() and doAuth() for details).
         """
 
-        log2(INFO, "%s (pid=%d) created %s/%s for storage: %r" %
+        log2("%s (pid=%d) created %s/%s for storage: %r" %
              (self.__class__.__name__,
               os.getpid(),
               read_only and "RO" or "RW",
@@ -186,18 +190,17 @@ class ClientStorage(object):
               storage))
 
         if debug:
-            log2(INFO, "ClientStorage(): debug argument is no longer used")
+            log2("ClientStorage(): debug argument is no longer used")
 
         # wait defaults to True, but wait_for_server_on_startup overrides
         # if not None
         if wait_for_server_on_startup is not None:
             if wait is not None and wait != wait_for_server_on_startup:
-                log2(PROBLEM,
-                     "ClientStorage(): conflicting values for wait and "
-                     "wait_for_server_on_startup; wait prevails")
+                log2("ClientStorage(): conflicting values for wait and "
+                     "wait_for_server_on_startup; wait prevails",
+                     level=logging.WARNING)
             else:
-                log2(INFO,
-                     "ClientStorage(): wait_for_server_on_startup "
+                log2("ClientStorage(): wait_for_server_on_startup "
                      "is deprecated; please use wait instead")
                 wait = wait_for_server_on_startup
         elif wait is None:
@@ -253,10 +256,17 @@ class ClientStorage(object):
 
         self._tbuf = self.TransactionBufferClass()
         self._db = None
+        self._ltid = None # the last committed transaction
 
         # _serials: stores (oid, serialno) as returned by server
         # _seriald: _check_serials() moves from _serials to _seriald,
         #           which maps oid to serialno
+
+        # TODO:  If serial number matches transaction id, then there is
+        # no need to have all this extra infrastructure for handling
+        # serial numbers.  The vote call can just return the tid.
+        # If there is a conflict error, we can't have a special method
+        # called just to propagate the error.
         self._serials = []
         self._seriald = {}
 
@@ -293,13 +303,15 @@ class ClientStorage(object):
         # is executing.
         self._lock = threading.Lock()
 
-        t = self._ts = get_timestamp()
-        self._serial = `t`
-        self._oid = '\0\0\0\0\0\0\0\0'
-
         # Decide whether to use non-temporary files
-        self._cache = self.ClientCacheClass(storage, cache_size,
-                                            client=client, var=var)
+        if client is not None:
+            dir = var or os.getcwd()
+            cache_path = os.path.join(dir, "%s-%s.zec" % (client, storage))
+        else:
+            cache_path = None
+        self._cache = self.ClientCacheClass(cache_path, size=cache_size)
+        # TODO:  maybe there's a better time to open the cache?  Unclear.
+        self._cache.open()
 
         self._rpc_mgr = self.ConnectionManagerClass(addr, self,
                                                     tmin=min_disconnect_poll,
@@ -313,14 +325,11 @@ class ClientStorage(object):
             # doesn't succeed, call connect() to start a thread.
             if not self._rpc_mgr.attempt_connect():
                 self._rpc_mgr.connect()
-            # If the connect hasn't occurred, run with cached data.
-            if not self._ready.isSet():
-                self._cache.open()
 
     def _wait(self, timeout=None):
         if timeout is not None:
             deadline = time.time() + timeout
-            log2(BLATHER, "Setting deadline to %f" % deadline)
+            log2("Setting deadline to %f" % deadline, level=BLATHER)
         else:
             deadline = None
         # Wait for a connection to be established.
@@ -330,34 +339,16 @@ class ClientStorage(object):
         # still be going on.  This code must wait until validation
         # finishes, but if the connection isn't a zrpc async
         # connection it also needs to poll for input.
-        if self._connection.is_async():
-            while 1:
-                self._ready.wait(30)
-                if self._ready.isSet():
-                    break
-                if timeout and time.time() > deadline:
-                    log2(PROBLEM, "Timed out waiting for connection")
-                    break
-                log2(INFO, "Waiting for cache verification to finish")
-        else:
-            self._wait_sync(deadline)
-
-    def _wait_sync(self, deadline=None):
-        # If there is no mainloop running, this code needs
-        # to call poll() to cause asyncore to handle events.
+        assert self._connection.is_async()
         while 1:
+            self._ready.wait(30)
             if self._ready.isSet():
                 break
-            if deadline and time.time() > deadline:
-                log2(PROBLEM, "Timed out waiting for connection")
+            if timeout and time.time() > deadline:
+                log2("Timed out waiting for connection",
+                     level=logging.WARNING)
                 break
-            log2(INFO, "Waiting for cache verification to finish")
-            if self._connection is None:
-                # If the connection was closed while we were
-                # waiting for it to become ready, start over.
-                return self._wait(deadline - time.time())
-            else:
-                self._connection.pending(30)
+            log2("Waiting for cache verification to finish")
 
     def close(self):
         """Storage API: finalize the storage, releasing external resources."""
@@ -385,35 +376,25 @@ class ClientStorage(object):
         return self._ready.isSet()
 
     def sync(self):
-        """Handle any pending invalidation messages.
-
-        This is called by the sync method in ZODB.Connection.
-        """
-        # If there is no connection, return immediately.  Technically,
-        # there are no pending invalidations so they are all handled.
-        # There doesn't seem to be much benefit to raising an exception.
-
-        cn = self._connection
-        if cn is not None:
-            cn.pending()
+        # The separate async thread should keep us up to date
+        pass
 
     def doAuth(self, protocol, stub):
         if not (self._username and self._password):
-            raise AuthError, "empty username or password"
+            raise AuthError("empty username or password")
 
         module = get_module(protocol)
         if not module:
-            log2(PROBLEM, "%s: no such an auth protocol: %s" %
-                 (self.__class__.__name__, protocol))
+            log2("%s: no such an auth protocol: %s" %
+                 (self.__class__.__name__, protocol), level=logging.WARNING)
             return
 
         storage_class, client, db_class = module
 
         if not client:
-            log2(PROBLEM,
-                 "%s: %s isn't a valid protocol, must have a Client class" %
-                 (self.__class__.__name__, protocol))
-            raise AuthError, "invalid protocol"
+            log2("%s: %s isn't a valid protocol, must have a Client class" %
+                 (self.__class__.__name__, protocol), level=logging.WARNING)
+            raise AuthError("invalid protocol")
 
         c = client(stub)
 
@@ -441,21 +422,21 @@ class ClientStorage(object):
         succeeding register() call is deemed an optimal match, and any
         exception raised by register() is passed through.
         """
-        log2(INFO, "Testing connection %r" % conn)
-        # XXX Check the protocol version here?
+        log2("Testing connection %r" % conn)
+        # TODO:  Should we check the protocol version here?
         self._conn_is_read_only = 0
         stub = self.StorageServerStubClass(conn)
 
         auth = stub.getAuthProtocol()
-        log2(INFO, "Server authentication protocol %r" % auth)
+        log2("Server authentication protocol %r" % auth)
         if auth:
             skey = self.doAuth(auth, stub)
             if skey:
-                log2(INFO, "Client authentication successful")
+                log2("Client authentication successful")
                 conn.setSessionKey(skey)
             else:
-                log2(ERROR, "Authentication failed")
-                raise AuthError, "Authentication failed"
+                log2("Authentication failed")
+                raise AuthError("Authentication failed")
 
         try:
             stub.register(str(self._storage), self._is_read_only)
@@ -463,7 +444,7 @@ class ClientStorage(object):
         except POSException.ReadOnlyError:
             if not self._read_only_fallback:
                 raise
-            log2(INFO, "Got ReadOnlyError; trying again with read_only=1")
+            log2("Got ReadOnlyError; trying again with read_only=1")
             stub.register(str(self._storage), read_only=1)
             self._conn_is_read_only = 1
             return 0
@@ -479,7 +460,11 @@ class ClientStorage(object):
             # this method before it was stopped.
             return
 
-        # XXX would like to report whether we get a read-only connection
+        # invalidate our db cache
+        if self._db is not None:
+            self._db.invalidateCache()
+
+        # TODO:  report whether we get a read-only connection.
         if self._connection is not None:
             reconnect = 1
         else:
@@ -494,17 +479,23 @@ class ClientStorage(object):
         self._connection = conn
 
         if reconnect:
-            log2(INFO, "Reconnected to storage: %s" % self._server_addr)
+            log2("Reconnected to storage: %s" % self._server_addr)
         else:
-            log2(INFO, "Connected to storage: %s" % self._server_addr)
+            log2("Connected to storage: %s" % self._server_addr)
 
         stub = self.StorageServerStubClass(conn)
         self._oids = []
-        self._info.update(stub.get_info())
         self.verify_cache(stub)
-        if not conn.is_async():
-            log2(INFO, "Waiting for cache verification to finish")
-            self._wait_sync()
+
+        # It's important to call get_info after calling verify_cache.
+        # If we end up doing a full-verification, we need to wait till
+        # it's done.  By doing a synchonous call, we are guarenteed
+        # that the verification will be done because operations are
+        # handled in order.        
+        self._info.update(stub.get_info())
+
+        assert conn.is_async()
+
         self._handle_extensions()
 
     def _handle_extensions(self):
@@ -527,7 +518,8 @@ class ClientStorage(object):
             try:
                 canonical, aliases, addrs = socket.gethostbyaddr(host)
             except socket.error, err:
-                log2(BLATHER, "Error resolving host: %s (%s)" % (host, err))
+                log2("Error resolving host: %s (%s)" % (host, err),
+                     level=BLATHER)
                 canonical = host
             self._server_addr = str((canonical, addr[1]))
 
@@ -554,35 +546,35 @@ class ClientStorage(object):
         if last_inval_tid is not None:
             ltid = server.lastTransaction()
             if ltid == last_inval_tid:
-                log2(INFO, "No verification necessary "
-                     "(last_inval_tid up-to-date)")
-                self._cache.open()
+                log2("No verification necessary (last_inval_tid up-to-date)")
                 self._server = server
                 self._ready.set()
                 return "no verification"
 
             # log some hints about last transaction
-            log2(INFO, "last inval tid: %r %s\n"
+            log2("last inval tid: %r %s\n"
                  % (last_inval_tid, tid2time(last_inval_tid)))
-            log2(INFO, "last transaction: %r %s" %
+            log2("last transaction: %r %s" %
                  (ltid, ltid and tid2time(ltid)))
 
             pair = server.getInvalidations(last_inval_tid)
             if pair is not None:
-                log2(INFO, "Recovering %d invalidations" % len(pair[1]))
-                self._cache.open()
+                log2("Recovering %d invalidations" % len(pair[1]))
                 self.invalidateTransaction(*pair)
                 self._server = server
                 self._ready.set()
                 return "quick verification"
 
-        log2(INFO, "Verifying cache")
+        log2("Verifying cache")
         # setup tempfile to hold zeoVerify results
         self._tfile = tempfile.TemporaryFile(suffix=".inv")
         self._pickler = cPickle.Pickler(self._tfile, 1)
         self._pickler.fast = 1 # Don't use the memo
 
-        self._cache.verify(server.zeoVerify)
+        # TODO:  should batch these operations for efficiency; would need
+        # to acquire lock ...
+        for oid, tid, version in self._cache.contents():
+            server.verify(oid, version, tid)
         self._pending_server = server
         server.endZeoVerify()
         return "full verification"
@@ -601,8 +593,7 @@ class ClientStorage(object):
         This is called by ConnectionManager when the connection is
         closed or when certain problems with the connection occur.
         """
-        log2(PROBLEM, "Disconnected from storage: %s"
-             % repr(self._server_addr))
+        log2("Disconnected from storage: %s" % repr(self._server_addr))
         self._connection = None
         self._ready.clear()
         self._server = disconnected_stub
@@ -610,7 +601,7 @@ class ClientStorage(object):
 
     def __len__(self):
         """Return the size of the storage."""
-        # XXX Where is this used?
+        # TODO:  Is this method used?
         return self._info['length']
 
     def getName(self):
@@ -672,10 +663,10 @@ class ClientStorage(object):
             raise POSException.StorageTransactionError(self._transaction,
                                                        trans)
 
-    def abortVersion(self, version, transaction):
+    def abortVersion(self, version, txn):
         """Storage API: clear any changes made by the given version."""
-        self._check_trans(transaction)
-        oids = self._server.abortVersion(version, self._serial)
+        self._check_trans(txn)
+        tid, oids = self._server.abortVersion(version, id(txn))
         # When a version aborts, invalidate the version and
         # non-version data.  The non-version data should still be
         # valid, but older versions of ZODB will change the
@@ -683,25 +674,25 @@ class ClientStorage(object):
         # versions of ZODB, you'd get a conflict error if you tried to
         # commit a transaction with the cached data.
 
-        # XXX If we could guarantee that ZODB gave the right answer,
+        # If we could guarantee that ZODB gave the right answer,
         # we could just invalidate the version data.
         for oid in oids:
             self._tbuf.invalidate(oid, '')
-        return oids
+        return tid, oids
 
-    def commitVersion(self, source, destination, transaction):
+    def commitVersion(self, source, destination, txn):
         """Storage API: commit the source version in the destination."""
-        self._check_trans(transaction)
-        oids = self._server.commitVersion(source, destination, self._serial)
+        self._check_trans(txn)
+        tid, oids = self._server.commitVersion(source, destination, id(txn))
         if destination:
             # just invalidate our version data
             for oid in oids:
                 self._tbuf.invalidate(oid, source)
         else:
-            # destination is '', so invalidate version and non-version
+            # destination is "", so invalidate version and non-version
             for oid in oids:
-                self._tbuf.invalidate(oid, destination)
-        return oids
+                self._tbuf.invalidate(oid, "")
+        return tid, oids
 
     def history(self, oid, version, length=1):
         """Storage API: return a sequence of HistoryEntry objects.
@@ -711,6 +702,13 @@ class ClientStorage(object):
         """
         return self._server.history(oid, version, length)
 
+    def record_iternext(self, next=None):
+        """Storage API: get the mext database record.
+
+        This is part of the conversion-support API.
+        """
+        return self._server.record_iternext(next)
+
     def getSerial(self, oid):
         """Storage API: return current serial number for oid."""
         return self._server.getSerial(oid)
@@ -726,11 +724,14 @@ class ClientStorage(object):
         specified by the given object id and version, if they exist;
         otherwise a KeyError is raised.
         """
+        return self.loadEx(oid, version)[:2]
+
+    def loadEx(self, oid, version):
         self._lock.acquire()    # for atomic processing of invalidations
         try:
-            pair = self._cache.load(oid, version)
-            if pair:
-                return pair
+            t = self._cache.load(oid, version)
+            if t:
+                return t
         finally:
             self._lock.release()
 
@@ -746,25 +747,55 @@ class ClientStorage(object):
             finally:
                 self._lock.release()
 
-            p, s, v, pv, sv = self._server.zeoLoad(oid)
+            data, tid, ver = self._server.loadEx(oid, version)
 
             self._lock.acquire()    # for atomic processing of invalidations
             try:
                 if self._load_status:
-                    self._cache.checkSize(0)
-                    self._cache.store(oid, p, s, v, pv, sv)
+                    self._cache.store(oid, ver, tid, None, data)
                 self._load_oid = None
             finally:
                 self._lock.release()
         finally:
             self._load_lock.release()
 
-        if v and version and v == version:
-            return pv, sv
-        else:
-            if s:
-                return p, s
-            raise KeyError, oid # no non-version data for this
+        return data, tid, ver
+
+    def loadBefore(self, oid, tid):
+        self._lock.acquire()
+        try:
+            t = self._cache.loadBefore(oid, tid)
+            if t is not None:
+                return t
+        finally:
+            self._lock.release()
+
+        t = self._server.loadBefore(oid, tid)
+        if t is None:
+            return None
+        data, start, end = t
+        if end is None:
+            # This method should not be used to get current data.  It
+            # doesn't use the _load_lock, so it is possble to overlap
+            # this load with an invalidation for the same object.
+
+            # If we call again, we're guaranteed to get the
+            # post-invalidation data.  But if the data is still
+            # current, we'll still get end == None.
+
+            # Maybe the best thing to do is to re-run the test with
+            # the load lock in the case.  That's slow performance, but
+            # I don't think real application code will ever care about
+            # it.
+
+            return data, start, end
+        self._lock.acquire()
+        try:
+            self._cache.store(oid, "", start, end, data)
+        finally:
+            self._lock.release()
+
+        return data, start, end
 
     def modifiedInVersion(self, oid):
         """Storage API: return the version, if any, that modfied an object.
@@ -807,8 +838,8 @@ class ClientStorage(object):
         days -- a number of days to subtract from the pack time;
             defaults to zero.
         """
-        # XXX Is it okay that read-only connections allow pack()?
-        # rf argument ignored; server will provide it's own implementation
+        # TODO: Is it okay that read-only connections allow pack()?
+        # rf argument ignored; server will provide its own implementation
         if t is None:
             t = time.time()
         t = t - (days * 86400)
@@ -816,6 +847,8 @@ class ClientStorage(object):
 
     def _check_serials(self):
         """Internal helper to move data from _serials to _seriald."""
+        # serials are always going to be the same, the only
+        # question is whether an exception has been raised.
         if self._serials:
             l = len(self._serials)
             r = self._serials[:l]
@@ -826,18 +859,18 @@ class ClientStorage(object):
                 self._seriald[oid] = s
             return r
 
-    def store(self, oid, serial, data, version, transaction):
+    def store(self, oid, serial, data, version, txn):
         """Storage API: store data for an object."""
-        self._check_trans(transaction)
-        self._server.storea(oid, serial, data, version, self._serial)
+        self._check_trans(txn)
+        self._server.storea(oid, serial, data, version, id(txn))
         self._tbuf.store(oid, version, data)
         return self._check_serials()
 
-    def tpc_vote(self, transaction):
+    def tpc_vote(self, txn):
         """Storage API: vote on a transaction."""
-        if transaction is not self._transaction:
+        if txn is not self._transaction:
             return
-        self._server.vote(self._serial)
+        self._server.vote(id(txn))
         return self._check_serials()
 
     def tpc_begin(self, txn, tid=None, status=' '):
@@ -857,15 +890,8 @@ class ClientStorage(object):
         self._transaction = txn
         self._tpc_cond.release()
 
-        if tid is None:
-            self._ts = get_timestamp(self._ts)
-            id = `self._ts`
-        else:
-            self._ts = TimeStamp(tid)
-            id = tid
-
         try:
-            self._server.tpc_begin(id, txn.user, txn.description,
+            self._server.tpc_begin(id(txn), txn.user, txn.description,
                                    txn._extension, tid, status)
         except:
             # Client may have disconnected during the tpc_begin().
@@ -873,7 +899,6 @@ class ClientStorage(object):
                 self.end_transaction()
             raise
 
-        self._serial = id
         self._tbuf.clear()
         self._seriald.clear()
         del self._serials[:]
@@ -882,37 +907,37 @@ class ClientStorage(object):
         """Internal helper to end a transaction."""
         # the right way to set self._transaction to None
         # calls notify() on _tpc_cond in case there are waiting threads
-        self._ltid = self._serial
         self._tpc_cond.acquire()
         self._transaction = None
         self._tpc_cond.notify()
         self._tpc_cond.release()
 
     def lastTransaction(self):
-        return self._ltid
+        return self._cache.getLastTid()
 
-    def tpc_abort(self, transaction):
+    def tpc_abort(self, txn):
         """Storage API: abort a transaction."""
-        if transaction is not self._transaction:
+        if txn is not self._transaction:
             return
         try:
-            # XXX Are there any transactions that should prevent an
+            # Caution:  Are there any exceptions that should prevent an
             # abort from occurring?  It seems wrong to swallow them
             # all, yet you want to be sure that other abort logic is
             # executed regardless.
             try:
-                self._server.tpc_abort(self._serial)
+                self._server.tpc_abort(id(txn))
             except ClientDisconnected:
-                log2(BLATHER, 'ClientDisconnected in tpc_abort() ignored')
+                log2("ClientDisconnected in tpc_abort() ignored",
+                     level=BLATHER)
         finally:
             self._tbuf.clear()
             self._seriald.clear()
             del self._serials[:]
             self.end_transaction()
 
-    def tpc_finish(self, transaction, f=None):
+    def tpc_finish(self, txn, f=None):
         """Storage API: finish a transaction."""
-        if transaction is not self._transaction:
+        if txn is not self._transaction:
             return
         self._load_lock.acquire()
         try:
@@ -920,16 +945,18 @@ class ClientStorage(object):
                 raise ClientDisconnected(
                        'Calling tpc_finish() on a disconnected transaction')
 
-            tid = self._server.tpc_finish(self._serial)
-
+            # The calls to tpc_finish() and _update_cache() should
+            # never run currently with another thread, because the
+            # tpc_cond condition variable prevents more than one
+            # thread from calling tpc_finish() at a time.
+            tid = self._server.tpc_finish(id(txn))
             self._lock.acquire()  # for atomic processing of invalidations
             try:
-                self._update_cache()
+                self._update_cache(tid)
                 if f is not None:
-                    f()
+                    f(tid)
             finally:
                 self._lock.release()
-            self._cache.setLastTid(tid)
 
             r = self._check_serials()
             assert r is None or len(r) == 0, "unhandled serialnos: %s" % r
@@ -937,7 +964,7 @@ class ClientStorage(object):
             self._load_lock.release()
             self.end_transaction()
 
-    def _update_cache(self):
+    def _update_cache(self, tid):
         """Internal helper to handle objects modified by a transaction.
 
         This iterates over the objects in the transaction buffer and
@@ -945,39 +972,21 @@ class ClientStorage(object):
         """
         # Must be called with _lock already acquired.
 
-        # XXX not sure why _update_cache() would be called on
-        # a closed storage.
+        # Not sure why _update_cache() would be called on a closed storage.
         if self._cache is None:
             return
 
-        self._cache.checkSize(self._tbuf.get_size())
-        try:
-            self._tbuf.begin_iterate()
-        except ValueError, msg:
-            raise ClientStorageError, (
-                "Unexpected error reading temporary file in "
-                "client storage: %s" % msg)
-        while 1:
-            try:
-                t = self._tbuf.next()
-            except ValueError, msg:
-                raise ClientStorageError, (
-                    "Unexpected error reading temporary file in "
-                    "client storage: %s" % msg)
-            if t is None:
-                break
-            oid, v, p = t
-            if p is None: # an invalidation
-                s = None
-            else:
+        for oid, version, data in self._tbuf:
+            self._cache.invalidate(oid, version, tid)
+            # If data is None, we just invalidate.
+            if data is not None:
                 s = self._seriald[oid]
-            if s == ResolvedSerial or s is None:
-                self._cache.invalidate(oid, v)
-            else:
-                self._cache.update(oid, s, v, p)
+                if s != ResolvedSerial:
+                    assert s == tid, (s, tid)
+                    self._cache.store(oid, version, s, None, data)
         self._tbuf.clear()
 
-    def transactionalUndo(self, trans_id, trans):
+    def undo(self, trans_id, txn):
         """Storage API: undo a transaction.
 
         This is executed in a transactional context.  It has no effect
@@ -986,24 +995,11 @@ class ClientStorage(object):
         Zope uses this to implement undo unless it is not supported by
         a storage.
         """
-        self._check_trans(trans)
-        oids = self._server.transactionalUndo(trans_id, self._serial)
+        self._check_trans(txn)
+        tid, oids = self._server.undo(trans_id, id(txn))
         for oid in oids:
             self._tbuf.invalidate(oid, '')
-        return oids
-
-    def undo(self, transaction_id):
-        """Storage API: undo a transaction, writing directly to the storage."""
-        if self._is_read_only:
-            raise POSException.ReadOnlyError()
-        oids = self._server.undo(transaction_id)
-        self._lock.acquire()
-        try:
-            for oid in oids:
-                self._cache.invalidate(oid, '')
-        finally:
-            self._lock.release()
-        return oids
+        return tid, oids
 
     def undoInfo(self, first=0, last=-20, specification=None):
         """Storage API: return undo information."""
@@ -1047,7 +1043,8 @@ class ClientStorage(object):
         # Invalidation as result of verify_cache().
         # Queue an invalidate for the end the verification procedure.
         if self._pickler is None:
-            # XXX This should never happen
+            # This should never happen.  TODO:  assert it doesn't, or log
+            # if it does.
             return
         self._pickler.dump(args)
 
@@ -1060,15 +1057,15 @@ class ClientStorage(object):
         try:
             # versions maps version names to dictionary of invalidations
             versions = {}
-            for oid, version in invs:
+            for oid, version, tid in invs:
                 if oid == self._load_oid:
                     self._load_status = 0
-                self._cache.invalidate(oid, version=version)
-                versions.setdefault(version, {})[oid] = 1
+                self._cache.invalidate(oid, version, tid)
+                versions.setdefault((version, tid), {})[oid] = tid
 
             if self._db is not None:
-                for v, d in versions.items():
-                    self._db.invalidate(d, version=v)
+                for (version, tid), d in versions.items():
+                    self._db.invalidate(tid, d, version=version)
         finally:
             self._lock.release()
 
@@ -1085,22 +1082,27 @@ class ClientStorage(object):
         self._process_invalidations(InvalidationLogIterator(f))
         f.close()
 
-        log2(INFO, "endVerify finishing")
+        log2("endVerify finishing")
         self._server = self._pending_server
         self._ready.set()
         self._pending_conn = None
-        log2(INFO, "endVerify finished")
+        log2("endVerify finished")
 
     def invalidateTransaction(self, tid, args):
         """Invalidate objects modified by tid."""
-        self._cache.setLastTid(tid)
+        self._lock.acquire()
+        try:
+            self._cache.setLastTid(tid)
+        finally:
+            self._lock.release()
         if self._pickler is not None:
-            log2(BLATHER,
-                 "Transactional invalidation during cache verification")
+            log2("Transactional invalidation during cache verification",
+                 level=BLATHER)
             for t in args:
                 self._pickler.dump(t)
             return
-        self._process_invalidations(args)
+        self._process_invalidations([(oid, version, tid)
+                                     for oid, version in args])
 
     # The following are for compatibility with protocol version 2.0.0
 
@@ -1111,36 +1113,10 @@ class ClientStorage(object):
     end = endVerify
     Invalidate = invalidateTrans
 
-try:
-    StopIteration
-except NameError:
-    class StopIteration(Exception):
-        pass
-
-class InvalidationLogIterator:
-    """Helper class for reading invalidations in endVerify."""
-
-    def __init__(self, fileobj):
-        self._unpickler = cPickle.Unpickler(fileobj)
-        self.getitem_i = 0
-
-    def __iter__(self):
-        return self
-
-    def next(self):
-        oid, version = self._unpickler.load()
+def InvalidationLogIterator(fileobj):
+    unpickler = cPickle.Unpickler(fileobj)
+    while 1:
+        oid, version = unpickler.load()
         if oid is None:
-            raise StopIteration
-        return oid, version
-
-    # The __getitem__() method is needed to support iteration
-    # in Python 2.1.
-
-    def __getitem__(self, i):
-        assert i == self.getitem_i
-        try:
-            obj = self.next()
-        except StopIteration:
-            raise IndexError, i
-        self.getitem_i += 1
-        return obj
+            break
+        yield oid, version, None
