@@ -4,7 +4,7 @@
 # All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
@@ -25,16 +25,18 @@ try:
 except ImportError:
     from StringIO import StringIO
 
-import threading
 import time
 
+from persistent import Persistent
+from persistent.mapping import PersistentMapping
+import transaction
 from ZODB import DB
-from Persistence import Persistent
-from ZODB.referencesf import referencesf
+from ZODB.serialize import referencesf
 from ZODB.tests.MinPO import MinPO
 from ZODB.tests.StorageTestBase import snooze
 from ZODB.POSException import ConflictError, StorageError
-from ZODB.PersistentMapping import PersistentMapping
+
+from ZODB.tests.MTStorage import TestThread
 
 ZERO = '\0'*8
 
@@ -84,9 +86,16 @@ def dumps(obj):
     p = pickle.Pickler(s)
     p.persistent_id = getpersid
     p.dump(obj)
+    p.dump(None)
     return s.getvalue()
 
-
+def pdumps(obj):
+    s = StringIO()
+    p = pickle.Pickler(s)
+    p.dump(obj)
+    p.dump(None)
+    return s.getvalue()
+    
 
 class PackableStorageBase:
     # We keep a cache of object ids to instances so that the unpickler can
@@ -122,18 +131,14 @@ class PackableStorageBase:
             return u.load()
         return loads
 
-
-
-class PackableStorage(PackableStorageBase):
     def _initroot(self):
         try:
             self._storage.load(ZERO, '')
         except KeyError:
-            import PersistentMapping
-            from ZODB.Transaction import Transaction
+            from transaction import Transaction
             file = StringIO()
             p = cPickle.Pickler(file, 1)
-            p.dump((PersistentMapping.PersistentMapping, None))
+            p.dump((PersistentMapping, None))
             p.dump({'_container': {}})
             t=Transaction()
             t.description='initial database creation'
@@ -142,6 +147,8 @@ class PackableStorage(PackableStorageBase):
             self._storage.tpc_vote(t)
             self._storage.tpc_finish(t)
 
+class PackableStorage(PackableStorageBase):
+
     def checkPackEmptyStorage(self):
         self._storage.pack(time.time(), referencesf)
 
@@ -153,6 +160,159 @@ class PackableStorage(PackableStorageBase):
         self._initroot()
         self._storage.pack(time.time() - 10000, referencesf)
 
+    def _PackWhileWriting(self, pack_now):
+        # A storage should allow some reading and writing during
+        # a pack.  This test attempts to exercise locking code
+        # in the storage to test that it is safe.  It generates
+        # a lot of revisions, so that pack takes a long time.
+
+        db = DB(self._storage)
+        conn = db.open()
+        root = conn.root()
+
+        for i in range(10):
+            root[i] = MinPO(i)
+        transaction.commit()
+
+        snooze()
+        packt = time.time()
+
+        choices = range(10)
+        for dummy in choices:
+            for i in choices:
+                root[i].value = MinPO(i)
+                transaction.commit()
+
+        # How many client threads should we run, and how long should we
+        # wait for them to finish?  Hard to say.  Running 4 threads and
+        # waiting 30 seconds too often left a thread still alive on Tim's
+        # Win98SE box, during ZEO flavors of this test.  Those tend to
+        # run one thread at a time to completion, and take about 10 seconds
+        # per thread.  There doesn't appear to be a compelling reason to
+        # run that many threads.  Running 3 threads and waiting up to a
+        # minute seems to work well in practice.  The ZEO tests normally
+        # finish faster than that, and the non-ZEO tests very much faster
+        # than that.
+        NUM_LOOP_TRIP = 50
+        timer = ElapsedTimer(time.time())
+        threads = [ClientThread(db, choices, NUM_LOOP_TRIP, timer, i)
+                   for i in range(3)]
+        for t in threads:
+            t.start()
+
+        if pack_now:
+            db.pack(time.time())
+        else:
+            db.pack(packt)
+
+        for t in threads:
+            t.join(60)
+        liveness = [t.isAlive() for t in threads]
+        if True in liveness:
+            # They should have finished by now.
+            print 'Liveness:', liveness
+            # Combine the outcomes, and sort by start time.
+            outcomes = []
+            for t in threads:
+                outcomes.extend(t.outcomes)
+            # each outcome list has as many of these as a loop trip got thru:
+            #     thread_id
+            #     elapsed millis at loop top
+            #     elapsed millis at attempt to assign to self.root[index]
+            #     index into self.root getting replaced
+            #     elapsed millis when outcome known
+            #     'OK' or 'Conflict'
+            #     True if we got beyond this line, False if it raised an
+            #         exception (one possible Conflict cause):
+            #             self.root[index].value = MinPO(j)
+            def cmp_by_time(a, b):
+                return cmp((a[1], a[0]), (b[1], b[0]))
+            outcomes.sort(cmp_by_time)
+            counts = [0] * 4
+            for outcome in outcomes:
+                n = len(outcome)
+                assert n >= 2
+                tid = outcome[0]
+                print 'tid:%d top:%5d' % (tid, outcome[1]),
+                if n > 2:
+                    print 'commit:%5d' % outcome[2],
+                    if n > 3:
+                        print 'index:%2d' % outcome[3],
+                        if n > 4:
+                            print 'known:%5d' % outcome[4],
+                            if n > 5:
+                                print '%8s' % outcome[5],
+                                if n > 6:
+                                    print 'assigned:%5s' % outcome[6],
+                counts[tid] += 1
+                if counts[tid] == NUM_LOOP_TRIP:
+                    print 'thread %d done' % tid,
+                print
+
+            self.fail('a thread is still alive')
+
+        # Iterate over the storage to make sure it's sane, but not every
+        # storage supports iterators.
+        if not hasattr(self._storage, "iterator"):
+            return
+
+        it = self._storage.iterator()
+        for txn in it:
+            for data in txn:
+                pass
+        it.close()
+
+    def checkPackWhileWriting(self):
+        self._PackWhileWriting(pack_now=False)
+
+    def checkPackNowWhileWriting(self):
+        self._PackWhileWriting(pack_now=True)
+
+    def checkPackLotsWhileWriting(self):
+        # This is like the other pack-while-writing tests, except it packs
+        # repeatedly until the client thread is done.  At the time it was
+        # introduced, it reliably provoked
+        #     CorruptedError:  ... transaction with checkpoint flag set
+        # in the ZEO flavor of the FileStorage tests.
+
+        db = DB(self._storage)
+        conn = db.open()
+        root = conn.root()
+
+        choices = range(10)
+        for i in choices:
+            root[i] = MinPO(i)
+        transaction.commit()
+
+        snooze()
+        packt = time.time()
+
+        for dummy in choices:
+            for i in choices:
+                root[i].value = MinPO(i)
+                transaction.commit()
+
+        NUM_LOOP_TRIP = 100
+        timer = ElapsedTimer(time.time())
+        thread = ClientThread(db, choices, NUM_LOOP_TRIP, timer, 0)
+        thread.start()
+        while thread.isAlive():
+            db.pack(packt)
+            snooze()
+            packt = time.time()
+        thread.join()
+
+        # Iterate over the storage to make sure it's sane.
+        if not hasattr(self._storage, "iterator"):
+            return
+        it = self._storage.iterator()
+        for txn in it:
+            for data in txn:
+                pass
+        it.close()
+
+class PackableUndoStorage(PackableStorageBase):
+
     def checkPackAllRevisions(self):
         self._initroot()
         eq = self.assertEqual
@@ -162,11 +322,11 @@ class PackableStorage(PackableStorageBase):
         oid = obj.getoid()
         obj.value = 1
         # Commit three different revisions
-        revid1 = self._dostoreNP(oid, data=pickle.dumps(obj))
+        revid1 = self._dostoreNP(oid, data=pdumps(obj))
         obj.value = 2
-        revid2 = self._dostoreNP(oid, revid=revid1, data=pickle.dumps(obj))
+        revid2 = self._dostoreNP(oid, revid=revid1, data=pdumps(obj))
         obj.value = 3
-        revid3 = self._dostoreNP(oid, revid=revid2, data=pickle.dumps(obj))
+        revid3 = self._dostoreNP(oid, revid=revid2, data=pdumps(obj))
         # Now make sure all three revisions can be extracted
         data = self._storage.loadSerial(oid, revid1)
         pobj = pickle.loads(data)
@@ -215,11 +375,11 @@ class PackableStorage(PackableStorageBase):
         eq(loads(data).value, 0)
         # Commit three different revisions of the other object
         obj.value = 1
-        revid1 = self._dostoreNP(oid, data=pickle.dumps(obj))
+        revid1 = self._dostoreNP(oid, data=pdumps(obj))
         obj.value = 2
-        revid2 = self._dostoreNP(oid, revid=revid1, data=pickle.dumps(obj))
+        revid2 = self._dostoreNP(oid, revid=revid1, data=pdumps(obj))
         obj.value = 3
-        revid3 = self._dostoreNP(oid, revid=revid2, data=pickle.dumps(obj))
+        revid3 = self._dostoreNP(oid, revid=revid2, data=pdumps(obj))
         # Now make sure all three revisions can be extracted
         data = self._storage.loadSerial(oid, revid1)
         pobj = pickle.loads(data)
@@ -285,11 +445,11 @@ class PackableStorage(PackableStorageBase):
         eq(loads(data).value, 0)
         # Commit three different revisions of the first object
         obj1.value = 1
-        revid1 = self._dostoreNP(oid1, data=pickle.dumps(obj1))
+        revid1 = self._dostoreNP(oid1, data=pdumps(obj1))
         obj1.value = 2
-        revid2 = self._dostoreNP(oid1, revid=revid1, data=pickle.dumps(obj1))
+        revid2 = self._dostoreNP(oid1, revid=revid1, data=pdumps(obj1))
         obj1.value = 3
-        revid3 = self._dostoreNP(oid1, revid=revid2, data=pickle.dumps(obj1))
+        revid3 = self._dostoreNP(oid1, revid=revid2, data=pdumps(obj1))
         # Now make sure all three revisions can be extracted
         data = self._storage.loadSerial(oid1, revid1)
         pobj = pickle.loads(data)
@@ -305,7 +465,7 @@ class PackableStorage(PackableStorageBase):
         eq(pobj.value, 3)
         # Now commit a revision of the second object
         obj2.value = 11
-        revid4 = self._dostoreNP(oid2, data=pickle.dumps(obj2))
+        revid4 = self._dostoreNP(oid2, data=pdumps(obj2))
         # And make sure the revision can be extracted
         data = self._storage.loadSerial(oid2, revid4)
         pobj = pickle.loads(data)
@@ -348,7 +508,7 @@ class PackableStorage(PackableStorageBase):
         conn = db.open()
         root = conn.root()
 
-        txn = get_transaction()
+        txn = transaction.get()
         txn.note('root')
         txn.commit()
 
@@ -360,12 +520,12 @@ class PackableStorage(PackableStorageBase):
         obj.value = 7
 
         root['obj'] = obj
-        txn = get_transaction()
+        txn = transaction.get()
         txn.note('root -> o1')
         txn.commit()
 
         del root['obj']
-        txn = get_transaction()
+        txn = transaction.get()
         txn.note('root -x-> o1')
         txn.commit()
 
@@ -374,7 +534,7 @@ class PackableStorage(PackableStorageBase):
         log = self._storage.undoLog()
         tid = log[0]['id']
         db.undo(tid)
-        txn = get_transaction()
+        txn = transaction.get()
         txn.note('undo root -x-> o1')
         txn.commit()
 
@@ -382,60 +542,6 @@ class PackableStorage(PackableStorageBase):
 
         eq(root['obj'].value, 7)
 
-    def _PackWhileWriting(self, pack_now=0):
-        # A storage should allow some reading and writing during
-        # a pack.  This test attempts to exercise locking code
-        # in the storage to test that it is safe.  It generates
-        # a lot of revisions, so that pack takes a long time.
-
-        db = DB(self._storage)
-        conn = db.open()
-        root = conn.root()
-
-        for i in range(10):
-            root[i] = MinPO(i)
-        get_transaction().commit()
-
-        snooze()
-        packt = time.time()
-
-        for j in range(10):
-            for i in range(10):
-                root[i].value = MinPO(i)
-                get_transaction().commit()
-
-        threads = [ClientThread(db) for i in range(4)]
-        for t in threads:
-            t.start()
-
-        if pack_now:
-            db.pack(time.time())
-        else:
-            db.pack(packt)
-
-        for t in threads:
-            t.join(30)
-        for t in threads:
-            t.join(1)
-            self.assert_(not t.isAlive())
-
-        # Iterate over the storage to make sure it's sane, but not every
-        # storage supports iterators.
-        if not hasattr(self._storage, "iterator"):
-            return
-
-        iter = self._storage.iterator()
-        for txn in iter:
-            for data in txn:
-                pass
-        iter.close()
-
-    def checkPackWhileWriting(self):
-        self._PackWhileWriting(pack_now=0)
-
-    def checkPackNowWhileWriting(self):
-        self._PackWhileWriting(pack_now=1)
-
     def checkRedundantPack(self):
         # It is an error to perform a pack with a packtime earlier
         # than a previous packtime.  The storage can't do a full
@@ -454,19 +560,19 @@ class PackableStorage(PackableStorageBase):
         root = conn.root()
 
         root["d"] = d = PersistentMapping()
-        get_transaction().commit()
+        transaction.commit()
         snooze()
 
         obj = d["obj"] = C()
         obj.value = 1
-        get_transaction().commit()
+        transaction.commit()
         snooze()
         packt1 = time.time()
         lost_oid = obj._p_oid
 
         obj = d["anotherobj"] = C()
         obj.value = 2
-        get_transaction().commit()
+        transaction.commit()
         snooze()
         packt2 = time.time()
 
@@ -482,21 +588,19 @@ class PackableStorage(PackableStorageBase):
 
     def checkPackUndoLog(self):
         self._initroot()
-        eq = self.assertEqual
-        raises = self.assertRaises
         # Create a `persistent' object
         obj = self._newobj()
         oid = obj.getoid()
         obj.value = 1
         # Commit two different revisions
-        revid1 = self._dostoreNP(oid, data=pickle.dumps(obj))
+        revid1 = self._dostoreNP(oid, data=pdumps(obj))
         obj.value = 2
         snooze()
         packtime = time.time()
         snooze()
-        revid2 = self._dostoreNP(oid, revid=revid1, data=pickle.dumps(obj))
+        self._dostoreNP(oid, revid=revid1, data=pdumps(obj))
         # Now pack the first transaction
-        self.assertEqual(3,len(self._storage.undoLog()))
+        self.assertEqual(3, len(self._storage.undoLog()))
         self._storage.pack(packtime, referencesf)
         # The undo log contains only the most resent transaction
         self.assertEqual(1,len(self._storage.undoLog()))
@@ -519,9 +623,9 @@ class PackableStorage(PackableStorageBase):
         obj2.value = 2
 
         # Commit the first revision of each of them
-        revid11 = self._dostoreNP(oid1, data=pickle.dumps(obj1),
+        revid11 = self._dostoreNP(oid1, data=pdumps(obj1),
                                   description="1-1")
-        revid22 = self._dostoreNP(oid2, data=pickle.dumps(obj2),
+        revid22 = self._dostoreNP(oid2, data=pdumps(obj2),
                                   description="2-2")
 
         # remember the time. everything above here will be packed away
@@ -531,14 +635,14 @@ class PackableStorage(PackableStorageBase):
         # Commit two revisions of the first object
         obj1.value = 3
         revid13 = self._dostoreNP(oid1, revid=revid11,
-                                  data=pickle.dumps(obj1), description="1-3")
+                                  data=pdumps(obj1), description="1-3")
         obj1.value = 4
-        revid14 = self._dostoreNP(oid1, revid=revid13,
-                                  data=pickle.dumps(obj1), description="1-4")
+        self._dostoreNP(oid1, revid=revid13,
+                        data=pdumps(obj1), description="1-4")
         # Commit one revision of the second object
         obj2.value = 5
-        revid25 = self._dostoreNP(oid2, revid=revid22,
-                                  data=pickle.dumps(obj2), description="2-5")
+        self._dostoreNP(oid2, revid=revid22,
+                        data=pdumps(obj2), description="2-5")
         # Now pack
         self.assertEqual(6,len(self._storage.undoLog()))
         print '\ninitial undoLog was'
@@ -549,16 +653,55 @@ class PackableStorage(PackableStorageBase):
         for r in self._storage.undoLog(): print r
         # what can we assert about that?
 
-class ClientThread(threading.Thread):
 
-    def __init__(self, db):
-        threading.Thread.__init__(self)
+# A number of these threads are kicked off by _PackWhileWriting().  Their
+# purpose is to abuse the database passed to the constructor with lots of
+# random write activity while the main thread is packing it.
+class ClientThread(TestThread):
+
+    def __init__(self, db, choices, loop_trip, timer, thread_id):
+        TestThread.__init__(self)
         self.root = db.open().root()
+        self.choices = choices
+        self.loop_trip = loop_trip
+        self.millis = timer.elapsed_millis
+        self.thread_id = thread_id
+        # list of lists; each list has as many of these as a loop trip
+        # got thru:
+        #     thread_id
+        #     elapsed millis at loop top
+        #     elapsed millis at attempt
+        #     index into self.root getting replaced
+        #     elapsed millis when outcome known
+        #     'OK' or 'Conflict'
+        #     True if we got beyond this line, False if it raised an exception:
+        #          self.root[index].value = MinPO(j)
+        self.outcomes = []
 
-    def run(self):
-        for j in range(50):
+    def runtest(self):
+        from random import choice
+
+        for j in range(self.loop_trip):
+            assign_worked = False
+            alist = [self.thread_id, self.millis()]
+            self.outcomes.append(alist)
             try:
-                self.root[j % 10].value = MinPO(j)
-                get_transaction().commit()
+                index = choice(self.choices)
+                alist.extend([self.millis(), index])
+                self.root[index].value = MinPO(j)
+                assign_worked = True
+                transaction.commit()
+                alist.append(self.millis())
+                alist.append('OK')
             except ConflictError:
-                get_transaction().abort()
+                alist.append(self.millis())
+                alist.append('Conflict')
+                transaction.abort()
+            alist.append(assign_worked)
+
+class ElapsedTimer:
+    def __init__(self, start_time):
+        self.start_time = start_time
+
+    def elapsed_millis(self):
+        return int((time.time() - self.start_time) * 1000)
