@@ -4,7 +4,7 @@
 # All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
@@ -16,15 +16,13 @@
 Any storage that supports versions should be able to pass all these tests.
 """
 
-# XXX we should clean this code up to get rid of the #JF# comments.
-# They were introduced when Jim reviewed the original version of the
-# code.  Barry and Jeremy didn't understand versions then.
-
 import time
 
+import transaction
+from transaction import Transaction
+
 from ZODB import POSException
-from ZODB.referencesf import referencesf
-from ZODB.Transaction import Transaction
+from ZODB.serialize import referencesf
 from ZODB.tests.MinPO import MinPO
 from ZODB.tests.StorageTestBase import zodb_unpickle, snooze
 from ZODB import DB
@@ -48,26 +46,33 @@ class VersionStorage:
         revid1 = self._dostore(oid, data=MinPO(12))
         revid2 = self._dostore(oid, revid=revid1, data=MinPO(13),
                                version="version")
+        data, tid, ver = self._storage.loadEx(oid, "version")
+        self.assertEqual(revid2, tid)
+        self.assertEqual(zodb_unpickle(data), MinPO(13))
         oids = self._abortVersion("version")
         self.assertEqual([oid], oids)
         data, revid3 = self._storage.load(oid, "")
         # use repr() to avoid getting binary data in a traceback on error
-        self.assertEqual(`revid1`, `revid3`)
-        self.assertNotEqual(`revid2`, `revid3`)
+        self.assertNotEqual(revid1, revid3)
+        self.assertNotEqual(revid2, revid3)
+        data, tid, ver = self._storage.loadEx(oid, "")
+        self.assertEqual(revid3, tid)
+        self.assertEqual(zodb_unpickle(data), MinPO(12))
+        self.assertEqual(tid, self._storage.lastTransaction())
 
     def checkVersionedStoreAndLoad(self):
         eq = self.assertEqual
         # Store a couple of non-version revisions of the object
         oid = self._storage.new_oid()
         revid = self._dostore(oid, data=MinPO(11))
-        revid = self._dostore(oid, revid=revid, data=MinPO(12))
+        revid1 = self._dostore(oid, revid=revid, data=MinPO(12))
         # And now store some new revisions in a version
         version = 'test-version'
-        revid = self._dostore(oid, revid=revid, data=MinPO(13),
+        revid = self._dostore(oid, revid=revid1, data=MinPO(13),
                               version=version)
         revid = self._dostore(oid, revid=revid, data=MinPO(14),
                               version=version)
-        revid = self._dostore(oid, revid=revid, data=MinPO(15),
+        revid2 = self._dostore(oid, revid=revid, data=MinPO(15),
                               version=version)
         # Now read back the object in both the non-version and version and
         # make sure the values jive.
@@ -78,6 +83,20 @@ class VersionStorage:
         if hasattr(self._storage, 'getSerial'):
             s = self._storage.getSerial(oid)
             eq(s, max(revid, vrevid))
+        data, tid, ver = self._storage.loadEx(oid, version)
+        eq(zodb_unpickle(data), MinPO(15))
+        eq(tid, revid2)
+        data, tid, ver = self._storage.loadEx(oid, "other version")
+        eq(zodb_unpickle(data), MinPO(12))
+        eq(tid, revid2)
+        # loadSerial returns non-version data
+        try:
+            data = self._storage.loadSerial(oid, revid)
+            eq(zodb_unpickle(data), MinPO(12))
+            data = self._storage.loadSerial(oid, revid2)
+            eq(zodb_unpickle(data), MinPO(12))
+        except POSException.Unsupported:
+            pass
 
     def checkVersionedLoadErrors(self):
         oid = self._storage.new_oid()
@@ -89,11 +108,6 @@ class VersionStorage:
         self.assertRaises(KeyError,
                           self._storage.load,
                           self._storage.new_oid(), '')
-        # Try to load a bogus version string
-        #JF# Nope, fall back to non-version
-        #JF# self.assertRaises(KeyError,
-        #JF#                   self._storage.load,
-        #JF#                   oid, 'bogus')
         data, revid = self._storage.load(oid, 'bogus')
         self.assertEqual(zodb_unpickle(data), MinPO(11))
 
@@ -112,9 +126,6 @@ class VersionStorage:
     def checkVersionEmpty(self):
         # Before we store anything, these versions ought to be empty
         version = 'test-version'
-        #JF# The empty string is not a valid version. I think that this should
-        #JF# be an error. Let's punt for now.
-        #JF# assert self._storage.versionEmpty('')
         self.failUnless(self._storage.versionEmpty(version))
         # Now store some objects
         oid = self._storage.new_oid()
@@ -125,10 +136,6 @@ class VersionStorage:
         revid = self._dostore(oid, revid=revid, data=MinPO(14),
                               version=version)
         # The blank version should not be empty
-        #JF# The empty string is not a valid version. I think that this should
-        #JF# be an error. Let's punt for now.
-        #JF# assert not self._storage.versionEmpty('')
-
         # Neither should 'test-version'
         self.failUnless(not self._storage.versionEmpty(version))
         # But this non-existant version should be empty
@@ -165,7 +172,7 @@ class VersionStorage:
         oid = self._storage.new_oid()
         revid = self._dostore(oid, data=MinPO(49))
         revid = self._dostore(oid, revid=revid, data=MinPO(50))
-        nvrevid = revid = self._dostore(oid, revid=revid, data=MinPO(51))
+        revid = self._dostore(oid, revid=revid, data=MinPO(51))
         # Now do some stores in a version
         revid = self._dostore(oid, revid=revid, data=MinPO(52),
                               version=version)
@@ -179,7 +186,7 @@ class VersionStorage:
         eq = self.assertEqual
         oid, version = self._setup_version()
 
-        # XXX Not sure I can write a test for getSerial() in the
+        # Not sure I can write a test for getSerial() in the
         # presence of aborted versions, because FileStorage and
         # Berkeley storage give a different answer. I think Berkeley
         # is right and FS is wrong.
@@ -190,6 +197,20 @@ class VersionStorage:
         data, revid = self._storage.load(oid, '')
         eq(zodb_unpickle(data), MinPO(51))
 
+    def checkAbortVersionNonCurrent(self):
+        # Make sure the non-current serial number is correctly
+        # after a version is aborted.
+        oid, version = self._setup_version()
+        self._abortVersion(version)
+        data, tid, ver = self._storage.loadEx(oid, "")
+        # write a new revision of oid so that the aborted-version txn
+        # is not current
+        self._dostore(oid, revid=tid, data=MinPO(17))
+        ltid = self._storage.lastTransaction()
+        ncdata, ncstart, end = self._storage.loadBefore(oid, ltid)
+        self.assertEqual(data, ncdata)
+        self.assertEqual(tid, ncstart)
+
     def checkAbortVersionErrors(self):
         eq = self.assertEqual
         oid, version = self._setup_version()
@@ -197,23 +218,15 @@ class VersionStorage:
         t = Transaction()
         self._storage.tpc_begin(t)
 
-        #JF# The spec is silent on what happens if you abort or commit
-        #JF# a non-existent version. FileStorage consideres this a noop.
-        #JF# We can change the spec, but until we do ....
-        #JF# self.assertRaises(POSException.VersionError,
-        #JF#                   self._storage.abortVersion,
-        #JF#                   'bogus', t)
-
         # And try to abort the empty version
-        if (hasattr(self._storage, 'supportsTransactionalUndo')
-            and self._storage.supportsTransactionalUndo()):
-            # XXX FileStorage used to be broken on this one
+        if (hasattr(self._storage, 'supportsTransactionalUndo') and
+                self._storage.supportsTransactionalUndo()):
             self.assertRaises(POSException.VersionError,
                               self._storage.abortVersion,
                               '', t)
 
         # But now we really try to abort the version
-        oids = self._storage.abortVersion(version, t)
+        tid, oids = self._storage.abortVersion(version, t)
         self._storage.tpc_vote(t)
         self._storage.tpc_finish(t)
         eq(len(oids), 1)
@@ -222,9 +235,8 @@ class VersionStorage:
         eq(zodb_unpickle(data), MinPO(51))
 
     def checkCommitVersionErrors(self):
-        if not (hasattr(self._storage, 'supportsTransactionalUndo')
-            and self._storage.supportsTransactionalUndo()):
-            # XXX FileStorage used to be broken on this one
+        if not (hasattr(self._storage, 'supportsTransactionalUndo') and
+                self._storage.supportsTransactionalUndo()):
             return
         eq = self.assertEqual
         oid1, version1 = self._setup_version('one')
@@ -240,19 +252,18 @@ class VersionStorage:
             self._storage.tpc_abort(t)
 
     def checkNewSerialOnCommitVersionToVersion(self):
-        eq = self.assertEqual
         oid, version = self._setup_version()
-        data, vserial = self._storage.load(oid, version)
-        data, nserial = self._storage.load(oid, '')
+        data, vtid = self._storage.load(oid, version)
+        data, ntid = self._storage.load(oid, '')
 
         version2 = 'test version 2'
         self._commitVersion(version, version2)
-        data, serial = self._storage.load(oid, version2)
+        data, tid = self._storage.load(oid, version2)
 
-        self.failUnless(serial != vserial and serial != nserial,
-                        "New serial, %r, should be different from the old "
-                        "version, %r, and non-version, %r, serials."
-                        % (serial, vserial, nserial))
+        self.failUnless(tid != vtid and tid != ntid,
+                        "New tid, %r, should be different from the old "
+                        "version, %r, and non-version, %r, tids."
+                        % (tid, vtid, ntid))
 
     def checkModifyAfterAbortVersion(self):
         eq = self.assertEqual
@@ -333,13 +344,8 @@ class VersionStorage:
         data, revid = self._storage.load(oid1, '')
         eq(zodb_unpickle(data), MinPO(51))
 
-        #JF# Ditto
-        #JF# self.assertRaises(POSException.VersionError,
-        #JF#                   self._storage.load, oid1, version1)
         data, revid = self._storage.load(oid1, '')
         eq(zodb_unpickle(data), MinPO(51))
-        #JF# self.assertRaises(POSException.VersionError,
-        #JF#                   self._storage.load, oid1, version2)
         data, revid = self._storage.load(oid1, '')
         eq(zodb_unpickle(data), MinPO(51))
 
@@ -360,7 +366,6 @@ class VersionStorage:
         data, revid = self._storage.load(oid2, version2)
         eq(zodb_unpickle(data), MinPO(54))
 
-        #JF# To do a test like you want, you have to add the data in a version
         oid = self._storage.new_oid()
         revid = self._dostore(oid, revid=revid, data=MinPO(54), version='one')
         self.assertRaises(KeyError,
@@ -376,7 +381,7 @@ class VersionStorage:
         # Now abort the version and the creation
         t = Transaction()
         self._storage.tpc_begin(t)
-        oids = self._storage.abortVersion('one', t)
+        tid, oids = self._storage.abortVersion('one', t)
         self._storage.tpc_vote(t)
         self._storage.tpc_finish(t)
         self.assertEqual(oids, [oid])
@@ -388,12 +393,12 @@ class VersionStorage:
 
         obj = root["obj"] = MinPO("obj")
         root["obj2"] = MinPO("obj2")
-        txn = get_transaction()
+        txn = transaction.get()
         txn.note("create 2 objs in version")
         txn.commit()
 
         obj.value = "77"
-        txn = get_transaction()
+        txn = transaction.get()
         txn.note("modify obj in version")
         txn.commit()
 
@@ -401,7 +406,7 @@ class VersionStorage:
         # and versions for pack to chase
         info = db.undoInfo()
         db.undo(info[0]["id"])
-        txn = get_transaction()
+        txn = transaction.get()
         txn.note("undo modification")
         txn.commit()
 
@@ -409,7 +414,7 @@ class VersionStorage:
         self._storage.pack(time.time(), referencesf)
 
         db.commitVersion("testversion")
-        txn = get_transaction()
+        txn = transaction.get()
         txn.note("commit version")
         txn.commit()
 
@@ -417,7 +422,7 @@ class VersionStorage:
         root = cn.root()
         root["obj"] = "no version"
 
-        txn = get_transaction()
+        txn = transaction.get()
         txn.note("modify obj")
         txn.commit()
 
@@ -430,12 +435,12 @@ class VersionStorage:
 
         obj = root["obj"] = MinPO("obj")
         root["obj2"] = MinPO("obj2")
-        txn = get_transaction()
+        txn = transaction.get()
         txn.note("create 2 objs in version")
         txn.commit()
 
         obj.value = "77"
-        txn = get_transaction()
+        txn = transaction.get()
         txn.note("modify obj in version")
         txn.commit()
 
@@ -446,14 +451,14 @@ class VersionStorage:
         # and versions for pack to chase
         info = db.undoInfo()
         db.undo(info[0]["id"])
-        txn = get_transaction()
+        txn = transaction.get()
         txn.note("undo modification")
         txn.commit()
 
         self._storage.pack(t0, referencesf)
 
         db.commitVersion("testversion")
-        txn = get_transaction()
+        txn = transaction.get()
         txn.note("commit version")
         txn.commit()
 
@@ -461,7 +466,7 @@ class VersionStorage:
         root = cn.root()
         root["obj"] = "no version"
 
-        txn = get_transaction()
+        txn = transaction.get()
         txn.note("modify obj")
         txn.commit()
 
@@ -476,23 +481,22 @@ class VersionStorage:
 
         for name in names:
             root[name] = MinPO(name)
-            get_transaction().commit()
+            transaction.commit()
 
         for name in names:
             cn2 = db.open(version=name)
             rt2 = cn2.root()
             obj = rt2[name]
             obj.value = MinPO("version")
-            get_transaction().commit()
+            transaction.commit()
             cn2.close()
 
         root["d"] = MinPO("d")
-        get_transaction().commit()
+        transaction.commit()
         snooze()
 
         self._storage.pack(time.time(), referencesf)
         cn.sync()
-        cn._cache.clear()
 
         # make sure all the non-version data is there
         for name, obj in root.items():
@@ -506,11 +510,11 @@ class VersionStorage:
             obj = rt2[name].value
             self.assertEqual(obj.value, "version")
             obj.value = "still version"
-            get_transaction().commit()
+            transaction.commit()
             cn2.close()
 
         db.abortVersion("b")
-        txn = get_transaction()
+        txn = transaction.get()
         txn.note("abort version b")
         txn.commit()
 
@@ -519,7 +523,7 @@ class VersionStorage:
 
         L = db.undoInfo()
         db.undo(L[0]["id"])
-        txn = get_transaction()
+        txn = transaction.get()
         txn.note("undo abort")
         txn.commit()
 
@@ -528,3 +532,14 @@ class VersionStorage:
         cn2 = db.open(version="b")
         rt2 = cn2.root()
         self.assertEqual(rt2["b"].value.value, "still version")
+
+    def checkLoadBeforeVersion(self):
+        eq = self.assertEqual
+        oid = self._storage.new_oid()
+        revid1 = self._dostore(oid, data=1)
+        revid2 = self._dostore(oid, data=2, revid=revid1, version="kobe")
+        revid3 = self._dostore(oid, data=3, revid=revid2, version="kobe")
+        data, start_tid, end_tid = self._storage.loadBefore(oid, revid3)
+        eq(zodb_unpickle(data), MinPO(1))
+        eq(start_tid, revid1)
+        eq(end_tid, None)
