@@ -14,11 +14,13 @@ __version__='$Revision: 1.12 $'[11:-2]
 
 """BeforeTraverse interface and helper classes"""
 
-import logging
 from Acquisition import aq_base
+from logging import getLogger
 
 # Interface
 
+LOG = getLogger('ZPublisher')
+
 def registerBeforeTraverse(container, object, app_handle, priority=99):
     """Register an object to be called before a container is traversed.
 
@@ -85,7 +87,6 @@ class MultiHook:
     MultiHook calls the named hook from the class of the container, then
     the prior hook, then all the hooks in its list.
     """
-    logger = logging.getLogger('MultiHook')
     def __init__(self, hookname, prior, defined_in_class):
         self._hookname = hookname
         self._prior = prior
@@ -103,7 +104,7 @@ class MultiHook:
             try:
                 cob(container, request)
             except TypeError:
-                self.logger.error('%s call %s failed.' % (
+                LOG.error('%s call %s failed.' % (
                     `self._hookname`, `cob`), exc_info=True)
 
     def add(self, cob):
@@ -120,7 +121,6 @@ class NameCaller:
 
     >>> registerBeforeTraverse(folder, NameCaller('preop'), 'XApp')
     """
-    logger = logging.getLogger('BeforeTraverse')
 
     def __init__(self, name):
         self.name = name
@@ -150,5 +150,5 @@ class NameCaller:
             # Only catch exceptions that are likely to be logic errors.
             # We shouldn't catch Redirects, Unauthorizeds, etc. since
             # the programmer may want to raise them deliberately.
-            self.logger.error('Error while invoking hook: "%s"'
-                                % self.name, exc_info=True)
+            LOG.error('BeforeTraverse: Error while invoking hook: "%s"' % self.name, 
+                      exc_info=True)
