@@ -5,7 +5,7 @@
 #						 All Rights Reserved.
 #
 
-RCS_ID =  '$Id: ftp_server.py,v 1.23 2003/03/18 21:15:17 fdrake Exp $'
+RCS_ID =  '$Id: ftp_server.py 41297 2006-01-13 14:39:36Z andreasjung $'
 
 # An extensible, configurable, asynchronous FTP server.
 # 
@@ -705,13 +705,13 @@ class ftp_server (asyncore.dispatcher):
         self.ip = ip
         self.port = port
         self.authorizer = authorizer
-        
-        if hostname is None:
-            self.hostname = socket.gethostname()
-        else:
-            self.hostname = hostname
-            
-            # statistics
+
+        try:
+            self.hostname = socket.gethostbyaddr(ip)[0]
+        except socket.error:
+            self.hostname = ip
+
+        # statistics
         self.total_sessions = counter()
         self.closed_sessions = counter()
         self.total_files_out = counter()
