@@ -5,7 +5,7 @@
 # All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
@@ -26,6 +26,7 @@ Options:
 -l/--logfile -- log file to be read by logtail command
 -u/--user -- run the daemon manager program as this user (or numeric id)
 -m/--umask -- provide octal umask for files created by the managed process
+-s/--socket-name -- socket between zopectl and zdrun
 action [arguments] -- see below
 
 Actions are commands like "start", "stop" and "status".  If -i is
@@ -45,6 +46,7 @@ import Zope2.Startup
 from zdaemon.zdctl import ZDCmd
 from zdaemon.zdoptions import ZDOptions
 from ZConfig.components.logger.handlers import FileHandlerFactory
+from ZConfig.datatypes import existing_dirpath
 
 
 def string_list(arg):
@@ -86,6 +88,8 @@ class ZopeCtlOptions(ZDOptions):
         self.add("user", "runner.user", "u:", "user=")
         self.add("prompt", "runner.prompt", default="zopectl>")
         self.add("umask", "runner.umask", "m:", "umask=")
+        self.add("sockname", "runner.socket_name", "s:", "socket-name=",
+                 existing_dirpath, default=None)
 
     def realize(self, *args, **kw):
         ZDOptions.realize(self, *args, **kw)
@@ -116,7 +120,10 @@ class ZopeCtlOptions(ZDOptions):
             self.program = config.runner.program
         else:
             self.program = [os.path.join(self.directory, "bin", "runzope")]
-        self.sockname = os.path.join(self.clienthome, "zopectlsock")
+        if config.runner and config.runner.socket_name:
+            self.sockname = config.runner.socket_name
+        else:
+            self.sockname = os.path.join(self.clienthome, "zopectlsock")
         self.python = sys.executable
         self.zdrun = os.path.join(os.path.dirname(zdaemon.__file__),
                                   "zdrun.py")
@@ -242,12 +249,6 @@ class ZopeCmd(ZDCmd):
             script = os.path.join(zope_home, 'test.py') # no inplace build!
         assert os.path.exists(script)
 
-        # If --libdir is not supplied, use $INSTANCE_HOME/Products
-        # (rather than $INSTANCE_HOME/lib/python)
-        if '--libdir' not in args:
-            args.insert(0, 'Products')
-            args.insert(0, '--libdir')
-
         # Supply our config file by default.
         if '--config-file' not in args and '-C' not in args:
             args.insert(0, self.options.configfile)
@@ -264,14 +265,20 @@ class ZopeCmd(ZDCmd):
         pid = os.fork()
         if pid == 0:  # child
             os.execv(self.options.python, args)
-        else:
-            os.waitpid(pid, 0)
+        
+        # Parent process running (execv replaces process in child
+        while True:
+            try:
+                os.waitpid(pid, 0)
+            except (OSError, KeyboardInterrupt):
+                continue
+            else:
+                break
 
     def help_test(self):
         print "test [args]+ -- run unit / functional tests."
         print "                See $ZOPE_HOME/bin/test.py --help for syntax."
 
-
 def main(args=None):
     # This is exactly like zdctl.main(), but uses ZopeCtlOptions and
     # ZopeCmd instead of ZDCtlOptions and ZDCmd, so the default values
