@@ -1,7 +1,7 @@
 # Author: David Goodger
 # Contact: goodger@python.org
-# Revision: $Revision: 3206 $
-# Date: $Date: 2005-04-12 01:16:11 +0200 (Tue, 12 Apr 2005) $
+# Revision: $Revision: 3963 $
+# Date: $Date: 2005-10-28 18:12:56 +0200 (Fri, 28 Oct 2005) $
 # Copyright: This module has been placed in the public domain.
 
 """
@@ -167,3 +167,30 @@ def compound(name, arguments, options, content, lineno,
 
 compound.options = {'class': directives.class_option}
 compound.content = 1
+
+def container(name, arguments, options, content, lineno,
+              content_offset, block_text, state, state_machine):
+    text = '\n'.join(content)
+    if not text:
+        error = state_machine.reporter.error(
+            'The "%s" directive is empty; content required.' % name,
+            nodes.literal_block(block_text, block_text), line=lineno)
+        return [error]
+    try:
+        if arguments:
+            classes = directives.class_option(arguments[0])
+        else:
+            classes = []
+    except ValueError:
+        error = state_machine.reporter.error(
+            'Invalid class attribute value for "%s" directive: "%s".'
+            % (name, arguments[0]),
+            nodes.literal_block(block_text, block_text), line=lineno)
+        return [error]
+    node = nodes.container(text)
+    node['classes'].extend(classes)
+    state.nested_parse(content, content_offset, node)
+    return [node]
+
+container.arguments = (0, 1, 1)
+container.content = 1
