@@ -1,7 +1,7 @@
 # Authors: David Goodger, David Priest
 # Contact: goodger@python.org
-# Revision: $Revision: 3165 $
-# Date: $Date: 2005-04-05 04:55:06 +0200 (Tue, 05 Apr 2005) $
+# Revision: $Revision: 3900 $
+# Date: $Date: 2005-09-24 17:11:20 +0200 (Sat, 24 Sep 2005) $
 # Copyright: This module has been placed in the public domain.
 
 """
@@ -117,8 +117,8 @@ def csv_table(name, arguments, options, content, lineno,
         if ( not state.document.settings.file_insertion_enabled
              and (options.has_key('file') or options.has_key('url')) ):
             warning = state_machine.reporter.warning(
-                '"%s" directive disabled.' % name,
-                nodes.literal_block(block_text, block_text), line=lineno)
+                'File and URL access deactivated; ignoring "%s" directive.' %
+                name, nodes.literal_block(block_text,block_text), line=lineno)
             return [warning]
         check_requirements(name, lineno, block_text, state_machine)
         title, messages = make_title(arguments, state, lineno)
@@ -392,6 +392,9 @@ def check_list_content(node, name, options, content, lineno, block_text,
                 nodes.literal_block(block_text, block_text), line=lineno)
             raise SystemMessagePropagation(error)
         elif item_index:
+            # ATTN pychecker users: num_cols is guaranteed to be set in the
+            # "else" clause below for item_index==0, before this branch is
+            # triggered.
             if len(item[0]) != num_cols:
                 error = state_machine.reporter.error(
                     'Error parsing content block for the "%s" directive: '
