@@ -1,7 +1,7 @@
 # Authors: David Goodger; William Dode
 # Contact: goodger@users.sourceforge.net
-# Revision: $Revision: 3184 $
-# Date: $Date: 2005-04-07 21:36:11 +0200 (Thu, 07 Apr 2005) $
+# Revision: $Revision: 4229 $
+# Date: $Date: 2005-12-23 00:46:16 +0100 (Fri, 23 Dec 2005) $
 # Copyright: This module has been placed in the public domain.
 
 # New language mappings are welcome.  Before doing a new translation, please
@@ -39,6 +39,7 @@ directives = {
       u'chapeau': 'highlights',
       u'accroche': 'pull-quote',
       u'compound (translation required)': 'compound',
+      u'container (translation required)': 'container',
       #u'questions': 'questions',
       #u'qr': 'questions',
       #u'faq': 'questions',
@@ -54,8 +55,11 @@ directives = {
       u'remplacer': 'replace',
       u'remplace': 'replace',
       u'unicode': 'unicode',
+      u'date': 'date',
       u'classe': 'class',
       u'role (translation required)': 'role',
+      u'default-role (translation required)': 'default-role',
+      u'titre (translation required)': 'title',
       u'sommaire': 'contents',
       u'table-des-mati\u00E8res': 'contents',
       u'sectnum': 'sectnum',
