@@ -1,7 +1,7 @@
 # Author:    Adam Chodorowski
 # Contact:   chodorowski@users.sourceforge.net
-# Revision:  $Revision: 1.2.10.7 $
-# Date:      $Date: 2005/01/07 13:26:04 $
+# Revision:  $Revision: 4229 $
+# Date:      $Date: 2005-12-23 00:46:16 +0100 (Fri, 23 Dec 2005) $
 # Copyright: This module has been placed in the public domain.
 
 # New language mappings are welcome.  Before doing a new translation, please
@@ -36,12 +36,14 @@ directives = {
       u'highlights (translation required)': 'highlights',
       u'pull-quote (translation required)': 'pull-quote',
       u'compound (translation required)': 'compound',
+      u'container (translation required)': 'container',
       # u'fr\u00e5gor': 'questions',
       # NOTE: A bit long, but recommended by http://www.nada.kth.se/dataterm/:
       # u'fr\u00e5gor-och-svar': 'questions',
       # u'vanliga-fr\u00e5gor': 'questions',  
       u'table (translation required)': 'table',
       u'csv-table (translation required)': 'csv-table',
+      u'list-table (translation required)': 'list-table',
       u'meta': 'meta',
       # u'bildkarta': 'imagemap',   # FIXME: Translation might be too literal.
       u'bild': 'image',
@@ -50,11 +52,16 @@ directives = {
       u'r\u00e5': 'raw',            # FIXME: Translation might be too literal.
       u'ers\u00e4tt': 'replace', 
       u'unicode': 'unicode',
+      u'datum': 'date',
       u'class (translation required)': 'class',
       u'role (translation required)': 'role',
+      u'default-role (translation required)': 'default-role',
+      u'title (translation required)': 'title',
       u'inneh\u00e5ll': 'contents',
       u'sektionsnumrering': 'sectnum',
       u'target-notes (translation required)': 'target-notes',
+      u'header (translation required)': 'header',
+      u'footer (translation required)': 'footer',
       # u'fotnoter': 'footnotes',
       # u'citeringar': 'citations',
       }
