@@ -1,7 +1,7 @@
 # Author: David Goodger
 # Contact: goodger@users.sourceforge.net
-# Revision: $Revision: 1.2.10.7 $
-# Date: $Date: 2005/01/07 13:26:05 $
+# Revision: $Revision: 3892 $
+# Date: $Date: 2005-09-20 22:04:53 +0200 (Tue, 20 Sep 2005) $
 # Copyright: This module has been placed in the public domain.
 
 """
@@ -12,7 +12,7 @@ __docformat__ = 'reStructuredText'
 
 
 from docutils.readers import standalone
-from docutils.transforms import peps, references
+from docutils.transforms import peps, references, misc, frontmatter
 from docutils.parsers import rst
 
 
@@ -30,16 +30,14 @@ class Reader(standalone.Reader):
     config_section = 'pep reader'
     config_section_dependencies = ('readers', 'standalone reader')
 
-    default_transforms = (references.Substitutions,
-                          peps.Headers,
-                          peps.Contents,
-                          references.ChainedTargets,
-                          references.AnonymousHyperlinks,
-                          references.IndirectHyperlinks,
-                          peps.TargetNotes,
-                          references.Footnotes,
-                          references.ExternalTargets,
-                          references.InternalTargets,)
+    def get_transforms(self):
+        transforms = standalone.Reader.get_transforms(self)
+        # We have PEP-specific frontmatter handling.
+        transforms.remove(frontmatter.DocTitle)
+        transforms.remove(frontmatter.SectionSubTitle)
+        transforms.remove(frontmatter.DocInfo)
+        transforms.extend([peps.Headers, peps.Contents, peps.TargetNotes])
+        return transforms
 
     settings_default_overrides = {'pep_references': 1, 'rfc_references': 1}
 
