@@ -1,7 +1,7 @@
 # Author: David Goodger
 # Contact: goodger@users.sourceforge.net
-# Revision: $Revision: 1.2.10.4 $
-# Date: $Date: 2004/05/13 16:19:49 $
+# Revision: $Revision: 4152 $
+# Date: $Date: 2005-12-08 00:46:30 +0100 (Thu, 08 Dec 2005) $
 # Copyright: This module has been placed in the public domain.
 
 """
@@ -110,7 +110,8 @@ __docformat__ = 'restructuredtext'
 
 import sys
 import re
-from types import SliceType as _SliceType
+import types
+import unicodedata
 
 
 class StateMachine:
@@ -344,6 +345,10 @@ class StateMachine:
         finally:
             self.notify_observers()
 
+    def get_source(self, line_offset):
+        """Return source of line at absolute line offset `line_offset`."""
+        return self.input_lines.source(line_offset - self.input_offset)
+
     def abs_line_offset(self):
         """Return line offset of current line, from beginning of file."""
         return self.line_offset + self.input_offset
@@ -1112,7 +1117,7 @@ class ViewList:
     # indexing a list with a slice object; they just work).
 
     def __getitem__(self, i):
-        if isinstance(i, _SliceType):
+        if isinstance(i, types.SliceType):
             assert i.step in (None, 1),  'cannot handle slice with stride'
             return self.__class__(self.data[i.start:i.stop],
                                   items=self.items[i.start:i.stop],
@@ -1121,7 +1126,7 @@ class ViewList:
             return self.data[i]
 
     def __setitem__(self, i, item):
-        if isinstance(i, _SliceType):
+        if isinstance(i, types.SliceType):
             assert i.step in (None, 1), 'cannot handle slice with stride'
             if not isinstance(item, ViewList):
                 raise TypeError('assigning non-ViewList to ViewList slice')
@@ -1396,6 +1401,30 @@ class StringList(ViewList):
             block.data = [line[indent:] for line in block.data]
         return block
 
+    def pad_double_width(self, pad_char):
+        """
+        Pad all double-width characters in self by appending `pad_char` to each.
+        For East Asian language support.
+        """
+        if hasattr(unicodedata, 'east_asian_width'):
+            east_asian_width = unicodedata.east_asian_width
+        else:
+            return                      # new in Python 2.4
+        for i in range(len(self.data)):
+            line = self.data[i]
+            if isinstance(line, types.UnicodeType):
+                new = []
+                for char in line:
+                    new.append(char)
+                    if east_asian_width(char) in 'WF': # 'W'ide & 'F'ull-width
+                        new.append(pad_char)
+                self.data[i] = ''.join(new)
+
+    def replace(self, old, new):
+        """Replace all occurrences of substring `old` with `new`."""
+        for i in range(len(self.data)):
+            self.data[i] = self.data[i].replace(old, new)
+
 
 class StateMachineError(Exception): pass
 class UnknownStateError(StateMachineError): pass
@@ -1429,7 +1458,8 @@ class StateCorrection(Exception):
 def string2lines(astring, tab_width=8, convert_whitespace=0,
                  whitespace=re.compile('[\v\f]')):
     """
-    Return a list of one-line strings with tabs expanded and no newlines.
+    Return a list of one-line strings with tabs expanded, no newlines, and
+    trailing whitespace stripped.
 
     Each tab is expanded with between 1 and `tab_width` spaces, so that the
     next character's index becomes a multiple of `tab_width` (8 by default).
@@ -1442,7 +1472,7 @@ def string2lines(astring, tab_width=8, convert_whitespace=0,
     """
     if convert_whitespace:
         astring = whitespace.sub(' ', astring)
-    return [s.expandtabs(tab_width) for s in astring.splitlines()]
+    return [s.expandtabs(tab_width).rstrip() for s in astring.splitlines()]
 
 def _exception_data():
     """
