@@ -21,9 +21,9 @@ except ImportError:
     raise ImportError, 'Please install docutils 0.3.3+ from http://docutils.sourceforge.net/#download.'
 
 version = docutils.__version__.split('.')
-if version < ['0', '3', '3']:
+if not (version >= ['0', '4', '0'] or  version >= ['0', '4']):
     raise ImportError, """Old version of docutils found:
-Got: %(version)s, required: 0.3.3+
+Got: %(version)s, required: 0.4.0+
 Please remove docutils from %(path)s and replace it with a new version. You
 can download docutils at http://docutils.sourceforge.net/#download.
 """ % {'version' : docutils.__version__, 'path' : docutils.__path__[0] }
@@ -58,7 +58,7 @@ class Warnings:
 def render(src,
            writer='html4css1',
            report_level=1,
-           stylesheet='default.css',
+           stylesheet=None,
            input_encoding=default_input_encoding,
            output_encoding=default_output_encoding,
            language_code=default_language_code,
@@ -71,6 +71,7 @@ def render(src,
     settings['input_encoding'] = input_encoding
     settings['output_encoding'] = output_encoding
     settings['stylesheet'] = stylesheet
+    settings['stylesheet_path'] = None
     settings['file_insertion_enabled'] = 0
     settings['raw_enabled'] = 0
     if language_code:
@@ -94,7 +95,7 @@ def render(src,
 def HTML(src,
          writer='html4css1',
          report_level=1,
-         stylesheet='default.css',
+         stylesheet=None,
          input_encoding=default_input_encoding,
          output_encoding=default_output_encoding,
          language_code=default_language_code,
@@ -162,6 +163,9 @@ def HTML(src,
     
     warnings = ''.join(warning_stream.messages)
 
-    return output.encode(output_encoding)
+    if output_encoding != 'unicode':
+        return output.encode(output_encoding)
+    else:
+        return output
 
 __all__ = ("HTML", 'render')
