@@ -1,36 +1,45 @@
 ##############################################################################
 #
-# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
+# Copyright (c) 2002 Zope Corporation and Contributors. All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
-# FOR A PARTICULAR PURPOSE
+# FOR A PARTICULAR PURPOSE.
 #
 ##############################################################################
 """WebDAV support - collection objects.
 
-$Id: Collection.py 30356 2005-05-15 15:56:18Z yuppie $
+$Id: Collection.py 40300 2005-11-21 16:54:03Z efge $
 """
 
-import Globals, davcmds, Lockable
-from common import urlfix, rfc1123_date
-from Resource import Resource
-from AccessControl import getSecurityManager
 from urllib import unquote
+
+from Globals import InitializeClass
+from AccessControl import getSecurityManager
 from zExceptions import MethodNotAllowed, NotFound
-from webdav.common import Locked, PreconditionFailed
+from zope.interface import implements
+
+import davcmds
+import Lockable
+from common import Locked, PreconditionFailed
+from common import urlfix, rfc1123_date
+from interfaces import IDAVCollection
+from Resource import Resource
 
 
 class Collection(Resource):
+
     """The Collection class provides basic WebDAV support for
     collection objects. It provides default implementations
     for all supported WebDAV HTTP methods. The behaviors of some
     WebDAV HTTP methods for collections are slightly different
     than those for non-collection resources."""
 
+    implements(IDAVCollection)
+
     __dav_collection__=1
 
     def dav__init(self, request, response):
@@ -133,4 +142,4 @@ class Collection(Resource):
             return objectValues()
         return []
 
-Globals.default__class_init__(Collection)
+InitializeClass(Collection)
