@@ -12,20 +12,24 @@
 ##############################################################################
 """WebDAV support - null resource objects.
 
-$Id: NullResource.py 41329 2006-01-16 17:35:51Z tseaver $
+$Id: NullResource.py 67823 2006-05-02 09:55:17Z philikon $
 """
 
 import sys
 
-import Acquisition
-import Globals
+import Acquisition 
 import OFS.SimpleItem
+from Globals import InitializeClass
 from AccessControl import getSecurityManager
+from AccessControl import ClassSecurityInfo
+from AccessControl.Permissions import view as View
+from AccessControl.Permissions import add_folders
+from AccessControl.Permissions import webdav_lock_items
+from AccessControl.Permissions import webdav_unlock_items
 from Globals import Persistent, DTMLFile
 from OFS.CopySupport import CopyError
 from zExceptions import MethodNotAllowed
 from zExceptions import Unauthorized, NotFound, Forbidden, BadRequest
-from zope.app.content_types import guess_content_type
 
 import davcmds
 from common import aq_base, tokenFinder, IfParser
@@ -35,6 +39,8 @@ from interfaces import IWriteLock
 from Resource import Resource
 from WriteLockInterface import WriteLockInterface
 
+from zope.contenttype import guess_content_type
+
 
 class NullResource(Persistent, Acquisition.Implicit, Resource):
 
@@ -44,11 +50,7 @@ class NullResource(Persistent, Acquisition.Implicit, Resource):
     __implements__ = (WriteLockInterface,)
     __null_resource__=1
 
-    __ac_permissions__=(
-        ('View',                             ('HEAD',)),
-        ('Add Folders',                      ('MKCOL',)),
-        ('WebDAV Lock items',                ('LOCK',)),
-    )
+    security = ClassSecurityInfo()
 
     def __init__(self, parent, name, request=None):
         self.__name__=name
@@ -65,6 +67,7 @@ class NullResource(Persistent, Acquisition.Implicit, Resource):
             raise Conflict, 'Collection ancestors must already exist.'
         raise NotFound, 'The requested resource was not found.'
 
+    security.declareProtected(View, 'HEAD')
     def HEAD(self, REQUEST, RESPONSE):
         """Retrieve resource information without a response message body."""
         self.dav__init(REQUEST, RESPONSE)
@@ -90,7 +93,7 @@ class NullResource(Persistent, Acquisition.Implicit, Resource):
             ob=File(name, '', body, content_type=typ)
         return ob
 
-    PUT__roles__ = ('Anonymous',)
+    security.declarePublic('PUT')
     def PUT(self, REQUEST, RESPONSE):
         """Create a new non-collection resource.
         """
@@ -142,7 +145,7 @@ class NullResource(Persistent, Acquisition.Implicit, Resource):
 
         typ=REQUEST.get_header('content-type', None)
         if typ is None:
-            typ, enc = guess_content_type(name, body)
+            typ, enc=guess_content_type(name, body)
 
         factory = getattr(parent, 'PUT_factory', self._default_PUT_factory )
         ob = factory(name, typ, body)
@@ -167,6 +170,7 @@ class NullResource(Persistent, Acquisition.Implicit, Resource):
         RESPONSE.setBody('')
         return RESPONSE
 
+    security.declareProtected(add_folders, 'MKCOL')
     def MKCOL(self, REQUEST, RESPONSE):
         """Create a new collection resource."""
         self.dav__init(REQUEST, RESPONSE)
@@ -202,6 +206,7 @@ class NullResource(Persistent, Acquisition.Implicit, Resource):
         RESPONSE.setBody('')
         return RESPONSE
 
+    security.declareProtected(webdav_lock_items, 'LOCK')
     def LOCK(self, REQUEST, RESPONSE):
         """ LOCK on a Null Resource makes a LockNullResource instance """
         self.dav__init(REQUEST, RESPONSE)
@@ -253,8 +258,7 @@ class NullResource(Persistent, Acquisition.Implicit, Resource):
             RESPONSE.setHeader('Lock-Token', 'opaquelocktoken:' + token)
             RESPONSE.setBody(lock.asXML())
 
-
-Globals.default__class_init__(NullResource)
+InitializeClass(NullResource)
 
 
 class LockNullResource(NullResource, OFS.SimpleItem.Item_w__name__):
@@ -267,17 +271,14 @@ class LockNullResource(NullResource, OFS.SimpleItem.Item_w__name__):
     __locknull_resource__ = 1
     meta_type = 'WebDAV LockNull Resource'
 
-    __ac_permissions__ = (
-        ('WebDAV Unlock items',              ('UNLOCK',)),
-        ('View',                             ('manage_main',
-                                              'manage_workspace', 'manage')),
-        ('Add Folders',                      ('MKCOL',)),
-        ('WebDAV Lock items',                ('LOCK',)),
-        )
+    security = ClassSecurityInfo()
 
     manage_options = ({'label': 'Info', 'action': 'manage_main'},)
 
+    security.declareProtected(View, 'manage')
+    security.declareProtected(View, 'manage_main')
     manage = manage_main = DTMLFile('dtml/locknullmain', globals())
+    security.declareProtected(View, 'manage_workspace')
     manage_workspace = manage
     manage_main._setName('manage_main')  # explicit
 
@@ -292,7 +293,7 @@ class LockNullResource(NullResource, OFS.SimpleItem.Item_w__name__):
         self.id = self.__name__ = name
         self.title = "LockNull Resource '%s'" % name
 
-    title_or_id__roles__=None
+    security.declarePublic('title_or_id')
     def title_or_id(self):
         return 'Foo'
 
@@ -300,6 +301,7 @@ class LockNullResource(NullResource, OFS.SimpleItem.Item_w__name__):
         """Retrieve properties defined on the resource."""
         return Resource.PROPFIND(self, REQUEST, RESPONSE)
 
+    security.declareProtected(webdav_lock_items, 'LOCK')
     def LOCK(self, REQUEST, RESPONSE):
         """ A Lock command on a LockNull resource should only be a
         refresh request (one without a body) """
@@ -337,6 +339,7 @@ class LockNullResource(NullResource, OFS.SimpleItem.Item_w__name__):
         return RESPONSE
 
 
+    security.declareProtected(webdav_unlock_items, 'UNLOCK')
     def UNLOCK(self, REQUEST, RESPONSE):
         """ Unlocking a Null Resource removes it from its parent """
         self.dav__init(REQUEST, RESPONSE)
@@ -363,7 +366,7 @@ class LockNullResource(NullResource, OFS.SimpleItem.Item_w__name__):
             RESPONSE.setStatus(204)
         return RESPONSE
 
-    PUT__roles__ = ('Anonymous',)
+    security.declarePublic('PUT')
     def PUT(self, REQUEST, RESPONSE):
         """ Create a new non-collection resource, deleting the LockNull
         object from the container before putting the new object in. """
@@ -438,6 +441,7 @@ class LockNullResource(NullResource, OFS.SimpleItem.Item_w__name__):
         RESPONSE.setBody('')
         return RESPONSE
 
+    security.declareProtected(add_folders, 'MKCOL')
     def MKCOL(self, REQUEST, RESPONSE):
         """ Create a new Collection (folder) resource.  Since this is being
         done on a LockNull resource, this also involves removing the LockNull
@@ -485,4 +489,4 @@ class LockNullResource(NullResource, OFS.SimpleItem.Item_w__name__):
         RESPONSE.setBody('')
         return RESPONSE
 
-Globals.default__class_init__(LockNullResource)
+InitializeClass(LockNullResource)
