@@ -13,11 +13,11 @@
 ##############################################################################
 """RAMCache view
 
-$Id: ram.py 26617 2004-07-19 03:25:41Z pruggera $
+$Id: ram.py 67630 2006-04-27 00:54:03Z jim $
 """
 __docformat__ = 'restructuredtext'
 
-from zope.app.publisher.browser import BrowserView
+from zope.publisher.browser import BrowserView
 from zope.app.cache.interfaces.ram import IRAMCache
 
 class RAMCacheView(BrowserView):
