@@ -13,13 +13,16 @@
 ##############################################################################
 """Helpers for caching.
 
-$Id: caching.py 26617 2004-07-19 03:25:41Z pruggera $
+$Id: caching.py 67630 2006-04-27 00:54:03Z jim $
 """
 __docformat__ = 'restructuredtext'
 
+from zope.interface import classProvides
+from zope.component import ComponentLookupError
+from zope.schema.interfaces import IVocabularyFactory
 from zope.app import zapi
 from zope.app.cache.interfaces import ICacheable, ICache
-from zope.component import ComponentLookupError
+from zope.app.component.vocabulary import UtilityVocabulary
 
 def getCacheForObject(obj):
     """Returns the cache associated with `obj` or ``None``."""
@@ -37,3 +40,8 @@ def getLocationForCache(obj):
         return zapi.getPath(obj)
     except (ComponentLookupError, TypeError):
         return None
+
+class CacheNamesVocabulary(UtilityVocabulary):
+    classProvides(IVocabularyFactory)
+    interface = ICache
+    nameOnly = True
