@@ -13,7 +13,7 @@
 ##############################################################################
 """Unit tests for RAM Cache.
 
-$Id: test_ramcache.py 67630 2006-04-27 00:54:03Z jim $
+$Id: test_ramcache.py 77125 2007-06-27 12:54:42Z nouri $
 """
 from time import time
 from unittest import TestCase, TestSuite, main, makeSuite
@@ -87,6 +87,17 @@ class TestRAMCache(PlacelessSetup,
         self.assertEqual(s.maxAge, 2, "maxAge not set")
         self.assertEqual(s.cleanupInterval, 3, "cleanupInterval not set")
 
+    def test_timedCleanup(self):
+        from zope.app.cache.ram import RAMCache
+        import time
+        c = RAMCache()
+        c.update(cleanupInterval=1, maxAge=2)
+        lastCleanup = c._getStorage().lastCleanup
+        time.sleep(2)
+        c.set(42, "object", key={'foo': 'bar'})
+        # last cleanup should now be updated
+        self.failUnless(lastCleanup < c._getStorage().lastCleanup)
+
     def test_cache(self):
         from zope.app.cache import ram
         self.assertEqual(type(ram.caches), type({}),
@@ -449,15 +460,15 @@ class TestStorage(TestCase):
         value = 'yes'
         timestamp = time()
         s._data = {object:  {key1: [value, 1, 10],
-                             key2: [value, 2, 5],
-                             key3: [value, 3, 2]},
-                   object2: {key1: [value, 4, 2],
-                             key2: [value, 5, 1],
-                             key3: [value, 6, 1]}}
+                             key2: [value, 6, 5],
+                             key3: [value, 2, 2]},
+                   object2: {key1: [value, 5, 2],
+                             key2: [value, 3, 1],
+                             key3: [value, 4, 1]}}
         s.removeLeastAccessed()
         self.assertEqual(s._data,
                          {object:  {key1: [value, 1, 0],
-                                    key2: [value, 2, 0]}},
+                                    key2: [value, 6, 0]}},
                          'least records removed incorrectly')
 
         s = Storage(maxEntries=6)
