@@ -13,28 +13,28 @@
 ##############################################################################
 """Adding implementation tests
 
-$Id: test_adding.py 65722 2006-03-02 14:44:16Z hdima $
+$Id: test_adding.py 67630 2006-04-27 00:54:03Z jim $
 """
 import unittest
 
 import zope.interface
 import zope.security.checker
 from zope.component.interfaces import IFactory
-from zope.component.exceptions import ComponentLookupError
+from zope.component.interfaces import ComponentLookupError
 from zope.interface import implements, Interface, directlyProvides
 from zope.publisher.browser import TestRequest
 from zope.publisher.interfaces.browser import IBrowserRequest
+from zope.publisher.browser import BrowserView
 from zope.security.interfaces import ForbiddenAttribute
 from zope.testing.doctestunit import DocTestSuite
+from zope.exceptions.interfaces import UserError
+from zope.traversing.browser import AbsoluteURL
+from zope.traversing.browser.interfaces import IAbsoluteURL
+from zope.traversing.interfaces import IContainmentRoot
 
 from zope.app import zapi
 from zope.app.testing import ztapi
 from zope.app.testing.placelesssetup import PlacelessSetup, setUp, tearDown
-from zope.app.traversing.browser import AbsoluteURL
-from zope.app.traversing.browser.interfaces import IAbsoluteURL
-from zope.app.traversing.interfaces import IContainmentRoot
-from zope.app.exception.interfaces import UserError
-from zope.app.publisher.browser import BrowserView
 from zope.app.publisher.interfaces.browser import AddMenu
 from zope.app.publisher.interfaces.browser import IMenuItemType, IBrowserMenu
 from zope.app.publisher.browser.menu import BrowserMenuItem, BrowserMenu
