@@ -13,19 +13,19 @@
 ##############################################################################
 """Folder content component tests
 
-$Id: tests.py 71584 2006-12-18 09:20:36Z ctheune $
+$Id: tests.py 71583 2006-12-18 09:14:12Z ctheune $
 """
 
 from unittest import TestCase, TestSuite, main, makeSuite
 
+import zope.component
 from zope.testing.doctestunit import DocTestSuite
 from zope.testing import doctest
+from zope.dublincore.interfaces import IZopeDublinCore
+from zope.dublincore.annotatableadapter import ZDCAnnotatableAdapter
 
-from zope.app.testing import ztapi
 from zope.app.testing.functional import FunctionalDocFileSuite
-from zope.app.dublincore.interfaces import IZopeDublinCore
 from zope.app.folder.interfaces import IFolder
-from zope.app.dublincore.annotatableadapter import ZDCAnnotatableAdapter
 from zope.app.component.testing import PlacefulSetup
 from zope.app.component.tests.test_site import BaseTestSiteManagerContainer
 from zope.app.container.tests.test_icontainer import BaseTestIContainer
@@ -53,7 +53,8 @@ class FolderMetaDataTest(PlacefulSetup, TestCase):
     def setUp(self):
         PlacefulSetup.setUp(self)
         PlacefulSetup.buildFolders(self)
-        ztapi.provideAdapter(IFolder, IZopeDublinCore, ZDCAnnotatableAdapter)
+        zope.component.provideAdapter(ZDCAnnotatableAdapter, (IFolder,),
+                                      IZopeDublinCore)
 
 def test_suite():
     from zope.testing.doctestunit import DocTestSuite
