@@ -13,24 +13,19 @@
 ##############################################################################
 """Int Widget Functional Tests
 
-$Id: test_intwidget.py 39777 2005-10-31 19:08:20Z jim $
+$Id: test_intwidget.py 67630 2006-04-27 00:54:03Z jim $
 """
 import unittest
-from persistent import Persistent
 import transaction
+from persistent import Persistent
 
-from support import *
-
-from zope.interface import Interface
-from zope.interface import implements
-
-from zope.schema import Int, Choice
 import zope.security.checker
-
-from zope.app.traversing.api import traverse
+from zope.interface import Interface, implements
+from zope.schema import Int, Choice
+from zope.traversing.api import traverse
 
 from zope.app.testing.functional import BrowserTestCase
-
+from zope.app.form.browser.ftests.support import *
 
 class IIntTest(Interface):
 
