@@ -13,22 +13,20 @@
 ##############################################################################
 """TextWidget Tests
 
-$Id: test_textwidget.py 70410 2006-09-28 07:30:15Z yuppie $
+$Id: test_textwidget.py 68730 2006-06-18 05:35:59Z ctheune $
 """
 import unittest
-from persistent import Persistent
 import transaction
+from persistent import Persistent
 
+import zope.security.checker
 from zope.interface import Interface, implements
 from zope.schema import TextLine, Choice
-import zope.security.checker
-
-from support import *
-from zope.app.traversing.api import traverse
+from zope.traversing.api import traverse
 
+from zope.app.form.browser.ftests.support import *
 from zope.app.testing.functional import BrowserTestCase
 
-
 class ITextLineTest(Interface):
 
     s2 = TextLine(
