@@ -13,10 +13,10 @@
 ##############################################################################
 """File Widget tests
 
-$Id: test_filewidget.py 37569 2005-07-29 21:10:06Z poster $
+$Id: test_filewidget.py 69217 2006-07-20 03:56:26Z baijum $
 """
-import unittest, doctest
-
+import unittest
+from zope.testing import doctest
 from StringIO import StringIO
 from zope.app.form.interfaces import IInputWidget
 from zope.app.form.browser import FileWidget
