@@ -13,19 +13,20 @@
 ##############################################################################
 """Browser widgets with text-based input
 
-$Id: textwidgets.py 73027 2007-03-07 11:56:45Z zagy $
+$Id: textwidgets.py 73026 2007-03-07 10:49:25Z zagy $
 """
 __docformat__ = 'restructuredtext'
 
 from xml.sax import saxutils
 from zope.interface import implements
+from zope.datetime import parseDatetimetz
+from zope.datetime import DateTimeError
+from zope.i18n.format import DateTimeParseError
 
 from zope.app.form.interfaces import IInputWidget, ConversionError
 from zope.app.form.browser.interfaces import ITextBrowserWidget
 from zope.app.form.browser.widget import SimpleInputWidget, renderElement
 from zope.app.form.browser.widget import DisplayWidget
-from zope.app.datetimeutils import parseDatetimetz
-from zope.app.datetimeutils import DateTimeError
 
 from zope.app.i18n import ZopeMessageFactory as _
 
@@ -343,7 +344,7 @@ class TextAreaWidget(SimpleInputWidget):
             except ValueError, v:
                 raise ConversionError(_("Invalid unicode data"), v)
             else:
-                value = value.replace("\r\n", "\n")                
+                value = value.replace("\r\n", "\n")
         return value
 
     def _toFormValue(self, value):
@@ -352,7 +353,7 @@ class TextAreaWidget(SimpleInputWidget):
             value = value.replace("\n", "\r\n")
         else:
             value = u''
-            
+
         return value
 
     def __call__(self):
@@ -524,7 +525,6 @@ class FloatWidget(TextWidget):
             except ValueError, v:
                 raise ConversionError(_("Invalid floating point data"), v)
 
-
 class DatetimeWidget(TextWidget):
     """Datetime entry widget."""
 
@@ -535,11 +535,14 @@ class DatetimeWidget(TextWidget):
             return self.context.missing_value
         else:
             try:
+                # TODO: Currently datetimes return in local (server)
+                # time zone if no time zone information was given.
+                # Maybe offset-naive datetimes should be returned in
+                # this case? (DV)
                 return parseDatetimetz(input)
             except (DateTimeError, ValueError, IndexError), v:
                 raise ConversionError(_("Invalid datetime data"), v)
 
-
 class DateWidget(DatetimeWidget):
     """Date entry widget.
     """
@@ -550,6 +553,58 @@ class DateWidget(DatetimeWidget):
             v = v.date()
         return v
 
+class DateI18nWidget(TextWidget):
+    """I18n date entry widget.
+
+    The `displayStyle` attribute may be set to control the formatting of the
+    value.
+
+    `displayStyle` must be one of 'full', 'long', 'medium', 'short',
+    or None ('' is accepted an an alternative to None to support
+    provision of a value from ZCML).
+    """
+
+    _category = "date"
+
+    displayWidth = 20
+
+    displayStyle = None
+
+    def _toFieldValue(self, input):
+        if input == self._missing:
+            return self.context.missing_value
+        else:
+            try:
+                formatter = self.request.locale.dates.getFormatter(
+                    self._category, (self.displayStyle or None))
+                return formatter.parse(input)
+            except (DateTimeParseError, ValueError), v:
+                raise ConversionError(_("Invalid datetime data"),
+                    "%s (%r)" % (v, input))
+
+    def _toFormValue(self, value):
+        value = super(DateI18nWidget, self)._toFormValue(value)
+        if value:
+            formatter = self.request.locale.dates.getFormatter(
+                self._category, (self.displayStyle or None))
+            value = formatter.format(value)
+        return value
+
+class DatetimeI18nWidget(DateI18nWidget):
+    """I18n datetime entry widget.
+
+    The `displayStyle` attribute may be set to control the formatting of the
+    value.
+
+    `displayStyle` must be one of 'full', 'long', 'medium', 'short',
+    or None ('' is accepted an an alternative to None to support
+    provision of a value from ZCML).
+
+    NOTE: If you need timezone information you need to set `displayStyle`
+    to either 'long' or 'full' since other display styles just ignore it.
+    """
+
+    _category = "dateTime"
 
 class DateDisplayWidget(DisplayWidget):
     """Date display widget.
@@ -560,7 +615,6 @@ class DateDisplayWidget(DisplayWidget):
     `displayStyle` must be one of 'full', 'long', 'medium', 'short',
     or None ('' is accepted an an alternative to None to support
     provision of a value from ZCML).
-
     """
 
     cssClass = "date"
@@ -591,7 +645,6 @@ class DatetimeDisplayWidget(DateDisplayWidget):
     `displayStyle` must be one of 'full', 'long', 'medium', 'short',
     or None ('' is accepted an an alternative to None to support
     provision of a value from ZCML).
-
     """
 
     cssClass = "dateTime"
