@@ -13,7 +13,7 @@
 ##############################################################################
 """Generic Widget Tests
 
-$Id: test_widget.py 71640 2006-12-21 00:07:40Z jacobholm $
+$Id: test_widget.py 71639 2006-12-20 23:55:22Z jacobholm $
 """
 from unittest import TestSuite, main, makeSuite
 from zope.testing.doctestunit import DocTestSuite
@@ -106,11 +106,11 @@ class TestInputWidget(object):
     All widgets rely on a field and a request:
 
         >>> from zope.schema import Field
-        >>> from zope.component.tests.request import Request
         >>> field = Field()
         >>> from zope.interface import Interface
         >>> class ITestRequest(Interface):
         ...     pass
+        >>> from zope.app.component.tests.views import Request
         >>> widget = TestInputWidget(field, Request(ITestRequest))
 
     The default implementation of hasValidInput relies on
