@@ -13,8 +13,10 @@
 ##############################################################################
 """Interfaces related to the local interface service.
 
-$Id: interfaces.py 25177 2004-06-02 13:17:31Z jim $
+$Id: interfaces.py 26891 2004-08-04 04:34:55Z pruggera $
 """
+__docformat__ = 'restructuredtext'
+
 from zope.interface import Interface
 
 class IInterfaceBasedRegistry(Interface):
@@ -23,5 +25,5 @@ class IInterfaceBasedRegistry(Interface):
     def getRegistrationsForInterface(iface):
         """Return registrations related to iface.
 
-        The Return value is iterable of IRegistration object.
+        The Return value is iterable of `IRegistration` object.
         """
