@@ -13,14 +13,15 @@
 ##############################################################################
 """Vocabulary that provides a list of all interfaces its context provides.
 
-$Id: vocabulary.py 29341 2005-02-28 13:42:02Z srichter $
+$Id: vocabulary.py 67630 2006-04-27 00:54:03Z jim $
 """
 __docformat__ = 'restructuredtext'
 
-from zope.interface import providedBy
+from zope.interface import classProvides, providedBy
 from zope.security.proxy import removeSecurityProxy
 from zope.schema.vocabulary import SimpleVocabulary, SimpleTerm 
-from zope.app.component.interface import interfaceToName
+from zope.schema.interfaces import IVocabularyFactory
+from zope.component.interface import interfaceToName
 
 
 class ObjectInterfacesVocabulary(SimpleVocabulary):
@@ -50,6 +51,7 @@ class ObjectInterfacesVocabulary(SimpleVocabulary):
      'zope.app.interface.vocabulary.I3',
      'zope.interface.Interface']
     """
+    classProvides(IVocabularyFactory)
 
     def __init__(self, context):
         # Remove the security proxy so the values from the vocabulary
