@@ -13,13 +13,13 @@
 ##############################################################################
 """URL quoting for ZPT
 
-$Id: urlquote.py 27274 2004-08-26 12:09:15Z BjornT $
+$Id: urlquote.py 67630 2006-04-27 00:54:03Z jim $
 """
 __docformat__ = 'restructuredtext'
 
 import urllib
 from zope.interface import implements
-from zope.app.traversing.interfaces import IPathAdapter
+from zope.traversing.interfaces import IPathAdapter
 
 class URLQuote(object):
     r"""An adapter for URL quoting.
