@@ -13,23 +13,22 @@
 ##############################################################################
 """Principal Annotation Tests
 
-$Id: tests.py 30167 2005-04-25 18:00:30Z poster $
+$Id: tests.py 69217 2006-07-20 03:56:26Z baijum $
 """
-import doctest
+from zope.testing import doctest
 from unittest import TestCase, TestLoader, TextTestRunner
 
 import zope.component
 from zope.interface import implements
-from zope.app import zapi
-from zope.app.annotation.interfaces import IAnnotations
+from zope.annotation.interfaces import IAnnotations
+from zope.security.interfaces import IPrincipal
+
 from zope.app.principalannotation import PrincipalAnnotationUtility
-from zope.app.security.interfaces import IPrincipal
 from zope.app.component.testing import PlacefulSetup
-from zope.app.testing import ztapi, setup
+from zope.app.testing import setup
 from zope.app.principalannotation.interfaces import IPrincipalAnnotationUtility
 
 class Principal(object):
-
     implements(IPrincipal)
 
     def __init__(self, id):
