@@ -24,29 +24,29 @@ from ZODB.DB import DB
 from ZODB.DemoStorage import DemoStorage
 import transaction
 
+import zope.component
 from zope.interface.verify import verifyClass
 from zope.interface import implements, classImplements, implementedBy
 from zope.i18n.interfaces import IUserPreferredCharsets
-from zope.component.exceptions import ComponentLookupError
+from zope.component.interfaces import ComponentLookupError
 from zope.publisher.base import TestPublication, TestRequest
 from zope.publisher.http import IHTTPRequest, HTTPCharsets
 from zope.publisher.interfaces import IRequest, IPublishTraverse
 from zope.security import simplepolicies
 from zope.security.management import setSecurityPolicy, queryInteraction
 from zope.security.management import endInteraction
+from zope.traversing.interfaces import IPhysicallyLocatable
+from zope.location.interfaces import ILocation
 
-from zope.app import zapi
 from zope.app.testing.placelesssetup import PlacelessSetup
 from zope.app.testing import setup, ztapi
 
 from zope.app.error.interfaces import IErrorReportingUtility
-from zope.app.location.interfaces import ILocation
-from zope.app.traversing.interfaces import IPhysicallyLocatable
 from zope.app.security.principalregistry import principalRegistry
 from zope.app.security.interfaces import IUnauthenticatedPrincipal, IPrincipal
 from zope.app.publication.zopepublication import ZopePublication
 from zope.app.folder import Folder, rootFolder
-from zope.app.location import Location
+from zope.location import Location
 from zope.app.security.interfaces import IAuthenticationUtility
 
 class Principal(object):
@@ -125,7 +125,7 @@ class BasePublicationTests(PlacelessSetup, unittest.TestCase):
         connection.close()
         self.app = app
 
-        from zope.app.traversing.namespace import view, resource, etc
+        from zope.traversing.namespace import view, resource, etc
         ztapi.provideNamespaceHandler('view', view)
         ztapi.provideNamespaceHandler('resource', resource)
         ztapi.provideNamespaceHandler('etc', etc)
@@ -155,18 +155,23 @@ class ZopePublicationErrorHandling(BasePublicationTests):
         try:
             raise ConflictError
         except:
-            pass
-        self.assertRaises(Retry, self.publication.handleException,
-            self.object, self.request, sys.exc_info(), retry_allowed=True)
+            self.assertRaises(Retry, self.publication.handleException,
+                self.object, self.request, sys.exc_info(), retry_allowed=True)
+
+        try:
+            raise Retry(sys.exc_info())
+        except:
+            self.assertRaises(Retry, self.publication.handleException,
+                self.object, self.request, sys.exc_info(), retry_allowed=True)
 
     def testRetryNotAllowed(self):
         from ZODB.POSException import ConflictError
+        from zope.publisher.interfaces import Retry
         try:
             raise ConflictError
         except:
-            pass
-        self.publication.handleException(
-            self.object, self.request, sys.exc_info(), retry_allowed=False)
+            self.publication.handleException(
+                self.object, self.request, sys.exc_info(), retry_allowed=False)
         value = ''.join(self.request.response._result).split()
         self.assertEqual(' '.join(value[:6]),
                          'Traceback (most recent call last): File')
@@ -174,6 +179,32 @@ class ZopePublicationErrorHandling(BasePublicationTests):
                          'in testRetryNotAllowed raise ConflictError'
                          ' ConflictError: database conflict error')
 
+        try:
+            raise Retry(sys.exc_info())
+        except:
+            self.publication.handleException(
+                self.object, self.request, sys.exc_info(), retry_allowed=False)
+        value = ''.join(self.request.response._result).split()
+        self.assertEqual(' '.join(value[:6]),
+                         'Traceback (most recent call last): File')
+        self.assertEqual(' '.join(value[-8:]),
+                         'in testRetryNotAllowed raise Retry(sys.exc_info())'
+                         ' Retry: database conflict error')
+
+        try:
+            raise Retry
+        except:
+            self.publication.handleException(
+                self.object, self.request, sys.exc_info(), retry_allowed=False)
+        value = ''.join(self.request.response._result).split()
+        self.assertEqual(' '.join(value[:6]),
+                         'Traceback (most recent call last): File')
+        self.assertEqual(' '.join(value[-6:]),
+                         'in testRetryNotAllowed raise Retry'
+                         ' Retry: None')
+
+
+
     def testViewOnException(self):
         from zope.interface import Interface
         class E1(Exception):
@@ -347,8 +378,7 @@ class ZopePublicationErrorHandling(BasePublicationTests):
 
     def testAbortTransactionWithErrorReportingUtility(self):
         # provide our fake error reporting utility
-        sm = zapi.getGlobalSiteManager()
-        sm.provideUtility(IErrorReportingUtility, ErrorReportingUtility())
+        zope.component.provideUtility(ErrorReportingUtility())
 
         class FooError(Exception):
             pass
@@ -366,7 +396,7 @@ class ZopePublicationErrorHandling(BasePublicationTests):
         self.assertEqual(last_txn_info, new_txn_info)
 
         # instead, we expect a message in our logging utility
-        error_log = zapi.getUtility(IErrorReportingUtility)
+        error_log = zope.component.getUtility(IErrorReportingUtility)
         self.assertEqual(len(error_log.exceptions), 1)
         error_info, request = error_log.exceptions[0]
         self.assertEqual(error_info[0], FooError)
@@ -436,10 +466,10 @@ class ZopePublicationTests(BasePublicationTests):
 
     def testTransactionAnnotation(self):
         from zope.interface import directlyProvides
-        from zope.app.location.traversing import LocationPhysicallyLocatable
-        from zope.app.location.interfaces import ILocation
-        from zope.app.traversing.interfaces import IPhysicallyLocatable
-        from zope.app.traversing.interfaces import IContainmentRoot
+        from zope.location.traversing import LocationPhysicallyLocatable
+        from zope.location.interfaces import ILocation
+        from zope.traversing.interfaces import IPhysicallyLocatable
+        from zope.traversing.interfaces import IContainmentRoot
         ztapi.provideAdapter(ILocation, IPhysicallyLocatable,
                              LocationPhysicallyLocatable)
 
