@@ -15,17 +15,17 @@
 
 This module defines the schemas for browser directives.
 
-$Id: metadirectives.py 38705 2005-09-30 17:05:19Z anguenot $
+$Id: metadirectives.py 69358 2006-08-05 17:54:32Z flox $
 """
 from zope.interface import Interface
 from zope.configuration.fields import GlobalObject, GlobalInterface
 from zope.configuration.fields import Tokens, Path, PythonIdentifier, MessageID
-from zope.schema import TextLine, Text, Id, Int
+from zope.schema import TextLine, Text, Id, Int, Bool
+from zope.security.zcml import Permission
 
 from zope.app.component.metadirectives import IBasicViewInformation
-from zope.app.component.fields import LayerField
+from zope.app.component.back35 import LayerField
 from zope.app.publisher.browser.fields import MenuField
-from zope.app.security.fields import Permission
 
 #
 # browser views
@@ -566,8 +566,19 @@ class IAddMenuItemDirective(IMenuItem):
 # misc. directives
 #
 
+# BBB 2006/02/18, to be removed after 12 months
 class ILayerDirective(Interface):
-    """Defines a browser layer
+    """ *BBB: DEPRECATED*
+
+    Creating layers via ZCML has been deprecated.  The
+    'browser:layer' directive will be removed in Zope 3.5.  Layers
+    are now interfaces extending
+    'zope.publisher.interfaces.browser.IBrowserRequest'.
+    They do not need further registration.
+
+    **Previous documentation**
+
+    Defines a browser layer
 
     You must either specify a `name` or an `interface`. If you specify the
     name, then a layer interface will be created for you based on the name and
@@ -599,8 +610,24 @@ class ILayerDirective(Interface):
         required=False
         )
 
+    bbb_aware = Bool(
+        title=u"BBB-aware",
+        description=u"Backward-compatability aware?",
+        required=False
+        )
+
+# BBB 2006/02/18, to be removed after 12 months
 class ISkinDirective(Interface):
-    """Defines a browser skin
+    """ *BBB: DEPRECATED*
+
+    Creating skins via ZCML has been deprecated.  The 'browser:skin'
+    directive will be removed in Zope 3.5.  Skins are now interfaces
+    extending 'zope.publisher.interfaces.browser.IBrowserRequest'.
+    They are registered using the 'interface' directive.
+
+    **Previous documentation**
+
+    Defines a browser skin
 
     If you do not specify an `interface`, then one will be automatically
     created for you based on the name using the layers as base interfaces.
@@ -626,7 +653,7 @@ class ISkinDirective(Interface):
         title=u"A list of layers",
         description=u"""
         This should be in order of lookup. Usually one of the layers
-        has the same name as the skin, and the last skin should be
+        has the same name as the skin, and the last layer should be
         'default', unless you want to completely override all views.
         """,
         required=False,
