@@ -13,21 +13,20 @@
 ##############################################################################
 """Support for tests that need a simple site to be provided.
 
-$Id: support.py 29570 2005-03-18 22:53:37Z rogerineichen $
+$Id: support.py 67630 2006-04-27 00:54:03Z jim $
 """
+import zope.component
 from zope.interface import implements
+from zope.traversing.interfaces import IContainmentRoot
 
-from zope.app import zapi
 from zope.app.component.hooks import setSite
 from zope.app.component.interfaces import ISite
-from zope.app.traversing.interfaces import IContainmentRoot
-
 
 class Site:
     implements(ISite, IContainmentRoot)
 
     def getSiteManager(self):
-        return zapi.getGlobalSiteManager()
+        return zope.component.getGlobalSiteManager()
 
 site = Site()
 
