@@ -13,17 +13,15 @@
 ##############################################################################
 """Security Tests
 
-$Id: __init__.py 27082 2004-08-12 20:03:58Z srichter $
+$Id: __init__.py 67630 2006-04-27 00:54:03Z jim $
 """
-from zope.app import zapi
-from zope.app.security.permission import Permission
-from zope.app.security.interfaces import IPermission
+import zope.component
+from zope.security.interfaces import IPermission
+from zope.security.permission import Permission
 
 def addCheckerPublic():
     """Add the CheckerPublic permission as 'zope.Public'"""
 
-    utilityService = zapi.getGlobalService(zapi.servicenames.Utilities)
-
     perm = Permission('zope.Public', 'Public',
             """Special permission used for resources that are always public
 
@@ -31,4 +29,5 @@ def addCheckerPublic():
             it allows security computation to be bypassed.
             """
             )
-    utilityService.provideUtility(IPermission, perm, perm.id)
+    gsm = zope.component.getGlobalSiteManager()
+    gsm.registerUtility(perm, IPermission, perm.id)
