@@ -12,25 +12,22 @@
 ##############################################################################
 """DT_SQLVar Tests
 
-$Id: test_sqlscript.py 29143 2005-02-14 22:43:16Z srichter $
+$Id: test_sqlscript.py 67630 2006-04-27 00:54:03Z jim $
 """
 import unittest
 from zope.interface import implements, classImplements
+from zope.component.testing import PlacelessSetup
+from zope.traversing.interfaces import IPhysicallyLocatable
+from zope.annotation.interfaces import IAnnotatable, IAnnotations
+from zope.annotation.interfaces import IAttributeAnnotatable
+from zope.annotation.attribute import AttributeAnnotations
+from zope.rdb.interfaces import IZopeDatabaseAdapter
+from zope.rdb.interfaces import IZopeConnection, IZopeCursor
 
 from zope.app.testing import ztapi
-from zope.app.rdb.interfaces import IZopeDatabaseAdapter
-from zope.app.rdb.interfaces import IZopeConnection
-from zope.app.rdb.interfaces import IZopeCursor
-from zope.app.testing.placelesssetup import PlacelessSetup
-
-from zope.app.annotation.interfaces import IAnnotatable, IAnnotations
-from zope.app.annotation.interfaces import IAttributeAnnotatable
-from zope.app.annotation.attribute import AttributeAnnotations
-
 from zope.app.cache.interfaces import ICacheable, ICache
 from zope.app.cache.annotationcacheable import AnnotationCacheable
 from zope.app.cache.caching import getCacheForObject
-from zope.app.traversing.interfaces import IPhysicallyLocatable
 
 from zope.app.sqlscript.sqlscript import SQLScript, Arguments
 from zope.app.sqlscript.interfaces import ISQLScript
