@@ -13,14 +13,15 @@
 ##############################################################################
 """Stateful cookie tree
 
-$Id: cookie.py 29648 2005-03-23 01:12:47Z rogerineichen $
+$Id: cookie.py 68951 2006-07-02 13:02:27Z philikon $
 """
-from zope.app import zapi
+import zope.traversing.api
+from zope.traversing.interfaces import IContainmentRoot
+from zope.component.interfaces import IComponentLookup
+
 from zope.app.container.interfaces import IContainer
 from zope.app.folder.interfaces import IFolder
 from zope.app.component.interfaces import ISite
-from zope.component.interfaces import ISiteManager
-from zope.app.traversing.interfaces import IContainmentRoot
 
 from zope.app.tree.filters import OnlyInterfacesFilter
 from zope.app.tree.browser import StatefulTreeView
@@ -54,7 +55,7 @@ class CookieTreeView(StatefulTreeView):
         node.
         """
         parent = self.context
-        for parent in zapi.getParents(self.context):
+        for parent in zope.traversing.api.getParents(self.context):
             if ISite.providedBy(parent):
                 break
         return self.folderTree(parent)
@@ -63,7 +64,7 @@ class CookieTreeView(StatefulTreeView):
         """Cookie tree with only folders and the root container as
         root node.
         """
-        root = zapi.getRoot(self.context)
+        root = zope.traversing.api.getRoot(self.context)
         return self.folderTree(root)
 
     def virtualHostTree(self):
@@ -74,5 +75,5 @@ class CookieTreeView(StatefulTreeView):
         if vh:
             return self.folderTree(vh)
         else:
-            root = zapi.getRoot(self.context)
+            root = zope.traversing.api.getRoot(self.context)
             return self.folderTree(root)
