@@ -12,22 +12,14 @@
 ##############################################################################
 """Simple Page support
 
-$Id: page.py 38778 2005-10-05 20:09:32Z benji_york $
+This module has been deprecated and will be gone in Zope 3.5.
+
+$Id: page.py 67630 2006-04-27 00:54:03Z jim $
 """
-
-from zope import interface
-from zope.publisher.interfaces import NotFound
-from zope.app.publisher.browser import BrowserView
-from zope.formlib.interfaces import IPage
-
-class Page(BrowserView):
-    """Simple page-support class
-    """
-
-    interface.implements(IPage)
-
-    def browserDefault(self, request):
-        return self, ()
-
-    def publishTraverse(self, request, name):
-        raise NotFound(self, name, request)
+# BBB 2006/04/19 -- to be removed after 12 months
+import zope.deferredimport
+zope.deferredimport.deprecated(
+    "It has moved to zope.publisher.browser.BrowserPage.  This reference "
+    "will be gone in Zope 3.5.",
+    Page = 'zope.publisher.browser:BrowserPage',
+    )
