@@ -13,7 +13,7 @@
 ##############################################################################
 """Interfaces related to Locales
 
-$Id: locales.py 30750 2005-06-10 23:57:01Z philikon $
+$Id: locales.py 41114 2006-01-03 19:38:42Z jim $
 """
 import re
 from zope.interface import Interface, Attribute
@@ -618,3 +618,22 @@ class IDictionaryInheritance(ILocaleInheritance):
         If an key is not found or is None, the next higher up Locale
         object is consulted.
         """
+
+class ICollator(Interface):
+    """Provide support for collating text strings
+
+    This interface will typically be provided by adapting a locale.
+    """
+
+    def key(text):
+        """Return a collation key for the given text.
+        """
+
+    def cmp(text1, text2):
+        """Compare two text strings.
+
+        The return value is negative if text1 < text2, 0 is they are
+        equal, and positive if text1 > text2.
+        """
+    
+    
