@@ -13,19 +13,20 @@
 ##############################################################################
 """This is an 'abstract' test for the ITranslationDomain interface.
 
-$Id: test_itranslationdomain.py 39062 2005-10-11 18:36:32Z philikon $
+$Id: test_itranslationdomain.py 68769 2006-06-20 10:19:22Z hdima $
 """
 import unittest
 from zope.interface.verify import verifyObject
 from zope.interface import implements
 
 import zope.component
-from zope.component.tests.placelesssetup import PlacelessSetup
+from zope.component.testing import PlacelessSetup
 
 from zope.i18n.negotiator import negotiator
 from zope.i18n.interfaces import INegotiator, IUserPreferredLanguages
 from zope.i18n.interfaces import ITranslationDomain
 
+
 class Environment(object):
 
     implements(IUserPreferredLanguages)
@@ -47,7 +48,7 @@ class TestITranslationDomain(PlacelessSetup):
         self._domain = self._getTranslationDomain()
 
         # Setup the negotiator utility
-        zope.component.provideUtility(negotiator, INegotiator)        
+        zope.component.provideUtility(negotiator, INegotiator)
 
     def testInterface(self):
         verifyObject(ITranslationDomain, self._domain)
@@ -86,6 +87,12 @@ class TestITranslationDomain(PlacelessSetup):
                      default='Glorp Smurf Hmpf'),
            'Glorp Smurf Hmpf')
 
+    def testUnicodeDefaultValue(self):
+        translate = self._domain.translate
+        translated = translate('no way', target_language='en')
+        self.assertEqual(translated, "no way")
+        self.assert_(type(translated) is unicode)
+
     def testNoTargetLanguage(self):
         translate = self._domain.translate
         eq = self.assertEqual
