@@ -13,7 +13,7 @@
 ##############################################################################
 """HTTP Publisher
 
-$Id: http.py 69340 2006-08-02 14:16:04Z mj $
+$Id: http.py 70592 2006-10-09 19:15:56Z mgedmin $
 """
 import re, time, random
 from cStringIO import StringIO
@@ -204,7 +204,11 @@ class HTTPInputStream(object):
         self.cacheStream.write(data)
         return data
 
-    def readline(self):
+    def readline(self, size=None):
+        # XXX We should pass the ``size`` argument to self.stream.readline
+        # but twisted.web2.wsgi.InputStream.readline() doesn't accept it.
+        # See http://www.zope.org/Collectors/Zope3-dev/535 and
+        #     http://twistedmatrix.com/trac/ticket/1451
         data = self.stream.readline()
         self.cacheStream.write(data)
         return data
