@@ -15,7 +15,7 @@
 
 Note that this is used by tests in other packages.
 
-$Id: httprequest.py 27082 2004-08-12 20:03:58Z srichter $
+$Id: httprequest.py 38357 2005-09-07 20:14:34Z srichter $
 """
 
 from StringIO import StringIO
@@ -31,16 +31,13 @@ _testEnv =  {
 
 class TestRequest(HTTPRequest):
 
-    def __init__(self, body_instream=None, outstream=None, environ=None, **kw):
+    def __init__(self, body_instream=None, environ=None, **kw):
         if body_instream is None:
             body_instream = StringIO('')
-        if outstream is None:
-            outstream = StringIO()
-
 
         env = {}
         env.update(_testEnv)
         if environ: env.update(environ)
         env.update(kw)
 
-        super(TestRequest, self).__init__(body_instream, outstream, env)
+        super(TestRequest, self).__init__(body_instream, env)
